#
# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation version
# 2.1 of the License.
#  
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#  
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# END COPYRIGHT BLOCK
#
#
# This makefile is used to grab all the components needed to build.
#
#
#

COMMA :=,
NULLSTRING :=
SPACE :=$(NULLSTRING) # the space is between the ) and the #
	   
ifeq ($(ARCH), WINNT)
EXE_SUFFIX = .exe
else # unix - windows has no lib name prefix, except for nspr
LIB_PREFIX = lib
endif

include $(ABS_ROOT)/nsperl.mk

ifeq ($(INTERNAL_BUILD), 1)
include $(ABS_ROOT)/internal_buildpaths.mk
else
include $(ABS_ROOT)/buildpaths.mk
endif

all:

#Temporary integration area components
INT_LDAPSDK = 0

# NSPR20 Library
NSPR_LIBNAMES = plc4 plds4
ifeq ($(ARCH), SOLARIS)
  ifeq ($(NSPR_RELDATE), v4.2.2)
# no need after v4.4.1
NSPR_LIBNAMES += ultrasparc4 
# just need ultrasparc for now
LIBS_TO_PKG += $(addsuffix .$(DLL_SUFFIX),$(addprefix $(NSPR_LIBPATH)/lib,ultrasparc4))
  endif
endif
NSPR_LIBNAMES += nspr4
ifdef NSPR_INCLUDE_DIR
 NSPR_INCDIR = $(NSPR_INCLUDE_DIR)
else
 ifdef NSPR_SOURCE_ROOT
  NSPR_INCDIR = $(NSPR_SOURCE_ROOT)/dist/$(MOZ_OBJDIR_NAME)/include
 else
  NSPR_INCDIR = $(NSPR_BUILD_DIR)/include
 endif
endif
ifdef NSPR_LIB_DIR
 NSPR_LIBPATH = $(NSPR_LIB_DIR)
else
 ifdef NSPR_SOURCE_ROOT
  NSPR_LIBPATH = $(NSPR_SOURCE_ROOT)/dist/$(MOZ_OBJDIR_NAME)/lib
 else
  NSPR_LIBPATH = $(NSPR_BUILD_DIR)/lib
 endif
endif
NSPR_INCLUDE = -I$(NSPR_INCDIR)
NSPR_LIBS_TO_PKG = $(addsuffix .$(DLL_SUFFIX),$(addprefix $(NSPR_LIBPATH)/lib,$(NSPR_LIBNAMES)))

LIBS_TO_PKG += $(NSPR_LIBS_TO_PKG)
LIBS_TO_PKG_SHARED += $(NSPR_LIBS_TO_PKG) # needed for cmd line tools
ifeq ($(USE_SETUPSDK), 1)
  PACKAGE_SETUP_LIBS += $(NSPR_LIBS_TO_PKG)
endif
ifeq ($(USE_DSGW), 1)
  LIBS_TO_PKG_CLIENTS += $(NSPR_LIBS_TO_PKG) # for dsgw
endif

ifeq ($(ARCH), WINNT)
  NSPRDLL_NAME = $(addprefix lib, $(NSPR_LIBNAMES))
  NSPROBJNAME = $(addsuffix .lib, $(NSPRDLL_NAME))
  NSPRLINK = /LIBPATH:$(NSPR_LIBPATH) $(NSPROBJNAME)
  LIBNSPRDLL_NAMES = $(addsuffix .dll, $(addprefix $(NSPR_LIBPATH)/, \
	$(addprefix lib, $(NSPR_LIBNAMES))))
else
  NSPR_SOLIBS = $(addsuffix .$(DLL_SUFFIX),  $(addprefix $(LIB_PREFIX), $(NSPR_LIBNAMES)))
  NSPROBJNAME = $(addsuffix .a, $(addprefix $(LIB_PREFIX), $(NSPR_LIBNAMES))
  LIBNSPR = $(addprefix $(NSPR_LIBPATH)/, $(NSPR_SOLIBS))
  ifdef NSPR_LIB_DIR # we are using the system NSPR (shared lib)
    NSPRLINK = -L$(NSPR_LIBPATH) $(addprefix -l, $(NSPR_LIBNAMES)) 
  else
    NSPRLINK = 
  endif
endif

### DBM #############################

ifdef DBM_SOURCE_ROOT
  DBM_LIBPATH = $(DBM_SOURCE_ROOT)/dist/$(MOZ_OBJDIR_NAME)/lib
  DBM_INCDIR = $(DBM_SOURCE_ROOT)/dist/public/dbm
else
  DBM_LIBPATH = $(DBM_BUILD_DIR)/lib
  DBM_INCDIR = $(DBM_BUILD_DIR)/include
endif
DBM_INCLUDE = -I$(DBM_INCDIR)
DBM_LIBNAMES = dbm

ifeq ($(ARCH), WINNT)
  DBMOBJNAME = $(addsuffix .lib, $(DBM_LIBNAMES))
  LIBDBM = $(addprefix $(DBM_LIBPATH)/, $(DBMOBJNAME))
  DBMLINK = /LIBPATH:$(DBM_LIBPATH) $(DBMOBJNAME)
else
  DBM_SOLIBS = $(addsuffix .$(DLL_SUFFIX),  $(addprefix $(LIB_PREFIX), $(DBM_LIBNAMES)))
  DBMROBJNAME = $(addsuffix .a, $(addprefix $(LIB_PREFIX), $(DBM_LIBNAMES)))
  LIBDBM = $(addprefix $(DBM_LIBPATH)/, $(DBMROBJNAME))
  DBMLINK = -L$(DBM_LIBPATH) $(addprefix -l, $(DBM_LIBNAMES))
endif

### DBM END #############################

### SECURITY #############################
ifdef NSS_INCLUDE_DIR
 SECURITY_INCDIR = $(NSS_INCLUDE_DIR)
else
 ifdef SECURITY_SOURCE_ROOT
  SECURITY_INCDIR = $(SECURITY_SOURCE_ROOT)/dist/public/nss
 else
  SECURITY_INCDIR = $(SECURITY_BUILD_DIR)/include
 endif
endif

ifdef NSS_LIB_DIR
 SECURITY_LIBPATH = $(NSS_LIB_DIR)
else
 ifdef SECURITY_SOURCE_ROOT
  SECURITY_LIBPATH = $(SECURITY_SOURCE_ROOT)/dist/$(MOZ_OBJDIR_NAME)/lib
 else
  SECURITY_LIBPATH = $(SECURITY_BUILD_DIR)/lib
 endif
endif
SECURITY_INCLUDE = -I$(SECURITY_INCDIR)
# add crlutil and ocspclnt when we support CRL and OCSP cert checking in DS
SECURITY_LIBNAMES = ssl3 nss3 softokn3
# these libs have a corresponding .chk file
SECURITY_NEED_CHK = softokn3

SECURITY_LIBNAMES.pkg = $(SECURITY_LIBNAMES) smime3

# these are only needed on 32 bit Solaris and HP-UX
ifneq ($(USE_64), 1)
ifeq ($(ARCH), SOLARIS)
SECURITY_LIBNAMES.pkg += freebl_hybrid_3 freebl_pure32_3
# these libs have a corresponding .chk file
SECURITY_NEED_CHK += freebl_hybrid_3 freebl_pure32_3
endif
ifeq ($(ARCH), HPUX)
SECURITY_LIBNAMES.pkg += freebl_hybrid_3 freebl_pure32_3
# these libs have a corresponding .chk file
SECURITY_NEED_CHK += freebl_hybrid_3 freebl_pure32_3
endif
endif # USE_64

SECURITY_LIBS_TO_PKG = $(addsuffix .$(DLL_SUFFIX),$(addprefix $(SECURITY_LIBPATH)/$(LIB_PREFIX),$(SECURITY_LIBNAMES.pkg)))
SECURITY_LIBS_TO_PKG += $(addsuffix .chk,$(addprefix $(SECURITY_LIBPATH)/$(LIB_PREFIX),$(SECURITY_NEED_CHK)))
LIBS_TO_PKG += $(SECURITY_LIBS_TO_PKG)
LIBS_TO_PKG_SHARED += $(SECURITY_LIBS_TO_PKG) # for cmd line tools
ifeq ($(USE_SETUPSDK), 1)
  PACKAGE_SETUP_LIBS += $(SECURITY_LIBS_TO_PKG)
endif
ifeq ($(USE_DSGW), 1)
  LIBS_TO_PKG_CLIENTS += $(SECURITY_LIBS_TO_PKG) # for dsgw
endif

ifeq ($(ARCH), WINNT)
  SECURITYOBJNAME = $(addsuffix .$(LIB_SUFFIX), $(SECURITY_LIBNAMES))
  LIBSECURITY = $(addprefix $(SECURITY_LIBPATH)/, $(SECURITYOBJNAME))
  SECURITYLINK = /LIBPATH:$(SECURITY_LIBPATH) $(SECURITYOBJNAME)
else
  SECURITYOBJNAME = $(addsuffix .$(DLL_SUFFIX), $(addprefix $(LIB_PREFIX), $(SECURITY_LIBNAMES)))
  LIBSECURITY = $(addprefix $(SECURITY_LIBPATH)/, $(SECURITYOBJNAME))
  ifdef NSPR_LIB_DIR # we are using the system NSPR (shared lib)
    SECURITYLINK = -L$(SECURITY_LIBPATH) $(addprefix -l, $(SECURITY_LIBNAMES))
  else
    SECURITYLINK =
  endif
endif

# we need to package the root cert file in the alias directory
PACKAGE_SRC_DEST += $(SECURITY_LIBPATH)/$(LIB_PREFIX)nssckbi.$(DLL_SUFFIX) alias

### SECURITY END #############################

### SVRCORE #############################
ifdef SVRCORE_SOURCE_ROOT
  SVRCORE_LIBPATH = $(SVRCORE_SOURCE_ROOT)/dist/$(MOZ_OBJDIR_NAME)/lib
  SVRCORE_INCDIR = $(SVRCORE_SOURCE_ROOT)/dist/public/svrcore
else
  SVRCORE_LIBPATH = $(SVRCORE_BUILD_DIR)/lib
  SVRCORE_INCDIR = $(SVRCORE_BUILD_DIR)/include
endif
SVRCORE_INCLUDE = -I$(SVRCORE_INCDIR)
SVRCORE_LIBNAMES = svrcore

ifeq ($(ARCH), WINNT)
  SVRCOREOBJNAME = $(addsuffix .lib, $(SVRCORE_LIBNAMES))
  LIBSVRCORE = $(addprefix $(SVRCORE_LIBPATH)/, $(SVRCOREOBJNAME))
  SVRCORELINK = /LIBPATH:$(SVRCORE_LIBPATH) $(SVRCOREOBJNAME)
else
  SVRCOREOBJNAME = $(addsuffix .a, $(addprefix $(LIB_PREFIX), $(SVRCORE_LIBNAMES)))
  LIBSVRCORE = $(addprefix $(SVRCORE_LIBPATH)/, $(SVRCOREOBJNAME))
  SVRCORELINK = -L$(SVRCORE_LIBPATH) $(addprefix -l, $(SVRCORE_LIBNAMES))
endif

### SVRCORE END #############################

####################################################
# LDAP SDK
###################################################

ifdef LDAPCSDK_INCLUDE_DIR
 LDAPSDK_INCDIR = $(LDAPCSDK_INCLUDE_DIR)
else
 ifdef LDAPSDK_SOURCE_ROOT
  LDAPSDK_INCDIR = $(LDAPSDK_SOURCE_ROOT)/dist/public/ldap
 else
  LDAPSDK_INCDIR = $(LDAP_ROOT)/include
 endif
endif

ifdef LDAPCSDK_LIB_DIR
 LDAPSDK_LIBPATH = $(LDAPCSDK_LIB_DIR)
else
 ifdef LDAPSDK_SOURCE_ROOT
  LDAPSDK_LIBPATH = $(LDAPSDK_SOURCE_ROOT)/dist/lib
 else
  LDAPSDK_LIBPATH = $(LDAP_ROOT)/lib
 endif
endif

LDAPSDK_INCLUDE = -I$(LDAPSDK_INCDIR)

# package the command line programs
LDAPSDK_TOOLS = $(wildcard $(LDAPSDK_BINPATH)/ldap*$(EXE_SUFFIX))
BINS_TO_PKG_SHARED += $(LDAPSDK_TOOLS)
# package the include files - needed for the plugin API
LDAPSDK_INCLUDE_FILES = $(wildcard $(LDAPSDK_INCDIR)/*.h)
PACKAGE_SRC_DEST += $(subst $(SPACE),$(SPACE)plugins/slapd/slapi/include$(SPACE),$(LDAPSDK_INCLUDE_FILES))
PACKAGE_SRC_DEST += plugins/slapd/slapi/include

LDAP_LIBPATH = $(LDAPSDK_LIBPATH)
LDAP_INCLUDE = $(LDAPSDK_INCDIR)

ifeq ($(ARCH), WINNT)
  LDAP_LIBNAMES = ldapssl32v$(LDAP_SUF) ldap32v$(LDAP_SUF) ldappr32v$(LDAP_SUF)
  LDAPDLL_NAME = $(addprefix ns, $(LDAP_LIBNAMES))
  LDAPOBJNAME = $(addsuffix .lib, $(LDAPDLL_NAME))
  LDAPLINK = $(addprefix $(LDAP_LIBPATH)/, $(LDAPOBJNAME))
else
  LDAP_SOLIB_NAMES = ssldap$(LDAP_SUF)$(LDAP_DLL_PRESUF) ldap$(LDAP_SUF)$(LDAP_DLL_PRESUF) prldap$(LDAP_SUF)$(LDAP_DLL_PRESUF)
  LDAP_LIBNAMES = ldap$(LDAP_SUF)$(LDAP_DLL_PRESUF) lber$(LDAP_SUF)$(LDAP_DLL_PRESUF)
  LDAP_SOLIBS = $(addsuffix .$(LDAP_DLL_SUFFIX), $(addprefix lib, $(LDAP_SOLIB_NAMES)))
  LDAPOBJNAME = $(addsuffix .a, $(addprefix lib, $(LDAP_LIBNAMES)))
  ifdef LDAPCSDK_LIB_DIR # we are using the system LDAPCSDK.
    LDAPLINK = -L$(LDAP_LIBPATH) $(addprefix -l, $(LDAP_SOLIB_NAMES))
  else
    LDAPLINK = $(addprefix $(LDAP_LIBPATH)/, $(LDAPOBJNAME))
  endif
endif

### SASL package ##########################################
### The LDAP C SDK is now compiled with SASL support - this means
### that we will get undefined symbols when we link against the
### static LDAP libraries - we have to link with SASL to 
### resolve those symbols
ifeq ($(ARCH), Linux)
  ifeq ($(BUILD_ARCH), RHEL3)
    SASL_LIBPATH = /usr/kerberos/lib
    SASL_LINK = -L$(SASL_LIBPATH)
  else
# use default linker path
    SASL_LINK = 
  endif
  SASL_INCDIR = /usr/include/sasl
else
  ifdef SASL_SOURCE_ROOT
    SASL_LIBPATH = $(SASL_SOURCE_ROOT)/lib
    SASL_INCDIR = $(SASL_SOURCE_ROOT)/include/sasl
  else
    SASL_LIBPATH = $(SASL_BUILD_DIR)/lib
    SASL_INCDIR = $(SASL_BUILD_DIR)/include/sasl
  endif
  SASL_LINK = -L$(SASL_LIBPATH)
endif
SASL_INCLUDE = $(SASL_INCDIR)

ifeq ($(ARCH), WINNT)
  SASL_LIB_ROOT_NAME = sasl
  SASL_LINK = /LIBPATH:$(SASL_LIBPATH) lib$(SASL_LIB_ROOT_NAME).lib
  SASL_LIBS = lib$(SASL_LIB_ROOT_NAME).lib,lib$(SASL_LIB_ROOT_NAME).dll,saslDIGESTMD5.dll
else
  # for cyrus it's sasl2
  SASL_LIB_ROOT_NAME = sasl2
  SASL_LIBS = lib$(SASL_LIB_ROOT_NAME).a
  ifeq ($(ARCH), Linux)
# I don't think we need this anymore
    GSSAPI_LIBS=
#    GSSAPI_LIBS=-lgssapi_krb5
  endif
  ifeq ($(ARCH), SOLARIS)
    GSSAPI_LIBS=-lgss
  endif
  ifeq ($(ARCH), HPUX)
      GSSAPI_LIBS=-lgss
      ifeq ($(USE_64),1)
        GSSAPI_LIBS=-lgss
      endif
  endif

  SASL_LINK += -l$(SASL_LIB_ROOT_NAME) $(GSSAPI_LIBS)
endif

#################################################################
# Infozip (for Windows Packaging)
#################################################################
ifneq ($(INTERNAL_BUILD), 1)
infozip:
ifeq ($(ARCH), WINNT)
	-mkdir -p $(ABS_ROOT_PARENT)/components
	cp $(INFOZIP_ROOT)/zip.exe $(INFOZIP_ROOT)/unzip.exe $(ABS_ROOT_PARENT)/components
endif
endif

ifeq ($(INTERNAL_BUILD), 1)
include $(ABS_ROOT)/internal_comp_deps.mk
endif
