/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
*
**
** NAME
**   ldapu.h
**
** DESCRIPTION
**   Various LDAP util functions used during installation.
** NOTES
**
** HISTORY
** $Log: ldapu.h,v $
** Revision 1.3  2006/03/30 00:38:28  rmeggins
** Add using namespace std; after the include <iostream>
** Remove #include <iostream.h> in files that also include nsdefs.h
** Fix a potential buffer non-termination
**
** Revision 1.2  2006/03/27 21:18:34  nhosoi
** [186642] Directory Server Makefile updates for Internal build
** Changed the createSIE so that caller (Admin Server and Directory Server) passes
** the brand to SetupUtil.
**
** Revision 1.1.1.1  2005/07/29 22:16:28  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:26  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/07/27 01:04:49  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:12  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.4  2002/02/07 18:55:00  djani
** Another prototype of LdapError:retrive is added which accepts 'int sizelimit' as a parameter
**
** Revision 1.1.1.3  2001/11/16 00:18:56  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.2  2001/11/06 01:51:26  ahakim
** XXX AOL/NSCP Copyright XXX
**
** Revision 1.1.1.1  1999/03/04 23:26:55  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.41.4.13  1999/01/08 22:10:14  cxue
** isStandardScheam api
**
** Revision 1.1.2.41.4.12  1999/01/07 02:41:00  cxue
** change BIG_BUF to 2K. doUpdate argument to schema entry API
**
** Revision 1.1.2.41.4.11  1999/01/06 01:00:21  cxue
** syntaxToOID for schema class
**
** Revision 1.1.2.41.4.10  1998/11/20 00:53:15  pvo
** 333357 - need to close ldif file upon destruction of LdifEntry
**
** Revision 1.1.2.41.4.9  1998/11/13 23:35:13  gbeasley
** unified api
**
** Revision 1.1.2.41.4.8  1998/10/28 19:08:33  pvo
** 310620 - update build number upon re-install
**
** Revision 1.1.2.41.4.7  1998/10/21 21:52:15  adam
** Add deleteAttribute member function.
**
** Revision 1.1.2.41.4.6  1998/10/15 22:20:58  cxue
** add setupdefs.h
**
** Revision 1.1.2.41.4.5  1998/10/08 23:07:15  adam
** Add member function "change_ldap" to LdapEntry class.
**
** Revision 1.1.2.41.4.4  1998/10/05 23:51:01  cxue
** not return ldap handle from Ldap class. Instead using modify for general ldap modification purpose
**
** Revision 1.1.2.41.4.3  1998/09/23 22:14:53  pvo
** Support for LDAP referral
**
** Revision 1.1.2.41.4.2  1998/09/21 21:34:16  cxue
** comments
**
** Revision 1.1.2.41.4.1  1998/09/19 01:41:09  cxue
** class LdapSchema
**
** Revision 1.1.2.41  1998/08/19 21:35:00  gbeasley
** checkadmindomain
**
** Revision 1.1.2.40  1998/08/18 03:49:26  pvo
** 314468 - Support new presentation of Config Dir LDAP URL
**
** Revision 1.1.2.39  1998/08/12 23:44:47  pvo
** Add enumerations for LdapEntry attributes
**
** Revision 1.1.2.38  1998/06/25 16:56:26  gbeasley
** no init of parameters for guessadmindomain
**
** Revision 1.1.2.37  1998/06/25 16:12:07  gbeasley
** fix NT build break for guessAdminDomainName
**
** Revision 1.1.2.36  1998/06/24 17:40:14  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.35  1998/05/13 04:39:22  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.34  1998/05/08 02:17:45  pvo
** Support relocation of MC Admin.
**
** Revision 1.1.2.33  1998/05/07 00:56:42  pvo
** 112552 - New ACI
**
** Revision 1.1.2.32  1998/04/20 20:51:30  pvo
** 112701 - problem reconfiguring SIE
**
** Revision 1.1.2.31  1998/03/28 03:27:43  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.30  1998/03/13 21:24:42  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.29  1998/03/11 23:17:19  pvo
** Do not createSIE if it already exists.
**
** Revision 1.1.2.28  1998/03/04 02:40:12  pvo
** Declare get_gmtime() so C can call.
**
** Revision 1.1.2.27  1998/03/04 02:28:44  pvo
** Added support for ldif line continuation and common time format.
**
** Revision 1.1.2.26  1998/02/26 01:40:22  gbeasley
** more sie integration
**
** Revision 1.1.2.25  1998/02/26 00:13:39  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.24  1998/02/24 00:40:22  pvo
** Change to support change from globals.{h,cpp} to global.{h, cpp}
**
** Revision 1.1.2.23  1998/02/24 00:21:13  pvo
** Support topology changes.
**
** Revision 1.1.2.22  1998/02/08 23:12:57  pvo
** 105971 - More delegation support.
**
** Revision 1.1.2.21  1998/01/31 02:06:39  pvo
** 105218 - Create SIE should not suffix hostname to instance name
**
** Revision 1.1.2.20  1997/11/13 19:28:15  pvo
** More new stuffs.
**
** Revision 1.1.2.19  1997/11/09 19:25:43  pvo
** Added freeAttributes
**
** Revision 1.1.2.18  1997/11/07 22:42:45  pvo
** New code to support uninstallation.
**
** Revision 1.1.2.17  1997/10/25 00:35:10  pvo
** Added ldap entry and SIE removal code.
**
** Revision 1.1.2.16  1997/10/24 03:39:24  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.15  1997/10/23 03:23:25  pvo
** More change to support other platforms.
**
** Revision 1.1.2.14  1997/10/22 02:40:41  pvo
** Too many changes to list out here.
**
** Revision 1.1.2.13  1997/10/16 04:24:05  pvo
** Quite a lot of code changes for support updating and ACIs.
**
** Revision 1.1.2.12  1997/10/14 05:01:48  pvo
** More code.
**
** Revision 1.1.2.11  1997/10/10 22:50:54  pvo
** Writing ACI to netscapeRoot.
**
** Revision 1.1.2.10  1997/10/10 00:37:31  pvo
** Resolved conflicts in name space with zip library.
**
** Revision 1.1.2.9  1997/10/06 20:37:37  pvo
** Restructured dependencies.
**
** Revision 1.1.2.8  1997/10/04 01:49:20  pvo
** Added ldapEntry update code.
**
** Revision 1.1.2.7  1997/10/02 00:56:04  pvo
** Fixed compilation bugs.
**
** Revision 1.1.2.6  1997/09/30 18:40:35  pvo
** More documentation.
**
** Revision 1.1.2.5  1997/09/27 02:46:39  pvo
** Checked in.
**
**********************************************************************
*/
#ifndef LDAP_U_H
#define LDAP_U_H

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

#include "ldap.h"

#ifdef __cplusplus
}
#endif

#include "setupdefs.h"
#include "global.h"

typedef int LdapErrorCode;

/*
 * Errors code used by ldapu. Actually, they are an extension of the current
 * LDAP error codes.
 */

#define OKAY             0x00

/*
 * Our internally defined errors start here
 * LDAP errors ends at 0x6x something.
*/
#define INVALID_INPUT  0x90
#define INVALID_URL    0x91
#define INVALID_AUTH   0x92 /* Not authorized */
#define INVALID_USER   0x93 /* User does not exists or not accessible */
#define INVALID_OBJECT 0x94 /* The entry is empty */
#define INVALID_ROOT   0x95 /* NetscapeRoot does not exist */
#define INVALID_DN     0x96 /* DN does not exists */
#define CONN_FAILED    0x97 /* Failed connection to LDAP server */
#define ALREADY_EXISTS 0x98 /* Object already exists */
#define NOT_FOUND      0x99 /* Entry not found */
#define MISSING_ATTR   0x9A /* Missing mandator attributes */
#define INSTALL_DN_ERR 0x9B /* InstallDN is missing */

#define UNKNOWN_ERR    0x9F /* Some unknown LDAP error */


#ifndef __cplusplus

typedef void Ldap;
typedef void LdapEntry;

NSAPI_EXTERN void get_gmtime(char *, int);

#else

#include "nsdefs.h"

class Ldap;
class LdapEntry;

#endif

/*
 * I am required to expose these API in C, so be it
 * Following is some things to get started.  Read the C++ section for
 * more detailed information. Sorry, you get what you want.
 *
 * Working with ldapu routines:
 *
 *   First, you have to create an Ldap object, passing in an LDAP URL,
 *   a UID and password of the LDAP user who is supposed to have
 *   access to LDAP entries you want to manipulate, and the optional
 *   Bind DN and Bind password to bootstrap the initial search to
 *   authenticate the LDAP user. Specify NULLs if you don't have the
 *   Bind DN/Password.
 *
 *   To work with a generic LDAP entry, create an LdapEntry object
 *   using the previously created Ldap object. Use the retrieveEntry
 *   and retrieveEntries functions to retrieve one or more LDAP entries.
 *   retrieveEntries() right now just returns the first entry.
 *   Use getAttributes() to retrieve the values of the entry's attributes.
 *   To insert an entry, you must first set the required attributes
 *   using setAttribute() or setAttributes() to set single/multi-value
 *   attributes respectively. To determine if any entry exists without
 *   changing the internal of the LdapEntry, use the exists() function.
 *   After an entry is retrieved or inserted, use entryDN() to get the
 *   DN of the entry.
 *
 *   Note that the Ldap object must exists and be valid for the duration
 *   of the LdapEntry object that uses it. Call destroyLdapEntry() and
 *   destroyLdap() to delete the objects.
 *
 */

#ifdef __cplusplus
extern "C" {
#endif

/*
****************************************************************
** Ldap Object
*****************************************************************
*/
NSAPI_EXTERN LdapErrorCode createLdap(Ldap ** ldap,const char *ldapURL,const char *ldapUser,const char *ldapPwd,const char *binddn,const char *bindpwd);
NSAPI_EXTERN void          destroyLdap (Ldap *);
NSAPI_EXTERN const char    *userDN (const Ldap *);
NSAPI_EXTERN const char    *ldapServerVersion(const Ldap *);
NSAPI_EXTERN LdapEntry     *createLdapEntry(const Ldap *);
NSAPI_EXTERN void          destroyLdapEntry(LdapEntry *);
NSAPI_EXTERN Bool          entryExists(LdapEntry *, const char *entryDN);
NSAPI_EXTERN LdapErrorCode    retrieveLdapEntry(LdapEntry *, const char *entryDN);
NSAPI_EXTERN LdapErrorCode    retrieveLdapEntries(LdapEntry *, const char *criteria, int scope,const char *baseDN);
NSAPI_EXTERN LdapErrorCode    nextLdapEntry(LdapEntry *);

/*
 * Get all the attribute names in this entry
 */
NSAPI_EXTERN char **getLdapAttributeNames(const LdapEntry *);
NSAPI_EXTERN void freeLdapAttributeNames(LdapEntry *, char **attrNames);

/*
 * Giving the name of an attribute, return the NULL terminated list of
 * associated attribute value(s). Just use freeAttributes() to free the
 * returned list after use.
 */
NSAPI_EXTERN char **getLdapAttributes(const LdapEntry *, const char *name);
NSAPI_EXTERN void freeLdapAttributes(LdapEntry *, char **attributes);
NSAPI_EXTERN const char       *entryDN(const LdapEntry *);

/*
 * Setting attributes. For single-value attribute, use setAttribute. For
 * attributes that are typically multivalue (as you know), especially
 * objectclass, use setAttributes(). Don't mix setAttribute and setAttributes
 * on the same attribute - I am not that smart.
 */
NSAPI_EXTERN void setLdapAttribute(LdapEntry *, const char *name, const char *value);
NSAPI_EXTERN void addLdapAttribute(LdapEntry *, const char *name, const char *value);
NSAPI_EXTERN void addLdapAttributes(LdapEntry *, const char *name,const char **values);

NSAPI_EXTERN LdapErrorCode renameLdapEntry(LdapEntry *, const char *entryDN, const char *rdn);

NSAPI_EXTERN LdapErrorCode insertLdapEntry(LdapEntry *, const char *entryDN);

NSAPI_EXTERN LdapErrorCode updateLdapEntry(LdapEntry *, const char *entryDN);

NSAPI_EXTERN LdapErrorCode replaceLdapEntry(LdapEntry *, const char *entryDN);

/*
 * Drop an entry
 */
NSAPI_EXTERN LdapErrorCode dropLdapEntry(Ldap *, const char *entryDN);
NSAPI_EXTERN LdapErrorCode dropAllLdapEntries(Ldap *, const char *entryDN);

/*
****************************************************************
** SIE/ISIE creation routines
*****************************************************************
*/

/*
 * setupInstallFramework()
 */

NSAPI_EXTERN LdapErrorCode setupInstallFramework(const Ldap *, const char *adminDomain, const char *adminBrand);

/*
 * createSIE()
 *    Register a Server Instance Entry specified by sieEntry,
 *    associated with the NetscapeApplication specified by isieDN.
 *    The entry is to be inserted at subTreeDN.
 *
 *    This routine will perform the following:
 *       - Create an installed software instance under NetscapeRoot if
 *         it has not been created, otherwise, update the entry with
 *         appropriate information
 *
 *       - Generate the appropriate SIE entry's common
 *         name to guarantee the uniqueness of the SIE. A seeAlso
 *         link will be created from the SIE to the ISIE and vice versa.
 *
 *   Parameters:
 *     - sieEntry: LDAP entry for the SIE
 *     - appEntry: LDAP entry for the ISIE
 *     - hostName: name of this machine, does not need to be FQDN, BUT
 *        appEntry's serverHostName attribute must contain the FQDN
 *     - installedLocation: the location where this product is installed
 *     - subTreeDN: the DN of the subtree where the SIE is to be installed.
 *          The default is ou=Netscape Suitespot,<base-suffix>
 *   Absolute required attributes are:
 *    * appEntry
 *        - objectclass = netscapeApplication
 *        - name = product name, e.g. Netscape Enterprise Server
 *        - version = product version , e.g. 4.0a0
 *        - other attributes from the Product Information File.
 *    * sieEntry
 *        - objectclass (netscapeXXXServer, e.g. netscapeMailServer
 *               which must be inherited from NetscapeServer
 *        - sie's password
 *        - serverID
 *
 * Hint:
 *   use entryDN() to get the full DN of the ISIE and SIE
 *   and save them for later use.
 */
NSAPI_EXTERN LdapErrorCode createSIE(
         LdapEntry *sieEntry,
         LdapEntry *appEntry,
         const char *hostName,
         const char *installedLocation,
         const char *adminDomain,
         const char *adminBrand);

/*
 * removeSIE
 *   Remove the server instance, and the pointer to it from ISIE.
 *   Optionally, removeISIE can be set to True to remove the ISIE also.
 *   In general, removeISIE flag should always set to False. Only set it
 *   to True when used in binaries uninstallation code
 */

NSAPI_EXTERN LdapErrorCode removeSIE(Ldap *, const char *sieDN, Bool removeISIE);

/*
 * removeISIE
 *   Remove the installed software instance. To be used by uninstallation
 *   code.
 *   The ISIE can only be removed when there are no associated server
 *   instances.  The routine will remove the whole SIE hiearchy if
 *   possible up to bot not including netscapRoot.
 */
NSAPI_EXTERN LdapErrorCode removeISIE(Ldap *, const char *isieDN);

/*
 * updateISIEVersionInfo
 *   Update the ISIE with the version information as well as other related
 *   information. Used by the setup program only
 */
NSAPI_EXTERN LdapErrorCode updateISIEVersionInfo (
         Ldap *,
         LdapEntry *,
         const char *productName,
         const char *hostName,
         const char *installedLocation,
         const char *adminDomainName);             

/*
 * createACIForConfigData
 *   Set the appropriate access control for the LDAP entry that contains
 *   modifiable configuration data. e.g. cn=Configuration, cn=sie-cn, ou=
 *   Netscape Suitespot, o=example.com
 *     sieDN must be a valid DN for an existing SIE entry.
 *
 */
NSAPI_EXTERN LdapErrorCode createACIForConfigEntry (LdapEntry *configEntry, const char *sieDN);

/*
****************************************************************
** Miscellaneous routines
*****************************************************************
*/

/*
 * printEntry: print entry to fileName. set fileName to NULL
 *     to print to stdout
 */
NSAPI_EXTERN void printLdapEntry(const LdapEntry *, const char *fileName);

/*
 * verifyLdapUrl
 *   verify the validity of an LDAP URL (format, ldap server, etc...)
 */
extern LdapErrorCode verifyLdapUrl(
         const char *url,
         const char *bindDN,
         const char *bindPwd);

/*
 * authLdapUser()
 *   Just authenticate an Ldap user. This is just like a wrapper
 *   for createLdap() without returning the Ldap object.
 */
extern LdapErrorCode authLdapUser(
         const char *url,
         const char *ldapUser,
         const char *ldapPwd,
         const char *bindDN,
         const char *bindPwd);

/*
 * isSubTreeWritable
 *  Test whether a subtree is writable
 */
NSAPI_EXTERN LdapErrorCode isSubTreeWritable(const Ldap *, const char *subTreeDN);

/*
 * insertLdifEntries
 *   Insert Ldap entries specified in an LDIF-like file into LDAP
 *   below a given DN.
 *
 *   Ldap - pointer to a valid Ldap struct
 *   subTreeDN - the DN of the location to which the entries will be
 *     inserted. If the entry in the LDIF file has a DN attribute,
 *     this DN attribute should be a relative DN and will be
 *     prepended to subTreeDN to form the location where the entry is
 *     to be inserted. If subTreeDN is NULL, the DN attribute found
 *     in the entry will be used as an absolute DN to insert the
 *     entry. If subTreeDN is not NULL and there's no dn attribute
 *     in the LDIF entry, subTreeDN will be used as an absolute DN to
 *     insert the entry to. It's an error if both subTreeDN and the
 *     DN attribute are missing, in which case the entry is simply
 *     skipped.
 *
 *   ldifFile  - name of the LDIF file
 *   adminSIEDN - NULL in general. If not NULL, specify the DN of the
 *      Admin's SIE to be appended to the values of all className
 *      attributes found in the  Ldif entries.
 *
 */
NSAPI_EXTERN LdapErrorCode insertLdifEntries (const Ldap *ldap, const char *subTreeDN, const char *ldifFile, const char *adminSieDN);

NSAPI_EXTERN LdapErrorCode insertSchemaEntries (Ldap *ldap, const char *schemaFile);

NSAPI_EXTERN LdapErrorCode appendSchemaEntries (Ldap *ldap, const char *schemaFile);

/*
 * guessAdminDomain
 *   Guess the Administration Domain that might exist on this directory server
 */
NSAPI_EXTERN char * guessAdminDomainName(const char *ldapURL, const char *ldapUser, const char *ldapPwd);
/*
 * checkAdminDomain
 *   check the validity of the Administration Domain that exists on this directory server
 */
NSAPI_EXTERN LdapErrorCode checkAdminDomain(const Ldap *ldap, const char *adminDomain);

/*
 * addUserDirectory
 *   Configure a domain with a User Directory
 */
NSAPI_EXTERN LdapErrorCode addGlobalUserDirectory (const Ldap *ldap, const char *domain, const char *userDirUrl, const char *bindDN, const char *bindPwd);

/*
 * getUserDirectory
 *   Get the user Directory associated with this SIE
 */
NSAPI_EXTERN LdapErrorCode getUserDirectory(const Ldap *ldap, const char *sieDN, char **url, char **bindDN, char **bindPwd);

/*
 * getLocalUserDirectory
 *   Get the User Directory associated with this server root
 */
NSAPI_EXTERN LdapErrorCode getLocalUserDirectory (const Ldap *,
   const char *domain, const char *hostName, const char *serverRoot, char **userDirUrl, char **bindDN, char **bindPwd);

/*
 * getGlobalUserDirectory
 *   Get the User Directory associated with this domain
 */
NSAPI_EXTERN LdapErrorCode getGlobalUserDirectory (const Ldap *,
   const char *domain, char **userDirUrl, char **bindDN, char **bindPwd);

#ifdef __cplusplus
}
#endif

#ifdef __cplusplus

/*
** CLASS:
**   LdapError
**
** DESCRIPTION:
**   Ldap error object.
** NOTE:
**
*/

class LdapError
{
public:

   LdapError();

   LdapError (LdapErrorCode errCode, const char *msg = "");

   ~LdapError()
     {}

   /*
   ** Assignment operators
   **/

   LdapError & operator = (const LdapError &error);

   LdapError & operator = (LdapErrorCode error);

   operator LdapErrorCode() {
      return _errCode;
   }

   LdapErrorCode errorCode() {
      return _errCode;
   }

   const char *msg() {
      return _errMsg.data();
   }

   LdapError & print (ostream& os = cerr);

protected:

   LdapErrorCode _errCode;
   NSString      _errMsg;

};

/*
** CLASS:
**   Ldap
**
** DESCRIPTION:
**   An LDAP object
**
** NOTE:
**
*/

class Ldap
{
   friend class LdapEntry;

public:
  /*
   * MEMBER:
   *   Ldap::Ldap
   *
   * DESCRIPTION:
   *   Create an LDAP object.
   *   Giving an LDAP URL, a uid, a password, and the optional
   *    bind DN and bind password, try to authenticate against
   *    the LDAP server.
   *
   * INPUTS:
   *   - ldapURL: the URL of the LDAP server in the form
   *      ldap://<host>:<port>/<base suffix>
   *
   *   - ldapUser: the uid or DN of the user to
   *       authenticate against the LDAP server. If an uid is
   *       given, the routine will search for the uid that
   *       is authenticated using the supplied LDAP password.
   *
   *   - ldapPwd: the associated password.
   *
   *   - binddn/bindpwd: optional binddn and password used
   *     to initiate the initial LDAP search. If they are NULL,
   *     anonymous  search will be attempted.
   *
   * OUTPUTS:
   *   LdapErrorCode
   *
   * RETURN
   *   None
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   None
   * MEMORY:
   *
   * NOTE:
   */

   Ldap( LdapError & err,
             const char *ldapURL,
             const char *ldapUser, const char *ldapPwd,
             const char *binddn = 0, const char *bindpwd = 0);

   ~Ldap();

  /*
   * MEMBER:
   *   Ldap::host()
   *   Ldap::port
   *   Ldap::baseSuffix()
   *   Ldap::userDN()
   *   Ldap::userPassword()
   *   Ldap::version()
   *   Ldap::rebindDN()
   *   Ldap::rebindPwd()
   *
   * DESCRIPTION:
   *   Self explanatory.
   *
   * INPUTS:
   *   None
   *
   * OUTPUTS:
   *   - None
   * RETURN:
   *   See Description.
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   None
   * MEMORY:
   *
   */

   const char *host() const;

   int port() const;

   const char *baseSuffix() const;

   const char *userDN () const;

   const char *userPassword() const;

   const char *version() const;

   const char *rebindDN() const;

   const char *rebindPwd() const;

   /*
    * ldap_modify_s wrapper. return ldap error code
    */
   int modify(const char *dn, LDAPMod **mods);


private:

  /*
   * MEMBER:
   *   Ldap::mapLdapError
   *   Ldap::verifyUrl
   *   Ldap::authenticate
   *
   * DESCRIPTION:
   *   mapLdapError:
   *   Map a known LDAP error code to our error code
   *
   *   verifyUrl:
   *   Verify if an Ldap server is valid
   *
   *   Authenticate against an LDAP server.
   *   If an uid was given for ldapUser, the routine will search
   *   for the uid that is authenticated using the supplied LDAP
   *   password.
   *
   *   bindDN and bindPwd are used to initiate the initial LDAP
   *   search. If they were not supplied, anonymous  search will
   *   be attempted.
   *
   *   verifyUrl must be called before authenticate()
   *
   * INPUTS:
   *   None
   *
   * OUTPUTS:
   *   - None
   * RETURN:
   *   LdapError
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   None
   * MEMORY:
   *
   */

   LdapError mapLdapError(int err);
   LdapError verifyUrl (const char *ldapURL);

   LdapError authenticate(const char *ldapUser, const char *ldapPwd);

   NSString _ldapUser;
   NSString _ldapPwd;
   NSString _bindDN;
   NSString _bindPwd;
   NSString _rebindDN;
   NSString _rebindPwd;
   NSString _version;

   LDAP *_ldapHandle;
   LDAPURLDesc *_ldapUrlDesc;
};


/*
** CLASS:
**   LdapEntry
**
** DESCRIPTION:
**   An LDAP entry object
**
** NOTE:
**
*/

class LdapEntry
{
public:

   LdapEntry ()
     {}

   LdapEntry(const Ldap *ldap);

   ~LdapEntry();

   /*
    * Get the DN associated with this entry. Only valid after
    * a successful retrieve() or insert()
    */
   const char *entryDN() const;

   void clear()
     { imp.clear(); }

   Bool isEmpty() const
     { return imp.isEmpty(); }

   /*
    * Get the list of all attribute names in this entry
    */
   char **getAttributeNames() const;
   void freeAttributeNames(char **attrNames);

   /*
    * Set an attribute with value. If this is a multivalue attribute
    * it will become a single-value attribute
    */
   void setAttribute(const char *attr, const char *value);

   /*
    * Add a value to attribute. If it attribute does not exist
    * it will be created
    */
   void addAttribute(const char *attr, const char *value);
   void addAttributes(const char*attr, const char **values);

   /*
    * Get all the values associated with attribute
    */
   char **getAttributes(const char *attr) const;
   void freeAttributes (char **attrs);

   /* 
    * Delete an attribute
    */
   void deleteAttribute(const char *attr);

   /*
    * Get the value of an attribute (first one if multi-value)
    */
   char *getAttribute(const char *attr) const;
   /*
    * Verify the existence of an attribute/value pair
    */
   Bool checkAttribute(const char *attr, const char *value) const;

   /*
    * Return associated LdapObject
    */
   const Ldap * ldap(void) const
      { return _ldap; }

   void change_ldap(Ldap *new_ldap);

   /*
    * Determine whether entry at entryDN exists
    */
   Bool exists (const char *entryDN);

   /*
    * Retrieve the entry at entryDN
    */
   LdapError retrieve (const char *entryDN);
   LdapError retrieve (const char *criteria, int scope, const char *baseDN=0);
   LdapError retrieve (const char *criteria, int scope, int sizelimit, const char *baseDN);

   /*
    * Retrieve the next entry. The current object will be overwritten if
    * there is a next entry.
    * Return NOT_FOUND if there is no more entry, in which case the object
    * will not be overwritten.
    */
   LdapError next();

   int numEntries() const;

   /*
    * Rename this entry
    */

   LdapError rename(const char *entryDN, const char *rdn);

   /*
    * Insert this entry giving its DN. The entry must have been setup properly.
    */
   LdapError insert (const char *entryDN);

   /*
    * Update an entry. The entry must have existed. Note that
    * this routine does not support changing the relative
    * distinguished name.
    *
    * This routine updates the existing LDAP entry with the new
    * attributes/values specified in the LdapEntry object.
    * All existing attributes are kept. New attributes and values
    * are added.
    *
    * To remove an attribute, set it to empty string ""
    *
    */
   LdapError update(const char *entryDN);

   /*
    * Replace an entry. The entry must have existed.
    *
    * This routine replaces an existing LDAP entry with the new
    * one as specified in the LdapEntry object. Values of existing
    * attributes are replaced with values specified in the object.
    * If the LdapEntry object does not contain an attribute that
    * currently exists, that attribute will be deleted.
    *
    * To "replace" a subset of existing attributes, use update().
    *
    */
   LdapError replace(const char *entryDN);

   /*
    * Drop entry at entryDN
    */

   LdapError drop (const char *entryDN) const;
   LdapError dropAll (const char *entryDN);

   /*
    * Debugging stuffs
    */
   void printEntry(const char *filename) const;


private:

   Ldap     *_ldap;

   LDAPMessage *_entry, *_result;

   NSString _entryDN;
   NSString _subTreeDN;
   NSString _filter;
   NSString _errMsg;
   int _numEntries;

   void fillAttributes(LDAP *, LDAPMessage *);

   NVPair imp;
};

/*
** CLASS:
**   LdifEntry
**
** DESCRIPTION:
**   Simple class for reading LDIF file.
**
** NOTE:
**   Warning: does not support line continuation.
**
*/

class LdifEntry:public NVPair
{
public:
   LdifEntry();

   /*
    * Create the object from filename. If successful,
    * the first entry will be read
    */

   LdifEntry (const char *fileName);

   ~LdifEntry();

   /*
    * Read Ldif entries from fileName. The first entry
    * is immediately read. Subsequent entry can be
    * retrieved using nextEntry().
    *
    * Return -1 if any error at all.
    */
   int read(const char *fileName);

   /*
    * Check if LDIF entry is valid. Invalid if no LDIF
    * file is associated with it yet.
    */

   int isValid() const
    { return (_ldif != NULL);  }

   /*
    * Read next entry from LDIF file
    */
   int nextEntry();

private:

   Bool isBlank(const char *buf);

   void stripBlank(char *buf, Bool bothEnds = True);

   FILE  *_ldif;
};

/*
** CLASS:
**   LdapSchema
**
** DESCRIPTION:
**   Class for reading schema file.
**
**
*/

class LdapSchema
{
public:

   LdapSchema ()
     {}

   LdapSchema (Ldap *);
   ~LdapSchema();
   
   /*
    * Retrieve attributetypes and objectclasses from DS    
    * Return ldap error number in case of failure.     
    */
   int retrieve();

   /*
    * getValue
    *   Get attributetypes or objectclasses value
    * Parameters:
    *  type -- "attributetypes" for attribute value
    *          "objectclasses" for objectclasses value  
    *  name -- name is same as name part of value.
    * Note:
    *  retrieve() call required. Value is in the format stored in DS
    * eg. 
    *  ( 2.5.6.0 NAME 'top' DESC 'Standard ObjectClass' MUST ( objectclass )  MAY ( aci ) )
    * 
    */
   const char * getValue(char *type, char *name);

   /* updateEntryValue
    *   Update value stored in Directory server. If name exists, the API will try
    *   to update it. Otherwise it'll add new entry to Directory server
    *   
    * Parameter
    *    type  -- attributetypes or objectclasses
    *    name  -- name is same as name part of value.
    *    value -- value 
    * Return  
    *    ldap error number. 
    */
   int updateEntryValue(char *type, char *name, char *value, int doUpdate);

   /*
    * insertSchemaFile
    *   Read schema configuration file and update it's data to directory server.
    * 
    * return ldap error number.
    */
   int insertSchemaFile(const char *filename, int doUpdate);

private:
   char *getSchemaName(char *value);
   int domodify(LDAPMod **mods);
   Bool nextEntry(char *type, int typeLen, char *name, int nameLen, char *entryValue, int entryValueLen);
   Bool isBlank(const char *buf);
   void stripBlank(char *buf, Bool bothEnds);
   Bool formValue(char *type, NVPair *nv, char *entryValue, int entryValueLen);
   const char * syntaxToOID(const char *syntax);
   Bool isStandardSchema(const char *value);


   int debug_parsing;

   Ldap *_ldap;
   LdapEntry *_ldapEntry;

   NVPair _oc;
   NVPair _at;

   FILE * _schemaFile;
};



#endif /* cplusplus */

#endif
