/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
// -------------------------------------------------------------------------------
// setupnvpair.h
//
// This file contains the NVPair class flattened into a set of C functions.  This
// is done to work around the problem of exporting C++ classes on NT which is a
// pain in the ass.  Trust me, I prefer C++ to C and only do this out of sheer
// necessity.
//
// All the methods of NVPair have been turned into C functions with the exception
// of the operator methods and the methods which use NSString as arguments.  
// Overall, this amounts to roughly four or five methods being removed which is 
// not a huge deal.
//
// I've pretty much done the same thing as a C++ compiler to get the NVPair object
// into C: I turned the object into an opaque type named NVPairHandle and transformed
// every method in NVPair into a function which takes NVPairHandle as the first
// argument.  All the functions do is cast NVPairHandle back into an NVPair object
// and invoke the method on that object which the function specifies.  Pretty simple.
//
// The names of the functions ironically match the NVPair method names with minor
// variations.  All the functions have the following name format:
//
//    setupNVPair[C++ Method Name]
//
// However, in certain cases this does not work, such as constructors, destructors
// and overloaded methods.  In the case of constructors, they are all here and
// begin with setupNVPairCreate followed by an extension to distinguish them.  The
// destructor is named setupNVPairDestroy.  The rest of the method names have been
// more or less kept with minor modification.  That is why the functions here are
// not documented.  See nsdefs.h for the NVPair class itself.
//
// You deal with an NVPairHandle in the same way as an NVPair object.  You must 
// first create it via one of the setupNVPairCreate methods.  After that you are
// free to use any of the setupNVPair functions.  When finished, you simply call
// setupNVPairDestroy to get rid of the NVPairHandle.  Keep in mind that the
// functions are lazy about making sure the given pointer is a legit NVPair object.  
// If you pass in a bad NVPairHandle don't expect anything good to happen.
//
// Author: Brian Hannan
//
// -------------------------------------------------------------------------------

#ifndef __SETUPNVPAIR__
#define __SETUPNVPAIR__

#include "global.h"


// Keeps the C++ compiler from renaming C functions as C++ methods.
#ifdef __cplusplus
extern "C" {
#endif


// Handle to an NVPair object.  
typedef void* NVPairHandle;

// Most the NVPair C++ methods flattened to C.
NSAPI_EXTERN NVPairHandle setupNVPairCreateEmpty(void);
NSAPI_EXTERN NVPairHandle setupNVPairCreateFromFile(char *inFileName);
NSAPI_EXTERN NVPairHandle setupNVPairCreateFromNVPair(void *inNVPair);
NSAPI_EXTERN void         setupNVPairDestroy(NVPairHandle inNVPair);
NSAPI_EXTERN void         setupNVPairRead(NVPairHandle inNVPair, char *inFileName);
NSAPI_EXTERN Bool         setupNVPairIsEmpty(NVPairHandle inNVPair);
NSAPI_EXTERN void         setupNVPairClear(NVPairHandle inNVPair);
NSAPI_EXTERN char        *setupNVPairGetID(NVPairHandle inNVPair);
NSAPI_EXTERN void         setupNVPairSetID(NVPairHandle inNVPair, char *inID);
NSAPI_EXTERN char        *setupNVPairGetTimeStamp(NVPairHandle inNVPair);
NSAPI_EXTERN void         setupNVPairSetTimeStamp(NVPairHandle inNVPair);
NSAPI_EXTERN char        *setupNVPairGet(NVPairHandle inNVPair, char *inName);
NSAPI_EXTERN void         setupNVPairSetString(NVPairHandle inNVPair, char *inName, char *inValue);
NSAPI_EXTERN void         setupNVPairSetInt(NVPairHandle inNVPair, char *inName, int inValue);
NSAPI_EXTERN void         setupNVPairSetUInt(NVPairHandle inNVPair, char *inName, unsigned int inValue);
NSAPI_EXTERN void         setupNVPairSetLong(NVPairHandle inNVPair, char *inName, long inValue);
NSAPI_EXTERN void         setupNVPairSetULong(NVPairHandle  inNVPair,  char *inName, unsigned long inValue);
NSAPI_EXTERN void         setupNVPairSetStringBySection(NVPairHandle inNVPair, char *inSection, char *inName, char *inValue);            
NSAPI_EXTERN void         setupNVPairSetLongBySection(NVPairHandle inNVPair, char *inSection,  char *inName, long inValue);
NSAPI_EXTERN char        *setupNVPairGetListItem(NVPairHandle inNVPair, char *inName, int inIndex);
NSAPI_EXTERN char       **setupNVPairGetListItems(NVPairHandle inNVPair, char *inName);
NSAPI_EXTERN void         setupNVPairFreeListItems(NVPairHandle inNVPair, char **inItems);
NSAPI_EXTERN Bool         setupNVPairInList(NVPairHandle inNVPair, char *inName, char *inValue);
NSAPI_EXTERN Bool         setupNVPairListExists(NVPairHandle inNVPair, char *inName);
NSAPI_EXTERN void         setupNVPairAddString(NVPairHandle inNVPair, char *inList, char *inValue);
NSAPI_EXTERN void         setupNVPairAddStrings(NVPairHandle inNVPair, char *inList, char **inValues);
NSAPI_EXTERN void         setupNVPairAddStringBySection(NVPairHandle inNVPair, char *inSection, char *inList, char *inValue);
NSAPI_EXTERN void         setupNVPairAddStringsBySection(NVPairHandle inNVPair, char *inSection, char *inList, char **inValues);
NSAPI_EXTERN void         setupNVPairDelList(NVPairHandle  inNVPair, char *inName);
NSAPI_EXTERN void         setupNVPairDelListItem(NVPairHandle inNVPair, char *inName, char *inItem);
NSAPI_EXTERN int          setupNVPairNumName(NVPairHandle inNVPair);
NSAPI_EXTERN int          setupNVPairNumList(NVPairHandle inNVPair);
NSAPI_EXTERN int          setupNVPairNumListItems(NVPairHandle inNVPair, char *inList);
NSAPI_EXTERN char        *setupNVPairName(NVPairHandle inNVPair, int inIndex);
NSAPI_EXTERN char        *setupNVPairList(NVPairHandle inNVPair, int inIndex);
NSAPI_EXTERN int          setupNVPairNumSections(NVPairHandle inNVPair);
NSAPI_EXTERN NVPairHandle setupNVPairGetSectionByIndex(NVPairHandle inNVPair, int inIndex);
NSAPI_EXTERN NVPairHandle setupNVPairGetSectionByName(NVPairHandle inNVPair, char *inName);
NSAPI_EXTERN void         setupNVPairAddSection(NVPairHandle inNVPair, char *inName,  NVPairHandle inSubBlock);
NSAPI_EXTERN NVPairHandle setupNVPairCreateSection(NVPairHandle inNVPair, char *inName);
NSAPI_EXTERN void         setupNVPairRename(NVPairHandle inNVPair, char *inOldLabel, char *inNewLabel);
NSAPI_EXTERN void         setupNVPairRemove(NVPairHandle inNVPair, char *inLabel);
NSAPI_EXTERN void         setupNVPairToUTF8(NVPairHandle inNVPair, char *inLabel);
NSAPI_EXTERN void         setupNVPairToLocal(NVPairHandle inNVPair, char *inLabel);
NSAPI_EXTERN int          setupNVPairWriteToFile(NVPairHandle inNVPair, char *inFileName);
NSAPI_EXTERN int          setupNVPairWrite(NVPairHandle inNVPair);
NSAPI_EXTERN void         setupNVPairSetFormat(NVPairHandle inNVPair, int inWhich);

#ifdef __cplusplus
}
#endif

#endif
