/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
**
** NAME:
**   ldapu.cc
**
** DESCRIPTION:
**   Various LDAP utilities used during installation.
**
** NOTES:
**
** HISTORY:
** $Log: ldapu.cpp,v $
** Revision 1.2  2006/03/27 21:18:35  nhosoi
** [186642] Directory Server Makefile updates for Internal build
** Changed the createSIE so that caller (Admin Server and Directory Server) passes
** the brand to SetupUtil.
**
** Revision 1.1.1.1  2005/07/29 22:16:29  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:27  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/07/27 01:04:49  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:13  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.11  2002/02/07 18:51:14  djani
** Now, LdapEntry::retrieve also takes 'int sizelimit' as a parameter per NDA team's request. Also, at the end of the method, sizelimit is set to -1 for that ld.
**
** Revision 1.1.1.10  2001/11/06 01:51:31  ahakim
** XXX AOL/NSCP Copyright XXX
**
** Revision 1.1.1.9  2001/01/17 01:12:39  ahakim
** 528703: ldap write error when upgrading
**
** Revision 1.1.1.8  2000/09/07 03:55:45  peterz
** for attribute equal to passwordExpirationTime, always do a REPLACE ldap operation
**
** Revision 1.1.1.7  2000/08/31 23:36:50  peterz
** replace * with LDAP_ALL_USER_ATTRS and terminate the array with NULL
**
** Revision 1.1.1.6  2000/08/23 21:08:33  peterz
** retrieve all attributes besides operational attributes (recommended by mark whal)
**
** Revision 1.1.1.5  2000/08/18 02:22:17  peterz
** fix bug 436889 -- ldap_search_s returns operational attributes
**
** Revision 1.1.1.4  1999/06/04 06:03:43  cxue
** 354173. version parsing code crashes when connecting to Sun's Directory server
**
** Revision 1.1.1.3  1999/04/20 17:25:39  cxue
** set LDAP_OPT_PROTOCAL_VERSION to 3
**
** Revision 1.1.1.2  1999/03/26 18:38:06  cxue
** bug 347018. Use printf instead of iostream
**
** Revision 1.1.1.1  1999/03/04 23:28:26  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.74.4.20  1998/11/20 04:27:06  pvo
** Remove insertLdif and add updateLdif and replaceLdif
**
** Revision 1.1.2.74.4.19  1998/11/20 00:52:48  pvo
** 333357 - need to close ldif file upon destruction of LdifEntry
**
** Revision 1.1.2.74.4.18  1998/11/13 23:33:18  gbeasley
** unified api
**
** Revision 1.1.2.74.4.17  1998/11/12 21:08:30  gbeasley
** 334525 formAdminGroupSearchCriteria and correct spelling of nsRevisionNumber
**
** Revision 1.1.2.74.4.16  1998/11/10 23:58:20  cxue
** log error for schema and ldif items
**
** Revision 1.1.2.74.4.15  1998/10/29 22:43:34  pvo
** Change SLEEP to WAIT_A_BIT
**
** Revision 1.1.2.74.4.14  1998/10/29 04:22:53  pvo
** 331981 - add fix for installation problem against replicated DS
**
** Revision 1.1.2.74.4.13  1998/10/28 19:08:37  pvo
** 310620 - update build number upon re-install
**
** Revision 1.1.2.74.4.12  1998/10/28 17:59:34  pvo
** Fixed referral problem with createACIForConfig()
**
** Revision 1.1.2.74.4.11  1998/10/21 21:51:29  adam
** Add deleteAttribute member function.
**
** Revision 1.1.2.74.4.10  1998/10/13 21:03:10  gbeasley
** changing nss to setup
**
** Revision 1.1.2.74.4.9  1998/10/12 04:27:51  gbeasley
** silent uninstall
**
** Revision 1.1.2.74.4.8  1998/10/08 23:07:46  adam
** Add member function "change_ldap" to LdapEntry class.
**
** Revision 1.1.2.74.4.7  1998/10/05 23:51:34  cxue
** not return ldap handle from Ldap class. Instead using modify for general ldap modification purpose
**
** Revision 1.1.2.74.4.6  1998/10/02 22:13:03  pvo
** Fixed DS instance creation problem with delegated admin
**
** Revision 1.1.2.74.4.5  1998/10/02 22:11:04  pvo
** Fixed DS instance creation problem with delegated admin
**
** Revision 1.1.2.74.4.4  1998/09/23 22:14:57  pvo
** Support for LDAP referral
**
** Revision 1.1.2.74  1998/08/22 00:39:33  cxue
** move formAdminDomainDN to nsutils.cpp
**
** Revision 1.1.2.73  1998/08/20 05:34:33  pvo
** Fixed ACI problem
**
** Revision 1.1.2.72  1998/08/19 21:35:57  gbeasley
** checkadmindomain
**
** Revision 1.1.2.71  1998/08/19 17:43:25  cxue
** return NULL instead of o=netscaperoot if guessAdminDomain can not find it
**
** Revision 1.1.2.70  1998/08/18 03:49:38  pvo
** 314468 - Support new presentation of Config Dir LDAP URL
**
** Revision 1.1.2.69  1998/08/12 23:45:44  pvo
** Add enumerations for LdapEntry attributes
**
** Revision 1.1.2.68  1998/08/11 01:51:31  gbeasley
** Change replace to update
**
** Revision 1.1.2.67  1998/07/17 03:27:28  pvo
** Reduce redundant statements to assist mem corruption debugging
**
** Revision 1.1.2.66  1998/07/16 17:11:48  pvo
** 300883 - Fix Express install problem. Reviewed by Tony.
**
** Revision 1.1.2.65  1998/07/01 01:06:42  pvo
** add timestamp to SIE
**
** Revision 1.1.2.64  1998/06/30 02:29:19  pvo
** Assume version 4 if cn=monitor search fails
**
** Revision 1.1.2.63  1998/06/24 17:40:45  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.62  1998/05/30 00:44:38  gbeasley
** 120112 domain name verify
**
** Revision 1.1.2.61  1998/05/28 21:16:11  pvo
** 122386 - use i18n_isspace() rather than isspace()
**
** Revision 1.1.2.60  1998/05/23 01:08:47  gbeasley
** 116918 nt admin core dumps
**
** Revision 1.1.2.59  1998/05/14 23:48:48  gbeasley
** just removed compiler warnings
**
** Revision 1.1.2.58  1998/05/13 04:39:27  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.57  1998/05/08 16:35:48  pvo
** Fixed compilation error
**
** Revision 1.1.2.56  1998/05/08 02:17:49  pvo
** Support relocation of MC Admin.
**
** Revision 1.1.2.55  1998/05/07 00:56:15  pvo
** 112552 - New ACI
**
** Revision 1.1.2.54  1998/05/01 23:24:38  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.53  1998/04/22 21:13:22  gbeasley
** fix memory trap
**
** Revision 1.1.2.52  1998/04/20 21:40:09  pvo
** 114084 - Write global prefs aci
**
** Revision 1.1.2.51  1998/04/20 20:51:32  pvo
** 112701 - problem reconfiguring SIE
**
** Revision 1.1.2.50  1998/03/28 03:28:07  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.49  1998/03/18 18:24:09  pvo
** Fixed an ACI problem.
**
** Revision 1.1.2.48  1998/03/13 21:24:45  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.47  1998/03/11 23:17:41  pvo
** Do not createSIE if it already exists.
**
** Revision 1.1.2.46  1998/03/07 04:39:34  pvo
** Support schema changes.
** Let adminSIE be a member of other SIE groups in the same server root.
**
** Revision 1.1.2.45  1998/03/07 03:22:23  pvo
** Support schema changes.
** Allow writing empty attribute from Ldif file.
** Set new ACI to open up Global Preferences.
**
** Revision 1.1.2.44  1998/03/04 02:27:39  pvo
** Added support for Ldif line continuation and common time format.
**
** Revision 1.1.2.43  1998/03/03 22:24:17  gbeasley
** removed warnings
**
** Revision 1.1.2.42  1998/02/27 23:44:44  pvo
** Fixed delegation setting problem.
**
** Revision 1.1.2.41  1998/02/27 04:38:30  gbeasley
** aix build break with win32 code
**
** Revision 1.1.2.40  1998/02/27 03:51:28  gbeasley
** nt sie fix to expace " ldapu.cpp
**
** Revision 1.1.2.39  1998/02/26 04:02:35  pvo
** Attempt to create Installation Root DN  in case the user has the privilege.
**
** Revision 1.1.2.38  1998/02/26 01:39:33  gbeasley
** more sie integration
**
** Revision 1.1.2.37  1998/02/26 00:13:48  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.36  1998/02/24 00:40:28  pvo
** Change to support change from globals.{h,cpp} to global.{h, cpp}
**
** Revision 1.1.2.35  1998/02/24 00:21:30  pvo
** Support topology change.
**
** Revision 1.1.2.34  1998/02/15 00:03:09  pvo
** 106454  - Uninstall causes seg fault
**
** Revision 1.1.2.33  1998/02/10 00:06:35  pvo
** 105971 - Allow local admin to access other servers within the same
** server root.
**
** Revision 1.1.2.32  1998/02/08 23:13:14  pvo
** 105971- More delegation support.
**
** Revision 1.1.2.31  1998/02/08 21:50:46  pvo
** 105971 - Delegation support
**
** Revision 1.1.2.30  1998/02/04 21:46:43  pvo
** 105539 - admin server instance naming
**
** Revision 1.1.2.29  1998/01/30 21:44:29  pvo
** 104884 Uninstall of first servers caused subsequent SIE create to fail
** 105218 Create SIE should not suffix hostname to SIE instance name
**
** Revision 1.1.2.28  1998/01/29 21:49:27  pvo
** Bug 104982 - show error if ldap URL does not contain a port number
**
** Revision 1.1.2.27  1998/01/23 01:42:04  pvo
** Support writing NYR into DS.
**
** Revision 1.1.2.26  1998/01/22 04:49:36  pvo
** Changed Directory Administrators to SuiteSpot Administrators.
** Changed Admin Group to SuiteSpot 4.0
**
** Revision 1.1.2.25  1997/12/13 04:28:37  pvo
** Support new schema.
**
** Revision 1.1.2.24  1997/12/13 00:21:50  pvo
** Added global preferences.
**
** Revision 1.1.2.23  1997/12/03 19:01:24  pvo
** Do not write ACI to NetscapeRoot at initial installation since they should
** have been set up during directory server installation already.
**
** Revision 1.1.2.22  1997/11/15 04:59:24  pvo
** Fixed memory leak problems and redo + operator for NSString.
**
** Revision 1.1.2.21  1997/11/14 05:20:18  pvo
** Fixed a couple of  freeAttributes.
**
** Revision 1.1.2.20  1997/11/13 19:28:20  pvo
** More new stuffs.
**
** Revision 1.1.2.19  1997/11/09 19:26:15  pvo
** API name changes. Added freeAttributes and freeListItems.
**
** Revision 1.1.2.18  1997/11/07 22:43:23  pvo
** New code to support uninstallation.
**
** Revision 1.1.2.17  1997/10/28 09:11:11  achan
** Added #include <strings.h> for AIX to get declaration for strcasecmp().
**
** Revision 1.1.2.16  1997/10/28 01:06:08  pvo
** Added DEFAULT_SIE_ACI.
**
** Revision 1.1.2.15  1997/10/25 00:37:02  pvo
** Fixed compilation error.
**
** Revision 1.1.2.14  1997/10/25 00:35:15  pvo
** Added ldap entry and SIE removal code.
**
** Revision 1.1.2.13  1997/10/24 03:39:28  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.12  1997/10/23 03:23:29  pvo
** More change to support other platforms.
**
** Revision 1.1.2.11  1997/10/22 02:40:46  pvo
** Too many changes to list out here.
**
** Revision 1.1.2.10  1997/10/16 04:22:42  pvo
** Quite a lot of code changes for ACI support.
**
** Revision 1.1.2.9  1997/10/14 05:01:53  pvo
** More code.
**
** Revision 1.1.2.8  1997/10/11 01:44:58  pvo
** Fixed some more bugs.
**
** Revision 1.1.2.7  1997/10/11 01:26:54  pvo
** Fixed bugs.
**
** Revision 1.1.2.6  1997/10/10 22:50:59  pvo
** Writing ACI to netscapeRoot.
**
** Revision 1.1.2.5  1997/10/10 04:24:45  pvo
** Added the setting of default ACI at netscapeRoot level.
**
** Revision 1.1.2.4  1997/10/10 00:37:50  pvo
** Resolved conflict in namespace with zip library
**
** Revision 1.1.2.3  1997/10/06 23:50:28  pvo
** Removed unused variables.
**
** Revision 1.1.2.2  1997/10/06 23:43:25  pvo
** Cleaned up for NT compilation.
**
** Revision 1.1.2.1  1997/10/06 20:36:37  pvo
** Restructure depdencies and changed suffixes from .cc to .cpp
**
** Revision 1.1.2.9  1997/10/04 01:49:04  pvo
** Added LdapEntry update code.
**
** Revision 1.1.2.8  1997/10/02 00:56:17  pvo
** Fixed compilation bugs.
**
** Revision 1.1.2.7  1997/09/29 22:52:14  pvo
** Additional code.
**
** Revision 1.1.2.6  1997/09/27 02:46:57  pvo
** Checked in.
**
*********************************************************************/

extern "C" {
#include <stdlib.h>
#include <stdio.h>
#if defined(AIX)
   #include <strings.h>
#endif
#include <string.h>
#include <ctype.h>
}

#include "global.h"
#include "ldapu.h"
#include "misc.h"
#include "nsutils.h"
#include "setupapi.h"
#include "setupdefs.h"
#include "setupldap.h"

extern "C" {
static int LDAP_CALL LDAP_CALLBACK rebind_proc (LDAP *ld, char **dnp, char **passwdp, int *, int, void *);
}

////////////////////////////////////////////////////////////////
//// CLASS LdapError
////////////////////////////////////////////////////////////////
LdapError::LdapError()
{
   _errCode = OKAY;
}

LdapError::LdapError(LdapErrorCode errCode, const char *msg)
{
   _errCode = errCode;
   _errMsg  = ldap_err2string(errCode);
   _errMsg = _errMsg + "." + msg;
}


LdapError &
LdapError::operator = (const LdapError & error)
{
   if (this != & error)
   {
      _errCode = error._errCode;
      _errMsg    = error._errMsg;
   }
   return *this;
}

LdapError &
LdapError::operator = (LdapErrorCode errCode)
{
   _errCode   = errCode;
   _errMsg    = ldap_err2string(errCode);
   return *this;
}

LdapError &
LdapError::print (ostream & os)
{
   if (_errCode != OKAY)
   {
      printf("-------- Ldap Error Information --------\n");
      printf(" Error Code:       %d\n", (int) _errCode);
      printf(" Error Message:    %s\n", _errMsg.data());
   }
   return *this;
}

////////////////////////////////////////////////////////////////
//// CLASS Ldap
////////////////////////////////////////////////////////////////

Ldap::Ldap(LdapError & err,
            const char *ldapURL,
            const char *ldapUser, const char *ldapPwd,
            const char *binddn, const char *bindpwd)
    :
    _ldapHandle(NULL), _ldapUrlDesc(NULL), _bindDN(binddn), _bindPwd(bindpwd),
    _rebindDN(binddn), _rebindPwd(bindpwd)
{
    err = verifyUrl(ldapURL);

    if (err == OKAY)
    {
       LdapEntry ldapMonitor(this);
       ldapMonitor.retrieve("cn=monitor");

       if (ldapMonitor.getAttribute("Version") == NULL)
       {
          // Assume 4.0
          _version = "4.0";
       }
       else
       {
         char *version;
         char *s1;
         char *s2;

         /*
          *  Set arbitrary version = "3.0" in case we can't 
          *  read version number from cn=monitor. This generally
          *  means the DS is not from Netscape
          */ 
         _version = "3.0";

         version = strdup(ldapMonitor.getAttribute("Version"));

         if (s1 = strchr(version, '/'))
         {
           if (s2 = strchr(s1 + 1, ' '))
            *s2 = '\0';
           _version = s1 + 1;
         }

         free (version);
       }
    }
    if (err == OKAY && ldapUser != NULL && ldapPwd != NULL)
    {
       err = authenticate(ldapUser, ldapPwd);
    }
}

Ldap::~Ldap()
{
   if (_ldapUrlDesc)
   {
      ldap_free_urldesc(_ldapUrlDesc);
   }

   if (_ldapHandle)
   {
      ldap_unbind(_ldapHandle);
   }
}

/*********************************************************************
**
** FUNCTION:    Ldap::mapLdapError
** DESCRIPTION:
**
** INPUTS:      LDAP Error
** OUTPUTS:     None
** RETURN:
**  LdapErrorCode
**
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
LdapError
Ldap::mapLdapError(int err)
{
   if ((err & 0xf0) == 0x50)
   {
      err = CONN_FAILED;
   }
   else if (err == LDAP_PARTIAL_RESULTS)
   {
      err = INVALID_DN;
   }
   else if (err == LDAP_NO_SUCH_OBJECT)
   {
      err = INVALID_USER;
   }
   else if (err == LDAP_STRONG_AUTH_REQUIRED ||
            err == LDAP_INAPPROPRIATE_AUTH   ||
            err == LDAP_INVALID_CREDENTIALS ||
            err == LDAP_INSUFFICIENT_ACCESS)
   {
      err = INVALID_AUTH;
   }
   return err;
}

/*********************************************************************
**
** FUNCTION:    Ldap::verifyUrl
** DESCRIPTION:
**
** INPUTS:      url  - the ldap url.
** OUTPUTS:     none
** RETURN:
**
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
LdapError
Ldap::verifyUrl(const char *url)
{
   LdapErrorCode err;
   LDAPURLDesc *ludpp = NULL;
   LDAP        *ld    = NULL;

   if (ldap_is_ldap_url((char *)url) == 0)
   {
      err = INVALID_URL;
   }
   else if (ldap_url_parse ((char *) url , &ludpp) || ludpp->lud_port == 0)
   {
      err = INVALID_URL;
   }
   else if ((ld = ldap_init (ludpp->lud_host, ludpp->lud_port)) == NULL)

   {
      err = CONN_FAILED;
   }
   else
   {
      int nVer = LDAP_VERSION3;
//    ldap_set_option (ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF);

      // set protocol version to ldapv3.  Required for smart referral provided by Hin 
      ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &nVer); 

      err = ldap_simple_bind_s(ld, NULL, NULL);

      if (err == LDAP_SUCCESS)
      {
         _ldapHandle = ld;
         _ldapUrlDesc = ludpp;
      }
   }

   if (err == OKAY)
   {
      _ldapHandle = ld;
      _ldapUrlDesc = ludpp;
   }
   else
   {
      if (ld)
      {
         ldap_unbind(ld);
           _ldapHandle = NULL;
      }

      if (ludpp)
      {
         ldap_free_urldesc(ludpp);
         _ldapUrlDesc = NULL;
      }
   }

   return mapLdapError(err);
}

/*********************************************************************
**
** FUNCTION:    Ldap::authenticate
** DESCRIPTION:
**
** INPUTS:      url  - the ldap url.
** OUTPUTS:     none
** RETURN:
**   OKAY: successful
**
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
LdapError
Ldap::authenticate(const char *ldapUser, const char *ldapPwd)
{
  LdapError err;
  LDAPMessage* msg = NULL;
  int  numEntries;
  char *dn = NULL;
  char ldapFilter[MED_BUF];

  LDAPURLDesc *ludpp = _ldapUrlDesc;
  LDAP        *ld    = _ldapHandle;

  // Set rebind proc
  ldap_set_rebind_proc(ld, rebind_proc, this);

  // Check to see if ldapUser entered is a DN or just an id
  if (strpbrk(ldapUser, "=,") != NULL)
  {
     // Looks like a DN

     _rebindDN   = ldapUser;
     _rebindPwd  = ldapPwd;

     err = ldap_simple_bind_s(ld, ldapUser, ldapPwd);

     if (err == LDAP_SUCCESS)
     {
        _ldapUser = ldapUser;
     }
  }
  else if ((err = ldap_simple_bind_s(ld, _bindDN.data(), _bindPwd.data())) == LDAP_SUCCESS)
  {
     snprintf(ldapFilter, sizeof(ldapFilter), "(uid=%s)", ldapUser);
     if (!memchr(ldapFilter, 0, sizeof(ldapFilter)))
     {
         err = INVALID_INPUT;
         goto done;
     }

     err = ldap_search_s(ld, ludpp->lud_dn,
                             LDAP_SCOPE_SUBTREE, ldapFilter, NULL,
                             0, &msg);

     if (err != LDAP_SUCCESS || ldap_count_entries(ld, msg) == 0)
     {
         ldap_msgfree(msg);
         err = NOT_FOUND;

         // If this is an authentication against o=NetscapeRoot,
         // Need to authenticate against the UserDirectory
         if (ludpp->lud_dn && !strcasecmp(ludpp->lud_dn, DEFAULT_ROOT_DN))
         {
            LdapEntry domainEntry(this);
            char *userDirectoryURL=NULL, *bindDN=NULL, *bindPwd=NULL;


            err = domainEntry.retrieve("(objectclass=nsAdminDomain)", LDAP_SCOPE_ONELEVEL, ludpp->lud_dn);

            if (err == OKAY)
            {
               err = NOT_FOUND;

               numEntries = domainEntry.numEntries();

               while (numEntries--)
               {
                  err = getGlobalUserDirectory(this, domainEntry.entryDN(),
                        &userDirectoryURL, &bindDN, &bindPwd);

                  if (err == OKAY)
                  {
                     Ldap ldap(err, userDirectoryURL, ldapUser, ldapPwd, bindDN, bindPwd);
                     if (err == OKAY)
                     {
                        _ldapUser = ldap.userDN();
                        break;
                     }
                  }
                  if (userDirectoryURL) free(userDirectoryURL);
                  if (bindDN) free(bindDN);
                  if (bindPwd) free(bindPwd);

                  if (numEntries)
                  {
                     domainEntry.next();
                  }
               }
           }
        }
     }
     else
     {
        numEntries = ldap_count_entries(ld, msg);

        LDAPMessage *entry = ldap_first_entry(ld, msg);
        dn = ldap_get_dn(ld, entry);

        _rebindDN = dn;
        _rebindPwd = ldapPwd;

        while ((err = ldap_simple_bind_s(ld, dn, ldapPwd)) != LDAP_SUCCESS)
        {
            entry = ldap_next_entry(ld, entry);
            if (entry == NULL)
            {
               err = LDAP_INSUFFICIENT_ACCESS;
               break;
            }
            else
            {
               if (dn)
               {
                  ldap_memfree(dn);
               }

               dn = ldap_get_dn(ld, entry);
               _rebindDN = dn;
            }
        }

        if (err == OKAY)
        {
           _ldapUser = dn;
           ldap_memfree(dn);
        }

        ldap_msgfree(msg);
     }
   }
done:
   if (err == OKAY)
   {
      _ldapPwd = ldapPwd;
   }
   else
   {
      ldap_unbind(ld);
      ldap_free_urldesc(ludpp);
      _ldapHandle = NULL;
      _ldapUrlDesc = NULL;
   }

   return mapLdapError(err);
}

const char *
Ldap::host() const
{
   if (_ldapUrlDesc)
   {
      return _ldapUrlDesc->lud_host;
   }
   return NULL;
}

int
Ldap::port() const
{
   if (_ldapUrlDesc)
   {
      return _ldapUrlDesc->lud_port;
   }
   else
   {
      return 389;
   }
}

const char *
Ldap::baseSuffix() const
{
   if (_ldapUrlDesc)
   {
       return _ldapUrlDesc->lud_dn;
   }
   else
   {
      return NULL;
   }
}
const char *
Ldap::userDN () const
{
   return _ldapUser.data();
}

const char *
Ldap::userPassword() const
{
   return _ldapPwd.data();
}

const char *
Ldap::version() const
{
   return _version.data();
}

const char *
Ldap::rebindDN() const
{
   return _rebindDN.data();
}

const char *
Ldap::rebindPwd() const
{
   return _rebindPwd.data();
}


int Ldap::modify(const char *dn, LDAPMod **mods)
{
   return ldap_modify_s(_ldapHandle, dn, mods);
}

///////////////////////////////////////////////////////////////
/// CLASS LdapEntry
///////////////////////////////////////////////////////////////

LdapEntry::LdapEntry( const Ldap *ldap)
{
   _ldap = (Ldap *) ldap;
   _entry = NULL;
   _result = NULL;
   _numEntries = 0;
}

LdapEntry::~LdapEntry()
{
   if (_result)
      ldap_msgfree(_result);
}

void
LdapEntry::setAttribute (const char *attr, const char *value)
{
   imp.delList (attr);
   imp.add(attr, value);
}

void
LdapEntry::addAttribute (const char *attr, const char *value)
{
   imp.add(attr,value);
}

void
LdapEntry::addAttributes(const char *attr, const char **values)
{
   if (values == NULL || attr == NULL)
      return;

   for (int i = 0; values[i] != NULL && i < 500; i++) // Guard
   {
     addAttribute(attr, values[i]);
   }
}

char *
LdapEntry::getAttribute (const char *attr) const
{
   char **tmp;
   char *s;

   if (attr == NULL || (tmp = imp.getListItems(attr)) == NULL ||
         tmp[0] == NULL)
   {
      return NULL;
   }
   else
   {
      s = strdup(tmp[0]);
      imp.freeListItems(tmp);
   }
   return s;
}

char **
LdapEntry::getAttributeNames() const
{
   char **list = NULL;
   int num;
   char *s;

   num = imp.numList();

   if (num > 0)
   {
      list = (char **) malloc (sizeof(char *)*(num+1));
      for (int i = 0; i < num ; ++i)
      {
         if (s = (char *) imp.list(i))
         {
            list[i] = strdup(s);
         }
      }
      list[num] = NULL;
   }
   return list;
}

void
LdapEntry::freeAttributeNames(char **attrs)
{
   imp.freeListItems(attrs);
}

char **
LdapEntry::getAttributes(const char *attr) const
{
   return imp.getListItems(attr);
}

void
LdapEntry::deleteAttribute(const char *attr)
{
   imp.delList(attr);
}

void
LdapEntry::freeAttributes(char **attributes)
{
   imp.freeListItems(attributes);
}

const char *
LdapEntry::entryDN() const
{
   return _entryDN.data();
}

Bool
LdapEntry::checkAttribute(const char *attr, const char *value) const
{
   return imp.inList(attr, value);
}

void
LdapEntry::change_ldap(Ldap *new_ldap)
{
   _ldap = new_ldap;
}

Bool
LdapEntry::exists(const char *entryDN)
{
   LDAPMessage *result;
   LDAP *ld;
   Bool exists = False;
   LdapError err;

   ld = (LDAP *) _ldap->_ldapHandle;

   err = ldap_search_s (ld, entryDN, LDAP_SCOPE_BASE, OBJECT_CLASS_FILTER, NULL, 0, &result);

   if (err == OKAY)
   {
      if (ldap_count_entries(ld, result))
      {
         char *dn;
         exists = True;
         dn = ldap_get_dn(ld, result);
     _entryDN = dn;
         ldap_memfree(dn);

      }
   }

   return exists;
}

LdapError
LdapEntry::retrieve(const char *entryDN)
{
   return retrieve("(objectclass=*)", LDAP_SCOPE_BASE, entryDN);
}

LdapError
LdapEntry::rename(const char *dn, const char *rdn)
{
   LdapError err;

   if (rdn == NULL)
   {
      err = INVALID_INPUT;
   }
   else
   {
      err = ldap_modrdn2_s(_ldap->_ldapHandle, dn, rdn, 1);
   }

   if (err == OKAY)
   {
      // Change the entryDN
      char **explodedDN = ldap_explode_dn(dn, 0);
      char **s = explodedDN;

      _entryDN = rdn;
      ++s;

      while (*s)
      {
         _entryDN = _entryDN + LDAP_PATHSEP + *s;
         ++s;
      }

      ldap_value_free(explodedDN);
   }

   return err;
}

void
LdapEntry::fillAttributes(LDAP *ld, LDAPMessage *entry)
{
   char *attr;
   char **vals;
   char *dn;
   BerElement *ber;

   dn = ldap_get_dn(ld, entry);
   _entryDN = dn;
   ldap_memfree(dn);

   // Clear my self first
   imp.clear();

   for (attr = ldap_first_attribute(ld, _entry, &ber);
        attr != NULL;
        attr = ldap_next_attribute( ld, _entry, ber))
   {
      if ( (vals = ldap_get_values( ld, _entry, attr)) != NULL)
      {
         for (int i = 0; vals[i] != NULL; i++)
         {
            // Store the multiple attribute values
            imp.add(attr, vals[i]);
         }
         ldap_value_free(vals);
      }
      ldap_memfree(attr);
   }

   if (ber != NULL)
   {
      ldap_ber_free( ber, 0);
   }
}


LdapError
LdapEntry::retrieve(const char *criteria, int scope, const char *entryDN)
{
   return retrieve(criteria, scope, -1, entryDN);
}

LdapError
LdapEntry::retrieve(const char *criteria, int scope, int sizelimit, const char *entryDN)
{
   LdapError err;
   int nolimit = -1;
   LDAP *ld;
   char *attrs[] = {(char *)LDAP_ALL_USER_ATTRS,
       (char *)"aci",
       (char *)"passwordExpirationTime",
       (char *)"passwordExpWarned", 
       (char *)"passwordRetryCount",
       (char *)"retryCountResetTime",
       (char *)"accountUnlockTime",
       (char *)"passwordHistory",
       (char *)"passwordAllowChangeTime",
       (char *)"nsUniqueId",
       (char *)"nsLookthroughLimit",
       (char *)"nsSizeLimit",
       (char *)"nsTimeLimit",
       (char *)"nsIdleTimeout",
       (char *)"nsRole",
       (char *)"nsRoleDN",
       (char *)"nsAccountLock",
       NULL};

   if (_result)
   {
      ldap_msgfree(_result);
      _result = _entry = NULL;
   }

   _numEntries = 0;

   // Search starting from _entryDN using filter as criteria
   ld = (LDAP *) _ldap->_ldapHandle;

   // Set the size limit to what the caller wants
   ldap_set_option(ld, LDAP_OPT_SIZELIMIT, (void *) &sizelimit);

   //err = ldap_search_s (ld, entryDN, scope, criteria, NULL, 0, &_result);
   err = ldap_search_s (ld, entryDN, scope, criteria, attrs, 0, &_result);

   // After the search, we don't want that sizelimit lingering about, so set
   // it to no limit (-1)
   ldap_set_option(ld, LDAP_OPT_SIZELIMIT, (void *) &nolimit);

   if (err == LDAP_NO_SUCH_OBJECT)
   {
      err = NOT_FOUND;
   }
   else if (err == LDAP_SUCCESS)
   {
      _numEntries = ldap_count_entries(ld, _result);
      if (_numEntries == 0)
      {
         err = NOT_FOUND;
      }
      else
      {
         _entry = ldap_first_entry (ld, _result);
         fillAttributes(ld, _entry);
      }
   }

   return err;
}

LdapError
LdapEntry::next()
{
   LDAPMessage *e;
   LDAP *ld = (LDAP *) _ldap->_ldapHandle;

   if (_entry == NULL || _numEntries <= 1)
   {
      return  NOT_FOUND;
   }
   else if ((e = ldap_next_entry(ld, _entry)) == NULL)
   {
      return NOT_FOUND;
   }
   else
   {
      _entry = e;

      fillAttributes(ld, _entry);
   }

   return OKAY;

}

int
LdapEntry::numEntries() const
{
   return _numEntries;
}


LdapError
LdapEntry::insert (const char *entryDN)
{
   int numAttributes;
   LDAPMod **attributes;
   int i;
   char *s = NULL;
   char *matched;

   if (imp.isEmpty())
   {
      return INVALID_OBJECT;
   }

   // Make sure some vars are set properly
   addAttribute("objectclass", "top");

   numAttributes = imp.numList();
   attributes = new LDAPMod * [numAttributes+1];
   for (i = 0; i < numAttributes; i++)
   {
      attributes[i] = new LDAPMod;
      attributes[i]->mod_op = 0;
      s = (char *) imp.list(i);
      attributes[i]->mod_type = s;
      attributes[i]->mod_values = (char **) getAttributes(s);
   }
   attributes[i] = NULL;

   LDAP *ld = (LDAP *) _ldap->_ldapHandle;

   LdapError err = ldap_add_s (ld, entryDN, attributes);

   ldap_get_lderrno(ld, &matched, NULL);


   // Free the damn thing
   for (i = 0; i < numAttributes; i++)
   {
      freeAttributes (attributes[i]->mod_values);
      delete attributes[i];
   }

   delete [] attributes;

   if (err == OKAY)
   {
      _entryDN = entryDN;
      _numEntries = 1;
   }

   return err;
}

LdapError
LdapEntry::update(const char *entryDN)
{
   int numAttributes;
   LDAPMod **attributes;

   int i;
   char *attr = NULL;
   char **newvals = NULL;
   char **oldvals = NULL;
   char *matched;
   LdapError err;

   if (imp.isEmpty())
   {
      return INVALID_OBJECT;
   }

   LDAP *ld = (LDAP *) _ldap->_ldapHandle;
   LdapEntry oldEntry(_ldap);

   err = oldEntry.retrieve(entryDN);

   if (err)
   {
      return err;
   }

   numAttributes = imp.numList();
   attributes = new LDAPMod * [numAttributes+1];

   for (i= 0; i < numAttributes; i++)
   {
      attributes[i] = new LDAPMod;

      // Get the attribute
      attr = (char *) imp.list(i);

      // Get old attributes
      oldvals = oldEntry.getAttributes(attr);
      newvals    = getAttributes(attr);

      attributes[i]->mod_type = attr;
      if (oldvals == NULL)
      {
         attributes[i]->mod_op = LDAP_MOD_REPLACE;  // 528703
         attributes[i]->mod_values = newvals;
      }
      else
      {
         if (newvals[0] == "")
         {
            attributes[i]->mod_op = LDAP_MOD_DELETE;
            attributes[i]->mod_values = NULL;
         }
         else
         {
           /*
            * Merge the two lists
            */
           oldEntry.addAttributes(attr, (const char **) newvals);
           char **vals = oldEntry.getAttributes(attr);

           attributes[i]->mod_op = LDAP_MOD_REPLACE;
           attributes[i]->mod_values = vals;
         }
         freeAttributes (newvals);
         freeAttributes (oldvals);
      }
   }
   attributes[i] = NULL;

   err = ldap_modify_s (ld, entryDN, attributes);

   ldap_get_lderrno(ld, &matched, NULL);


   // Free the damn thing
   for (i = 0; i < numAttributes; i++)
   {
      freeAttributes (attributes[i]->mod_values);
      delete attributes[i];
   }

   delete [] attributes;

   if (err == OKAY)
   {
      _entryDN = entryDN;
      _numEntries = 1;
   }

   return err;
}

LdapError
LdapEntry::replace(const char *entryDN)
{
   int i;
   int numOldAttributes;
   int numNewAttributes;
   int numAttributes;

   LdapError err;
   LDAPMod **attributes;

   char *attr = NULL;
   char **vals = NULL;
   char *matched;

   if (imp.isEmpty())
   {
      return INVALID_OBJECT;
   }

   // Make sure some vars are set properly
   imp.add("objectclass", "top");

   LDAP *ld = (LDAP *) _ldap->_ldapHandle;
   LdapEntry oldEntry(_ldap);
   LdapEntry newEntry(_ldap);

   err = oldEntry.retrieve(entryDN);

   if (err)
   {
      return err;
   }

   vals = ldap_explode_dn(entryDN, 0);
   attr = strchr(vals[0], '=');
   *attr++ = 0;

   addAttribute(vals[0], attr);
   ldap_value_free(vals);

   numOldAttributes = oldEntry.imp.numList();
   numNewAttributes = imp.numList();

   for (i= 0; i < numOldAttributes; i++)
   {
      newEntry.addAttribute(oldEntry.imp.list(i), "");

   }
   for (i = 0; i < numNewAttributes; i++)
   {
      newEntry.addAttribute(imp.list(i), "");
   }

   numAttributes = newEntry.imp.numList();
   attributes = new LDAPMod * [numAttributes+1];

   for (i = 0; i < numAttributes; i++)
   {
      attributes[i] = new LDAPMod;

      attr = (char *) newEntry.imp.list(i);
      attributes[i]->mod_type = attr;

      if ((imp.listExists(attr) && oldEntry.imp.listExists(attr)) || 
          ((strcmp(attr, "passwordExpirationTime") == 0) || (strcmp(attr, "passwordexpirationtime") == 0)))
      {
         vals  = getAttributes(attr);
         attributes[i]->mod_values = vals;
         attributes[i]->mod_op = LDAP_MOD_REPLACE;
      }
      else if (imp.listExists(attr) && !oldEntry.imp.listExists(attr))
      {
         vals = getAttributes(attr);
         attributes[i]->mod_values = vals;
         attributes[i]->mod_op = LDAP_MOD_ADD;
      }
      else
      {
         attributes[i]->mod_values = NULL;
         attributes[i]->mod_op = LDAP_MOD_DELETE;
      }
   }

   attributes[i] = NULL;

   err = ldap_modify_s (ld, entryDN, attributes);

   ldap_get_lderrno(ld, &matched, NULL);


   // Free the damn thing
   for (i = 0; i < numAttributes; i++)
   {
      freeAttributes (attributes[i]->mod_values);
      delete attributes[i];
   }

   delete [] attributes;

   if (err == OKAY)
   {
      _entryDN = entryDN;
      _numEntries = 1;
   }

   return err;
}

LdapError
LdapEntry::drop (const char *entryDN) const
{
   LDAP *ld = (LDAP *) _ldap->_ldapHandle;

   return ldap_delete_s (ld, entryDN);
}

LdapError
LdapEntry::dropAll(const char *entryDN)
{
   LdapError err;
   LdapEntry childEntry(_ldap);

   err = childEntry.retrieve(OBJECT_CLASS_FILTER, LDAP_SCOPE_ONELEVEL, entryDN);

   if (err == NOT_FOUND)
   {
      // OK to delete this guy
      err = drop(entryDN);
   }
   else
   {
      int i = 0, numEntries = childEntry.numEntries();
      while (err == OKAY && i < numEntries)
      {
         err = dropAll(childEntry.entryDN());
         childEntry.next();
         i++;
      }
      if (err == OKAY)
      {
        err = drop(entryDN);
      }
   }
   return err;
}


void
LdapEntry::printEntry(const char *fileName) const
{
   imp.write(fileName);
}

//
// CLASS: LdifEntry
//

Bool
LdifEntry::isBlank(const char *buf)
{
   Bool ret = True;

   while (*buf != 0)
   {
      if (!i18n_isspace(*buf))
      {
         ret = False;
         break;
      }
      buf ++;
   }
   return ret;
}

void
LdifEntry::stripBlank(char *buf, Bool bothEnds)
{
   char *s = buf;
   int newlen = strlen(buf);

   while (i18n_isspace(*s))
   {
      ++s;
   }

   if (s > buf)
   {
      newlen -= (s - buf);
      memcpy(buf, s, newlen);
      buf[newlen] = '\0';
   }

   s = buf + newlen - 1;
   if (bothEnds)
   {
      while (i18n_isspace(*s))
      {
         --s;
      }
      *(s + 1) = '\0';
   }
}

LdifEntry::LdifEntry()
  :_ldif(0)
{
}

LdifEntry::LdifEntry(const char *fileName)
  :_ldif(0)
{
   if (fileName == NULL)
      return;

   _ldif = fopen(fileName,"r");
}

LdifEntry::~LdifEntry()
{
   if (_ldif)
   {
      fclose(_ldif);
   }
}

int
LdifEntry::read(const char *fileName)
{
   if (fileName == NULL)
   {
      return -1;
   }

   if (_ldif)
   {
      fclose(_ldif);
   }
   _ldif = fopen(fileName, "r");
   if (_ldif == NULL)
   {
      return -1;
   }
   return (nextEntry());
}

int
LdifEntry::nextEntry()
{
   Bool gotEntry = False;
   int bufsiz=BUFSIZ;
   char *buf=NULL;
   NSString name;
   NSString value;

   clear();

   if (_ldif == NULL)
      return -1;

   buf = (char *) malloc(bufsiz);
   if (buf == NULL)
   {
      return -1;
   }

   while ( fgets( buf, bufsiz, _ldif) )
   {
      while ( strlen( buf ) == bufsiz - 1 && buf[bufsiz-2] != '\n' )
      {
         /* line longer than bufsiz */
         buf = (char *)realloc( buf, bufsiz * 2 );
         fgets ( buf + bufsiz - 1, bufsiz + 1, _ldif );
         bufsiz *= 2;
      }

      /* Skip newline */
      buf[strlen(buf)-1] = 0;

      if ( buf[0] == '#' )
      {
         // Comments - skip
         continue;
      }
      else if (isBlank(buf) == True)
      {
         // End of an entry
         if (gotEntry)
         {
            break;
         }
         // else just skip those blank lines
      }
      else if (buf[0] == ' ')
      {
         // A continuation line
         stripBlank(buf, False);

         if (name != (char *) NULL)
         {
            value = value + buf;
         }
         // else just skip those invalid lines
      }
      else
      {
         char *p;

         // Must be a valid attribute/value line
         if (name != (char *) NULL)
         {
            // Add the last attribute/value pair
            add(name, value);
            name = (char *) NULL;
            value = (char *) NULL;
         }

         p = strchr(buf, ':');
         if (p == NULL)
         {
            continue;
         }
         else
         {
            *p++ = NULL;
            gotEntry = True;
            stripBlank(buf, True);
            stripBlank(p, False);
            name = buf;
            value = p;
         }
      }
   }

   if (name != (char *) NULL)
   {
      add(name, value);
   }

   if (buf)
      free(buf);

   if (gotEntry == False)
      return -1;
   else
      return 0;
}

/////////////////////////////////////////////////////////////////////////////
/// The Great Divide starts here
/////////////////////////////////////////////////////////////////////////////

extern "C"
{

static int LDAP_CALL LDAP_CALLBACK
rebind_proc (LDAP *ld, char **dnp, char **passwdp,
                   int *authmethodp, int freeit, void *arg)
{
  if (freeit == 0) {
    Ldap *ldap = (Ldap *)arg;
    *dnp = (char *) ldap->rebindDN();
    *passwdp = (char *) ldap->rebindPwd();
    *authmethodp = LDAP_AUTH_SIMPLE;
  }

  return LDAP_SUCCESS;
}

NSAPI_EXTERN LdapErrorCode createLdap (Ldap **ldap,
                                   const char *ldapURL,
                                   const char *ldapUser,
                                   const char *ldapPwd,
                                   const char *binddn,
                                   const char *bindpwd)
{
   return setupCreateLdap (ldap, ldapURL, ldapUser, ldapPwd, binddn, bindpwd);
}

NSAPI_EXTERN const char *userDN (const Ldap *ldap)
{
   return setupGetUserDN(ldap);
}

NSAPI_EXTERN const char *ldapServerVersion (const Ldap *ldap)
{
   return setupGetServerVersion(ldap);
}

NSAPI_EXTERN void destroyLdap(Ldap *ldap)
{
   setupDestroyLdap(ldap);
}

NSAPI_EXTERN LdapEntry *createLdapEntry (const Ldap *ldap)
{
   return setupCreateLdapEntry(ldap);
}


NSAPI_EXTERN void destroyLdapEntry(LdapEntry *ldapEntry)
{
   setupDestroyLdapEntry(ldapEntry);
}

NSAPI_EXTERN const char *entryDN (const LdapEntry *ldapEntry)
{
   return setupGetEntryDN (ldapEntry);
}

NSAPI_EXTERN Bool entryExists(LdapEntry *ldapEntry, const char *entryDN)
{
   return setupEntryExists(ldapEntry, entryDN);
}

NSAPI_EXTERN LdapErrorCode retrieveLdapEntry(LdapEntry *ldapEntry, const char *entryDN)
{
   return setupRetrieveLdapEntry(ldapEntry, entryDN);
}

NSAPI_EXTERN LdapErrorCode retrieveLdapEntries(LdapEntry *ldapEntry,
      const char *criteria, int scope, const char *baseDN)
{
   return setupRetrieveLdapEntries(ldapEntry,
                       criteria, scope, baseDN);
}

NSAPI_EXTERN LdapErrorCode nextLdapEntry(LdapEntry *ldapEntry)
{
   return setupNextLdapEntry(ldapEntry);
}

NSAPI_EXTERN char **getLdapAttributeNames(const LdapEntry *ldapEntry)
{
   return setupGetLdapAttributeNames(ldapEntry);
}

NSAPI_EXTERN void freeLdapAttributeNames(LdapEntry *ldapEntry, char **attrs)
{
   setupFreeLdapAttributeNames(ldapEntry, attrs);
}

NSAPI_EXTERN char **getLdapAttributes(const LdapEntry *ldapEntry, const char *name)
{
    return setupGetLdapAttributes(ldapEntry, name);
}

NSAPI_EXTERN void freeLdapAttributes(LdapEntry *ldapEntry, char **attrs)
{
   setupFreeLdapAttributes(ldapEntry, attrs);
}

NSAPI_EXTERN void setLdapAttribute(LdapEntry *ldapEntry, const char *name, const char *value)
{
   setupSetLdapAttribute(ldapEntry, name, value);
}

NSAPI_EXTERN void addLdapAttribute (LdapEntry *ldapEntry, const char *name, const char *value)
{
   setupAddLdapAttribute (ldapEntry, name, value);
}

NSAPI_EXTERN void addLdapAttributes(LdapEntry *ldapEntry, const char *name, const char **values)
{
   setupAddLdapAttributes(ldapEntry, name, values);
}

NSAPI_EXTERN LdapErrorCode renameLdapEntry(LdapEntry *ldapEntry, const char *entryDN, const char *rdn)
{
   return setupRenameLdapEntry(ldapEntry, entryDN, rdn);
}
NSAPI_EXTERN LdapErrorCode replaceLdapEntry(LdapEntry *ldapEntry, const char *entryDN)
{
   return setupReplaceLdapEntry(ldapEntry, entryDN);
}
NSAPI_EXTERN LdapErrorCode insertLdapEntry(LdapEntry *ldapEntry, const char *entryDN)
{
   return setupInsertLdapEntry(ldapEntry, entryDN);
}
NSAPI_EXTERN LdapErrorCode updateLdapEntry(LdapEntry *ldapEntry, const char *entryDN)
{
   return setupUpdateLdapEntry(ldapEntry, entryDN);
}
NSAPI_EXTERN LdapErrorCode dropLdapEntry (Ldap *ldap, const char *entryDN)
{
   return setupDropLdapEntry (ldap, entryDN);
}
NSAPI_EXTERN LdapErrorCode dropAllLdapEntries (Ldap *ldap, const char *entryDN)
{
   return setupDropAllLdapEntries (ldap, entryDN);
}
NSAPI_EXTERN void printLdapEntry (const LdapEntry *ldapEntry, const char *fn)
{
   setupPrintLdapEntry(ldapEntry, fn);
}
/*********************************************************************
**
** FUNCTION:    verifyLdapUrl
** DESCRIPTION:
**   Verify that an LDAP URL is valid
**
** INPUTS:      url  - the ldap url.
** OUTPUTS:     none
** RETURN:
**   Refer to error codes defined in ldapu.h
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
LdapErrorCode
verifyLdapUrl(const char *url, const char *bindDN, const char *bindPwd)
{
   return setupVerifyLdapUrl(url, bindDN, bindPwd);
}

/*********************************************************************
**
** FUNCTION:    authLdapUser
** DESCRIPTION:
**   One-shot authentication of an LDAP user.
**
** INPUTS:
**   url  - the ldap url.
**   ldapUser - the user
**   ldapPwd  - the password
**   binddn, bindpwd - optional DN and Password to bind as
*
** OUTPUTS:     none

**   0 - Good
**   Other - failed (refer to ldapu.h)
**
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
LdapErrorCode
authLdapUser(const char *url,
             const char *ldapUser, const char *ldapPwd,
             const char *binddn, const char *bindpw)
{
   return setupAuthLdapUser(url,ldapUser, ldapPwd,binddn, bindpw);
}

NSAPI_EXTERN LdapErrorCode
createSIE(LdapEntry *sieEntry, LdapEntry *appEntry, const char *hostName,
          const char *installedLocation, const char *adminDomain,
          const char *adminBrand)
{
   return setupCreateSIE(sieEntry,appEntry,hostName,installedLocation,adminDomain,adminBrand);
}

NSAPI_EXTERN LdapErrorCode
updateISIEVersionInfo (Ldap *ldap, LdapEntry *appEntry,
                       const char *productName, const char *hostName,
                       const char *installedLocation, const char *adminDomain)
{
   return setupUpdateISIEVersionInfo (ldap, appEntry,
                                      productName, hostName,
                                      installedLocation, adminDomain);
}
NSAPI_EXTERN LdapErrorCode
removeSIE (Ldap *ldap, const char *sieDN, Bool deleteAll)
{
   return setupRemoveSIE (ldap, sieDN,deleteAll);
}
NSAPI_EXTERN LdapErrorCode
removeISIE(Ldap *ldap, const char *isieDN)
{
   return setupRemoveISIE(ldap, isieDN);
}
NSAPI_EXTERN LdapErrorCode
createACIForConfigEntry (LdapEntry *configEntry, const char *sieDN)
{
   return setupCreateACIForConfigEntry (configEntry, sieDN);
}
NSAPI_EXTERN LdapErrorCode
insertLdifEntries (const Ldap *ldap, const char *subTreeDN, const char *ldifFile, const char *adminSieDN)
{
   return setupUpdateLdifEntries(ldap, subTreeDN, ldifFile, adminSieDN);
}
NSAPI_EXTERN LdapErrorCode
checkAdminDomain(const Ldap *ldap, const char *adminDomain)
{
   return setupCheckAdminDomain(ldap, adminDomain);
}
/*********************************************************************
**
** FUNCTION:    isSubTreeWritable
** DESCRIPTION:
**
** INPUTS:      none
** OUTPUTS:     none
** RETURN:
**   True/False
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      This Ldap object must have been authenticated
** MEMORY:
**********************************************************************
*/
NSAPI_EXTERN LdapErrorCode
isSubTreeWritable(const Ldap *ldap, const char *subTreeDN)
{
   return setupIsSubTreeWritable(ldap, subTreeDN);
}

NSAPI_EXTERN char *
guessAdminDomainName(const char *ldapURL, const char *ldapUser, const char *ldapPwd)
{
   return setupGuessAdminDomainName(ldapURL, ldapUser, ldapPwd);
}
NSAPI_EXTERN LdapErrorCode
addGlobalUserDirectory (const Ldap *ldap, const char *domain, const char *userDirectoryUrl, const char *bindDN, const char *bindPwd)
{
   return setupAddGlobalUserDirectory (ldap, domain, userDirectoryUrl, bindDN, bindPwd);
}
NSAPI_EXTERN LdapErrorCode
getGlobalUserDirectory(const Ldap *ldap, const char *domain,
   char **url, char **bindDN, char **bindPwd)
{
   return setupGetGlobalUserDirectory(ldap, domain,url, bindDN, bindPwd);
}

NSAPI_EXTERN LdapErrorCode
getUserDirectory(const Ldap *ldap, const char *sieDN,
   char **url, char **bindDN, char **bindPwd)
{
   return setupGetUserDirectory(ldap, sieDN, url, bindDN, bindPwd);
}
NSAPI_EXTERN LdapErrorCode
getLocalUserDirectory(const Ldap *ldap, const char *domain, const char *hostName, const char * serverRoot, char **url, char **bindDN, char **bindPwd)
{
   return setupGetLocalUserDirectory(ldap, domain, hostName, serverRoot, url, bindDN, bindPwd);
}
} /* End extern "C" */
