/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
**
** Contact:    Phong Vo (pvo@netscape.com)
**
** Name: prodinfo.c
**
** $Log: prodinfo.cpp,v $
** Revision 1.2  2006/03/23 15:59:10  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:29  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:27  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/07/27 01:04:49  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:13  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.5.4.3  2003/08/26 19:39:43  djani
** Bug 606829: check if filename is not NULL before performing strstr() on it.
**
** Revision 1.1.1.5.4.2  2002/04/23 23:05:07  djani
** Bug 604074: Now we don't change the mode to rw------- for slapd.inf
**
** Revision 1.1.1.5.4.1  2002/04/13 01:19:43  djani
** Bug 520937: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.5  2001/11/06 01:51:31  ahakim
** XXX AOL/NSCP Copyright XXX
**
** Revision 1.1.1.4  1999/03/06 02:12:39  cxue
** Take out code which converts backslash n to 0x0A
**
** Revision 1.1.1.2  1999/03/05 02:42:30  cxue
** Bug 34434. disable multiline parser on NT
**
** Revision 1.1.1.1  1999/03/04 23:28:29  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.12.6.2  1999/02/04 02:10:03  cxue
** nvpair reads multi-line string. multi-line is separated by \n
**
** Revision 1.1.2.12.6.1  1999/01/29 00:03:24  cxue
** bug 341008. doesn't allocate enough memory
**
** Revision 1.1.2.12  1998/07/17 02:24:11  pvo
** Eliminate some memory leaks
**
** Revision 1.1.2.11  1998/07/05 19:24:13  pvo
** 233676 - UTF8 conversion problem
**
** Revision 1.1.2.10  1998/06/27 02:55:25  pvo
** Name change
**
** Revision 1.1.2.9  1998/06/23 02:41:09  pvo
** Fixed UTF8 conversion bug
**
** Revision 1.1.2.8  1998/06/06 02:45:55  pvo
** Support NVPair utf8 conversion
**
** Revision 1.1.2.7  1998/06/05 03:38:05  pvo
** 118281 - additional code for UTF8 conversion
**
** Revision 1.1.2.6  1998/05/28 21:15:49  pvo
** 122386 - use i18n_isspace() rather than isspace()
**
** Revision 1.1.2.5  1998/02/26 01:39:34  gbeasley
** more sie integration
**
** Revision 1.1.2.4  1998/02/24 01:05:23  pvo
** Initial setup for integrating UX and NT SIE functions.
**
** Revision 1.1.2.3  1997/12/12 21:54:08  pvo
** Various bug fixes.
**
** Revision 1.1.2.2  1997/11/26 04:42:49  pvo
** Fixed up for cross-platform compilation.
**
** Revision 1.1.2.1  1997/11/26 02:12:30  pvo
** Support for name with embedded blanks.
**
** Revision 1.1.2.7  1997/11/15 04:59:25  pvo
** Fixed memory leak problems and redo + operator for NSString.
**
** Revision 1.1.2.6  1997/11/11 05:40:49  pvo
** Fixed bugs.
**
** Revision 1.1.2.5  1997/11/07 22:43:24  pvo
** New code to support uninstallation.
**
** Revision 1.1.2.4  1997/10/24 03:39:29  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.3  1997/10/10 04:24:46  pvo
** Added the setting of default ACI at netscapeRoot level.
**
** Revision 1.1.2.2  1997/10/06 23:43:26  pvo
** Cleaned up for NT compilation.
**
** Revision 1.1.2.1  1997/10/06 20:40:32  pvo
** Copied here from libadminutil to eliminate too much baggage due
** to dependencies.
**
**
***********************************************************************/

/***********************************************************************
** Includes
***********************************************************************/

extern "C" {
#include    <stdio.h>
#include        <string.h>
#ifndef XP_WIN32
#include        <sys/stat.h>
#include        <strings.h>
#endif
#include        <stdlib.h>
#include        <malloc.h>
#include        <ctype.h>
}

#ifdef XP_WIN32
#define strcasecmp _stricmp
#endif

#include        "global.h"
#include    "prodinfo.h"
#include        "utf8.h"

static void addValue (ADM_piTree *, const char *name, const char *value);
static void addBoolean (ADM_piTree *, const char *name, const char *value);
static void addToList (ADM_piTree *, const char *name, const char *value);
static ADM_piList *addList (ADM_piTree *, const char *name);
static ADM_piList *getList (const ADM_piTree *, const char *name);

/*
 * Return the index of the last non blank + 1
 * 0 if everything is blank
 */
static int
remove_trailing_blanks(char *buf)
{
   int i;

   i = strlen( buf );
   i--;
   while ((i >= 0) && (i18n_isspace(buf[i])))
   {
      i--;
   }
   buf[++i] = '\0';

   return i;
}

ADM_piTree *
ADM_pi_init(const char *file)
{
   return ADM_pi_init(file, 0);
}

ADM_piTree *
ADM_pi_init( const char *file, int format )
{
  FILE       *f = NULL;
  size_t     bufsiz = BUFSIZ;
  char       *buf = NULL;
  int        i;
  ADM_piTree *tree = NULL;
  ADM_piTree *bigTree = NULL;
  char       *name = NULL;
  size_t     namesiz = BUFSIZ;
  int        srcLen;

  if (file)
  {
     f = fopen(file, "r");
     if ( !f ) {
        file = NULL;
     }
  }

  bigTree = (ADM_piTree *) malloc( sizeof(ADM_piTree) );
  if (NULL == bigTree) return NULL;
  memset(bigTree, 0, sizeof (ADM_piTree));

  bigTree->format = format;

  if (!file)
     return bigTree;

  tree = bigTree;

  buf = (char *)malloc(bufsiz);
  if (buf == NULL) return tree;

  name = (char *)malloc(namesiz);
  if (name == NULL) return tree;

  while ( fgets( buf, bufsiz, f ) ) {
    while ( strlen( buf ) == bufsiz - 1 && buf[bufsiz-2] != '\n' ) {
      /* line longer than bufsiz */
      buf = (char *)realloc( buf, bufsiz * 2 );
      if (buf == NULL) return tree;
      fgets( buf + bufsiz - 1, bufsiz + 1, f );
      bufsiz *= 2;
    }
    if ( buf[0] == '#' || buf[0] == '\n' ) {    /* comment or empty */
      continue;
    }
    if ( buf[0] == '[' ) {    /* install date */
      int end = 0;
      for ( i = strlen( buf ) - 1 ; i-- ; ) {
         if ( buf[i] == ']' ) {
             buf[i] = '\0';    /* clean up date */
             break;
          }
      }
      /* need to know if there are at least 5 tokens in buf - end will only be set
         if there are at least 5 whitespace delimited tokens in (buf+1)
      */
      sscanf (buf + 1, "%*s %*s %*s %*s %*s%n", &end);

      if (end)
      {
         if ( tree->when ) {
            break;    /* this is a previous date, stop reading */
         }
         tree->when = strdup( buf + 1 );
         continue;
      }

      /*
       * New section
       */
      {
      char *bp = strtok(buf, "[ ");
      int newseclen = strlen(bp) + 1;
      if (namesiz < newseclen) {
        namesiz = newseclen;
        name = (char *)realloc(name, newseclen);
        if (name == NULL) return tree;
      }
      strncpy(name, bp, namesiz);
      }
      /*
       * Section [General] is treated as the mother of section
       */
      if (!strcasecmp(name, "(latest-installation-date)"))
      {
         ; /* Skip */
      }
      else if (!strcasecmp(name, "General"))
      {
         /* Escape back to bigTree */
         tree = bigTree;
      }
      else
      {
         tree = ADM_pi_addSection (bigTree, name);
         tree->format = bigTree->format;
      }

      /* From now on, all operations are to/from the section */
    }
    else {    /* name value pair */
      int    j;
      int    type;
      char   *tok;

      /* clean trailing whitespace */
      srcLen = remove_trailing_blanks( buf);
      if (srcLen == 0) /* blank line */
      {
         continue;
      }

      /* trim opening whitespace */
      for ( i = 0 ; i < srcLen && i18n_isspace( buf[i] ) ; ++i )
      ;
      /* get name */
      if (tree->format == 1)
      {
         for (j = 0;
            i < srcLen && buf[i] != '=';
            i++, j++)
         {
            if (j == namesiz) {
               namesiz *= 2;
               name = (char *)realloc(name, namesiz);
               if (name == NULL) return tree;
            }
            name[j] = buf[i];
         }
      }
      else if (tree->format == 2)
      {
         for (j = 0;
            i < srcLen && buf[i] != ':';
            i++, j++)
         {
            if (j == namesiz) {
               namesiz *= 2;
               name = (char *)realloc(name, namesiz);
               if (name == NULL) return tree;
            }
            name[j] = buf[i];
         }
      }
      else /* Assume format == 0 */
      {
         for ( j = 0 ;
               i < srcLen && ! i18n_isspace( buf[i] ) && buf[i] != '=' &&
               buf[i] != '?' && buf[i] != ':' ;
               ++i, ++j )
         {
             if (j == namesiz) {
                namesiz *= 2;
                name = (char *)realloc(name, namesiz);
                if (name == NULL) return tree;
             }
             name[j] = buf[i];
         }
      }
      name[j] = '\0';    /* terminate name */

      if (remove_trailing_blanks(name) == 0) /* Blank name */
         continue;

      /* get variable type */
      while ( i < srcLen && i18n_isspace( buf[i] ) ) {
         ++i;
      }
      switch ( buf[i] ) {
      case ':':    /* list */
      case '?':    /* single boolean */
      case '=':    /* name value pair */
         type = buf[i];
         ++i;    /* skip past specifier */
         break;
      default:    /* magnus.conf style name value pair */
         type = '=';    /* fake an '=' */
          break;
      }
      /* skip whitespace to get to value */
      while ( i < srcLen && i18n_isspace( buf[i] ) ) {
         ++i;
      }

      /* PVO - need to skip if this is an empty value/pair */

      switch ( type ) {
      case '=':    /* simple name value pair */
      case ':':
         addValue (tree, name, buf + i);
         break;
      case '?':    /* simple name boolean pair */
         addBoolean (tree, name, buf + i);
         break;
      case '@':    /* list variable PVO */
        for (tok = strtok(buf + i,", "); tok; tok = strtok(NULL,", "))
        {
           addToList (tree, name, tok);
        }

         break;
      }
    }
  }
  fclose( f );
  if ( buf ) free( buf );
  if ( name ) free( name );
  return bigTree;
}

void
ADM_pi_done( ADM_piTree *tree )
{
  int    i;
  int    j;

  if (!tree)
      return;

  if ( tree->when ) {
    free( tree->when );
  }
  for ( i = 0 ; i < tree->numVars ; ++i ) {
    free( tree->vars[i] );
    free( tree->vals[i] );
  }
  if ( tree->numVars ) {
    free( tree->vars );
    free( tree->vals );
  }
  for ( i = 0 ; i < tree->numBools ; ++i ) {
    free( tree->boolVars[i] );
  }
  if ( tree->numBools ) {
    free( tree->boolVars );
    free( tree->boolVals );
  }
  for ( i = 0 ; i < tree->numLists ; ++i ) {
    free( tree->lists[i].var );
    /* Not cheated anymore, free all allocated blocks */
    for (j = 0 ; j < tree->lists[i].numEls; j++)
       free( tree->lists[i].els[j] );    
    free( tree->lists[i].els );
  }
  if ( tree->numLists ) {
    free( tree->lists );
  }

  for ( i = 0; i < tree->numSects; ++i) {
     ADM_pi_done (tree->sects[i].sect);
     free(tree->sects[i].name);
  }
  if (tree->numSects) {
    free (tree->sects);
  }

  free( tree );
}

char *
ADM_pi_value( const ADM_piTree *tree, const char *var )
{
  int    i;

  for ( i = 0 ; i < tree->numVars ; ++i ) {
    if ( strcasecmp( var, tree->vars[i] ) == 0 ) {
      return tree->vals[i];
    }
  }
  return NULL;
}

char *
ADM_pi_when( const ADM_piTree *tree )
{
  return tree->when;
}

int
ADM_pi_canDo( const ADM_piTree *tree, const char *capability, const char *instance )
{
  int    j;
  ADM_piList *list;

  if ( instance )
  {
     /* go for the array */
     list = getList(tree, capability);
     if (list != NULL)
     {
        for ( j = 0 ; j < list->numEls ; ++j )
        {
           if ( strcasecmp( instance, list->els[j] ) == 0 )
           {
           return 1;
       }
        }
    return 0;    /* There is the capability, but no instance */
     }
     return -1;    /* not in the list */
  }
  else
  {    /* single boolean var */
    for ( j = 0 ; j < tree->numBools ; ++j ) {
      if ( strcasecmp( capability, tree->boolVars[j] ) == 0 ) {
    return tree->boolVals[j];
      }
    }
    return -1;    /* not in the list */
  }
}

ADM_piTree *
ADM_pi_section (const ADM_piTree *tree, const char *name)
{
   ADM_piSect *section;
   int i;

   i = tree->numSects;

   while (i-- > 0)
   {
      section = &tree->sects[i];
      if (!strcasecmp(section->name, name))
      {
         return (section->sect);
      }
   }
   return NULL;
}

ADM_piTree *
ADM_pi_nsection (const ADM_piTree *tree, int which)
{
   if (which < 0 || which > (tree->numSects-1))
   {
      return NULL;
   }
   else
   {
      return tree->sects[which].sect;
   }
}

char *
ADM_pi_sectName (const ADM_piTree *tree, int which)
{
   if (which < 0 || which > (tree->numSects-1))
   {
      return NULL;
   }
   else
   {
      return tree->sects[which].name;
   }
}

void
ADM_pi_rename(ADM_piTree *tree, const char *old, const char *newName)
{
  int i;

  if (newName == NULL || old == NULL)
  {
      return;
  }

  for ( i = 0 ; i < tree->numVars ; ++i )
  {
    if ( strcasecmp( old, tree->vars[i] ) == 0 )
    {
       free (tree->vars[i]);
       tree->vars[i] = strdup(newName);
       return;
    }
  }

  for ( i = 0; i < tree->numBools ; ++i)
  {
     if ( strcasecmp(old, tree->boolVars[i]) == 0)
     {
         free(tree->boolVars[i]);
         tree->boolVars[i] = strdup(newName);
         return ;
     }
  }

  for ( i = 0; i < tree->numLists ; ++i)
  {
     if ( strcasecmp(old, tree->lists[i].var) == 0)
     {
        free(tree->lists[i].var);
        tree->lists[i].var = strdup(newName);
     }
  }

  for ( i = 0; i < tree->numSects; ++i)
  {
     if ( strcasecmp(old, tree->sects[i].name) == 0)
     {
        free(tree->sects[i].name);
        tree->sects[i].name = strdup(newName);
     }
  }
}

void
ADM_pi_timeStamp (ADM_piTree *tree, const char *ts)
{
   if (tree->when)
      free(tree->when);

   tree->when = strdup(ts);
}

/***********************************************************************
** FUNCTION:    ADM_pi_list
** DESCRIPTION:
**   find a a list called name, and return it
** INPUTS:    tree -    the data structure
**        name -    the name of the list looked for
** OUTPUTS:    list -    pointer to array of char *
** RETURN:    -1 if no list by that name, else count of list
** SIDE EFFECTS:
**    None
** RESTRICTIONS:
**    None
** MEMORY:    returned structure is meant to be treated as readonly
***********************************************************************/

int
ADM_pi_list( const ADM_piTree *tree, const char *name, char ***elements )
{
  ADM_piList *list;

  list = getList(tree, name);
  if (list != NULL)
  {
     *elements = list->els;
     return list->numEls;
  }
  return -1;
}

/***********************************************************************
** FUNCTION:    ADM_pi_2List
** DESCRIPTION:
**   Convert a regular name/value pair to internal list format
** INPUTS:      tree -  the data structure
**              name -  the name of the key
** OUTPUTS:     list -  pointer to array of char *
** RETURN:
** SIDE EFFECTS:
**      The original key/value pair is deleted
** RESTRICTIONS:
**      None
** MEMORY:
***********************************************************************/
void
ADM_pi_2List( ADM_piTree *tree, const char *name)
{
   char *tok;
   char *list = ADM_pi_value (tree, name);

   if (list)
   {
      for (tok = strtok(list,", "); tok; tok = strtok(NULL,", "))
      {
         addToList (tree, name, tok);
      }

      ADM_pi_delValue(tree, name);

   }

}


/***********************************************************************
** FUNCTION:    ADM_pi_setValue
** DESCRIPTION:
**   Insert the name/value pair into the PI tree, replacing the value
**   of an existing name if exists.
** INPUTS:      tree -  the data structure
**              var  -  the name of the name/value pair to look for
**              val  -  the value to be inserted/replaced
** OUTPUTS:     None
** RETURN:      None
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
**
***********************************************************************/

void
ADM_pi_setValue (ADM_piTree *tree, const char *var, const char *val)
{
   int i;
   int found = 0;

   if (tree == NULL || var == NULL || val == NULL)
     return;

   for ( i = 0; i < tree->numVars; ++i)
   {
      if ( strcasecmp(var, tree->vars[i]) == 0)
      {
         found = 1;
         break;
      }
   }

   if (!found)
   {
      addValue (tree, var, val);
   }
   else
   {
      if (val != tree->vals[i])
      {
         free (tree->vals[i]);
         tree->vals[i] = strdup (val);
      }
   }
}

void
ADM_pi_setBoolean (ADM_piTree *tree, const char *capability, const char *val)
{
   int i;
   int found = 0;

   if (tree == NULL || capability == NULL || val == NULL ||
       (strcasecmp(val,"true") && strcasecmp(val,"false")))
      return;

   for ( i = 0; i < tree->numBools ; ++i)
   {
      if ( strcasecmp( capability, tree->boolVars[i]) == 0)
      {
         found = 1;
         break;
      }
   }
   if (!found)
   {
      addBoolean (tree, capability, val);
   }
   else
   {
      tree->boolVals[i] = strcasecmp(val, "true") == 0;
   }
}

void
ADM_pi_setCapability (ADM_piTree *tree, const char *capability, const char *instance)
{
   if (tree == NULL || capability == NULL || instance == NULL)
     return;

   addToList (tree, capability , instance);

   return;
}

void
ADM_pi_delCapability (ADM_piTree *tree, const char *capability, const char *instance)
{
   int j, k;
   ADM_piList *list;

   if (tree == NULL || capability == NULL || instance == NULL)
   {
      return;
   }

   list = getList(tree, capability);

   if (list)
   {
      for ( j = 0 ; j < list->numEls ; ++j )
      {
         if ( strcasecmp( instance, list->els[j] ) == 0 )
         {
            /* Found instance */
            free(list->els[j]);
            for ( k = j + 1; k < list->numEls; k++)
            {
               list->els[k-1] = list->els[k];
            }
            list->numEls--;
            break;
         }
      }
   }
}


void
ADM_pi_delValue (ADM_piTree *tree, const char *var)
{
   int i;
   int j;

   if (tree == NULL || var == NULL)
     return;

   for ( i = 0; i < tree->numVars; ++i)
   {
      if ( strcasecmp(var, tree->vars[i]) == 0)
      {
         free(tree->vars[i]);
         free(tree->vals[i]);
         for (j = i + 1; j < tree->numVars; ++j)
         {
            tree->vars[j-1] = tree->vars[j];
            tree->vals[j-1] = tree->vals[j];
            tree->vars[j] = tree->vals[j] = NULL;
         }
         --tree->numVars;
         break;
      }
   }
}

void
ADM_pi_delList (ADM_piTree *tree, const char *var)
{
   int i, j;

   if (tree == NULL || var == NULL)
      return;

   for ( i = 0; i < tree->numLists; i++)
   {
      if ( strcasecmp( var, tree->lists[i].var ) == 0 )
      {
         free(tree->lists[i].var);
         for (j = 0 ; j < tree->lists[i].numEls; j++)
         {
            free(tree->lists[i].els[j]);
         }
         free(tree->lists[i].els);

         for (j = i + 1; j < tree->numLists; j++)
         {
            tree->lists[j-1].numEls = tree->lists[j].numEls;
            tree->lists[j-1].var = tree->lists[j].var;
            tree->lists[j-1].els = tree->lists[j].els;
         }
         tree->numLists--;
         break;
      }
   }
}

void
ADM_pi_delSection (ADM_piTree *tree, const char *section)
{
   int i, j;

   for (i = 0 ; i < tree->numSects; i++)
   {
      if (!strcasecmp(tree->sects[i].name, section))
      {
         free(tree->sects[i].name);
         ADM_pi_done(tree->sects[i].sect);

         for (j = i + 1; j < tree->numSects; j++)
         {
            tree->sects[j-1].name = tree->sects[j].name;
            tree->sects[j-1].sect = tree->sects[j].sect;
         }
         tree->numSects--;
         break;
      }
   }
}

static void
addValue(ADM_piTree *tree, const char *var, const char *val)
{
/*   if (*var == '\0' || *val == '\0') PVO */
   if (*var == '\0')
      return;

   if (tree->numVars)
   {
      tree->vars = (char **) realloc( tree->vars,
                                      sizeof (char *) *
                                      ( tree->numVars + 1) );
      tree->vals = (char **) realloc( tree->vals,
                                      sizeof (char *) *
                                      ( tree->numVars + 1) );
   }
   else
   {
      tree->vars = (char **) malloc ( sizeof (char *));
      tree->vals = (char **) malloc ( sizeof (char *));
   }
   tree->vars[tree->numVars] = strdup( var );
   tree->vals[tree->numVars] = strdup (val);
   ++tree->numVars;
}

static void
addBoolean (ADM_piTree *tree, const char *var, const char *val)
{
/*     if (*var == '\0' || * val == '\0') PVO */
   if (*var == '\0')
        return;

     if ( tree->numBools ) {
       tree->boolVars = (char **) realloc( tree->boolVars,
                                           sizeof( char * ) *
                                           ( tree->numBools + 1 ) );

       tree->boolVals = (char *) realloc( tree->boolVals,
                                          sizeof( char ) *
                                           ( tree->numBools + 1 ) );
     } else {
       tree->boolVars = (char **) malloc( sizeof( char * ) );
       tree->boolVals = (char *) malloc( sizeof( char ) );
     }
     tree->boolVars[tree->numBools] = strdup( var );
     tree->boolVals[tree->numBools] = strcasecmp( val, "true" ) == 0;
     ++tree->numBools;
}

static ADM_piList *
getList (const ADM_piTree *tree, const char *name)
{
    int i;
    for ( i = 0 ; i < tree->numLists ; ++i ) {
      if ( strcasecmp( name, tree->lists[i].var ) == 0 )
      {
         return &(tree->lists[i]);
      }
    }
    return NULL;
}

/*
 * Create a list named 'name'. If it already exists, return the list
 */
static ADM_piList *
addList (ADM_piTree *tree, const char *name)
{
   ADM_piList *list;

   if (name == NULL || *name == '\0')
   {
      return NULL;
   }

   list = getList(tree, name);

   if (list != NULL)
   {
      return list;
   }

   if ( tree->numLists ) {
      tree->lists = (ADM_piList *) realloc( tree->lists,
                                            sizeof( ADM_piList ) *
                                            ( tree->numLists + 1 ) );
   } else {
      tree->lists = (ADM_piList *) malloc( sizeof( ADM_piList ) );
   }

   memset (&tree->lists[tree->numLists], 0, sizeof(ADM_piList));
   tree->lists[tree->numLists].var = strdup( name );

   ++tree->numLists;

   return &(tree->lists[tree->numLists-1]);
}

static void
addToList (ADM_piTree *tree, const char *name, const char *instance)
{
   int i;
   ADM_piList *list = addList(tree, name);

   if (list->numEls)
   {
      /* See if instance already exist */
      for (i=0; i < list->numEls; i++)
      {
         if (!strcasecmp(list->els[i] , instance))
         {
            return;
         }
      }

      list->els = (char **) realloc (list->els,
                                     sizeof (char *) *
                                      (list->numEls + 1));
   }
   else
   {
      list->els = (char **) malloc (sizeof (char *));
   }

   list->els[list->numEls] = strdup(instance);

   ++list->numEls;
}

static void
ADM_pi_writeTree (const ADM_piTree *tree, FILE *f, int format)
{
   int i, j;

   /* Writing the whole tree to disk */
   if (tree->when)
   {
      fprintf(f, "[%s]\n", tree->when);
   }

   for (i = 0 ; i < tree->numVars; i++)
   {
      if (format == 1)
      {
         fprintf(f, "%s=   ", tree->vars[i]);
      }
      else if (format == 2)
      {
         fprintf(f, "%s:   ", tree->vars[i]);
      }
      else
      {
         fprintf(f, "%s   ", tree->vars[i]);
      }

      if (tree->vals[i])
      {
         fprintf(f, "%s", tree->vals[i]);
      }
      fprintf(f, "\n");
   }

   for (i = 0; i < tree->numLists; i++)
   {
      if (format != 1)
      {
         fprintf(f, "%s:   " , tree->lists[i].var);
      }
      else
      {
         fprintf(f, "%s=   ", tree->lists[i].var);
      }

      for (j = 0 ; j < tree->lists[i].numEls ; j++)
      {
         if (tree->lists[i].els[j])
         {
            if (j == 0)
               fprintf(f, "%s", tree->lists[i].els[j]);
            else
               fprintf(f, ",%s", tree->lists[i].els[j]);
         }
      }

      fprintf(f,"\n");

   }

   for (i = 0 ; i < tree->numBools; i++)
   {
      if (format == 1)
      {
         fprintf(f, "%s=   ", tree->boolVars[i]);
      }
      else if (format == 2)
      {
         fprintf(f, "%s:   ", tree->boolVars[i]);
      }
      else
      {
         fprintf(f, "%s?   ", tree->boolVars[i]);
      }

      if (tree->boolVals[i])
      {
         fprintf(f,"%s\n", "true");
      }
      else
      {
         fprintf(f,"%s\n", "false");
      }
   }

   for (i = 0 ; i < tree->numSects; i++)
   {
      fprintf(f, "\n%c%s%c\n", '[', tree->sects[i].name, ']');
      ADM_pi_writeTree(tree->sects[i].sect , f, format);
   }
}

void
ADM_pi_setFormat(ADM_piTree *tree, int which)
{
   tree->format = which;
}

int
ADM_pi_write (const ADM_piTree *tree, const char *file)
{
   FILE *f;

   if (file == NULL)
   {
      f = stdout;
   }
   else
   {
      f = fopen( file, "w" );
   }

   if (f == NULL)
       return -1;

   /* Bug 520937: Correct the file permission to -rw for the user */
   /* Bug 604074: We don't change the mode for slapd.inf */
#ifndef XP_WIN32
   if (file && ((strstr(file, "slapd.inf")) == NULL)) 
   {
       fchmod (fileno(f), S_IRUSR | S_IWUSR);
   }
#endif

   if (tree->numVars && tree->format == 1)
   {
      fprintf(f, "%s\n", "[General]");
   }

   ADM_pi_writeTree (tree, f, tree->format);

   fclose(f);

   return 0;

}

/*
 * Dest must be empty at the beginning
 */
void
ADM_pi_copy (ADM_piTree *dest, const ADM_piTree *src)
{
   int i;

   if (dest == NULL || src == NULL || dest == src)
   {
      return;
   }

   if (src->when)
   {
      dest->when = strdup(src->when);
   }

   if (dest->numVars = src->numVars)
   {
      dest->vars = (char **) malloc(sizeof (char *) * dest->numVars);
      dest->vals = (char **) malloc(sizeof (char *) * dest->numVars);
      for (i = 0; i < src->numVars; ++i)
      {
         dest->vars[i] = strdup(src->vars[i]);
         dest->vals[i] = strdup(src->vals[i]);
      }
   }

   if (dest->numBools = src->numBools)
   {
      dest->boolVars = (char **) malloc(sizeof (char *) * dest->numBools);
      dest->boolVals = (char *) malloc(sizeof (char *) * dest->numBools);
      for (i = 0; i < src->numBools; ++i)
      {
         dest->boolVars[i] = strdup(src->boolVars[i]);
         dest->boolVals[i] = src->boolVals[i];
      }
   }

   if (dest->numLists = src->numLists)
   {
      dest->lists = (ADM_piList *) malloc(sizeof (ADM_piList) * dest->numLists);
      for (i = 0; i < src->numLists; i++)
      {
          dest->lists[i].var = strdup(src->lists[i].var);
          if (dest->lists[i].numEls = src->lists[i].numEls)
          {
             int j;

             dest->lists[i].els =
              (char **) malloc(sizeof(char *) * src->lists[i].numEls);

             for (j=0; j < src->lists[i].numEls; j++)
             {
                dest->lists[i].els[j] = strdup(src->lists[i].els[j]);
             }
          }
      }
   }

   if (dest->numSects = src->numSects)
   {
      dest->sects = (ADM_piSect *) malloc(sizeof(ADM_piSect) * dest->numSects);

      for (i = 0; i < src->numSects; i++)
      {
         dest->sects[i].sect = ADM_pi_init(NULL);
         ADM_pi_copy(dest->sects[i].sect, src->sects[i].sect);
      }
   }
}

ADM_piTree *
ADM_pi_addSection (ADM_piTree *tree, const char *name)
{
   ADM_piSect *p;

   if (tree->numSects)
   {
      tree->sects = (ADM_piSect *) realloc (tree->sects,
                                            sizeof( ADM_piSect) *
                                            ( tree->numSects + 1 ));
   }
   else
   {
      tree->sects = (ADM_piSect *) malloc ( sizeof (ADM_piSect));
   }

   p = &tree->sects[tree->numSects];

   tree->numSects++;

   p->name = strdup(name);

   return (p->sect = ADM_pi_init(NULL));
}

void
ADM_pi_convertEncoding(ADM_piTree *tree, const char *name, int toUTF8)
{
  int i, j, k;
  char *target;

  for ( i = 0 ; i < tree->numVars ; ++i ) 
  {
     j = -1;

     if (name != NULL)
     {
        if (strcasecmp(name, tree->vars[i]) == 0)
        { 
           j = i;
        }
     }
     else
     {
        j = i;
     }
     if (j >= 0)
     {
        if (toUTF8 == 1) // toUTF8
        {
           target = localToUTF8(tree->vals[j]);
        }
        else
        {
           target = UTF8ToLocal(tree->vals[j]);
        }

        free(tree->vals[j]);
        tree->vals[j] = target;

        if (name)
        {
           return;
        }
     }
  }

  for ( i = 0; i < tree->numLists; ++i)
  {
     j = -1;

     if (name != NULL)
     {
        if (strcasecmp(name, tree->lists[i].var) == 0)
        {
           j = i;
        }
     }
     else
     {
        j = i;
     }

     if (j >= 0)
     {
         for (k = 0 ; k < tree->lists[j].numEls ; k++)
         {
            if (tree->lists[j].els[k])
            {
               if (toUTF8)
                  target = localToUTF8(tree->lists[j].els[k]);
               else
                  target = UTF8ToLocal(tree->lists[j].els[k]);

               free(tree->lists[j].els[k]);
               tree->lists[j].els[k] = target;

               if (name)
               {
                  return;
               }
            }
         }
      }
   }
   if (name == NULL)
   {
      for ( i = 0; i < tree->numSects; ++i) 
      {
         ADM_pi_convertEncoding (tree->sects[i].sect, name, toUTF8);
      }
   }
}
