/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/

extern "C" {
#if defined(AIX)
   #include <strings.h>
#endif
#include <string.h>
#include <ctype.h>
};

#include "ldapu.h"

extern "C"
{


NSAPI_EXTERN int setupVerifyAdminDomain (const char *ldapURL, const char * szAdminDomain)
{

   LdapError err;

   Ldap ldap(err, ldapURL, NULL, NULL, NULL, NULL);

   err = checkAdminDomain (&ldap, szAdminDomain);

   return err;   
}

NSAPI_EXTERN int setupVersionCompare(const char *ver1, const char *ver2)
{
   /*
    * assumed format: "#.#" or "#.##", followed by an optional
    * alphanumeric suffix that must begin with a letter, followed by
    * a number.
    *
    *   e.g.
    *
    *   3.0a1 < 3.0b2 < 3.0 < 4.0a1 < 4.0a2 < 4.0b3 < 4.0
    */

   int minor1 = 0, minor2 = 0;
   const char *p1;
   const char *p2;

   if (ver1 == NULL)
   {
      ver1 = "4.0";
   }

   if (ver2 == NULL)
   {
      ver2 = "4.0";
   }

   if (!isdigit(*ver1) || !isdigit(*ver2) ||
       ver1[1] != '.' || ver2[1] != '.' ||
       *ver1 != *ver2)
      return strcasecmp(ver1, ver2);

   p1 = ver1 + 2;
   if (isdigit(*p1))
   {
      minor1 = 10 * (*p1++ - '0');
      if (isdigit(*p1))
         minor1 += *p1++ - '0';
   }

   p2 = ver2 + 2;
   if (isdigit(*p2))
   {
      minor2 = 10 * (*p2++ - '0');
      if (isdigit(*p2))
         minor2 += *p2++ - '0';
   }

   if (minor1 < minor2)
      return -1;
   if (minor1 > minor2)
      return 1;
   if (strcasecmp(p1, p2) == 0)
      return 0;
   if (*p1 == '\0')   /* e.g. 3.0 is newer than 3.0b1 */
      return 1;
   if (*p2 == '\0')
      return -1;
   return strcasecmp(p1, p2);
}


#define NO_SECURITY "{none}"

NSAPI_EXTERN int setupSecurityCompare(const char *sec1, const char *sec2)
{
   sec1 = sec1 ? sec1 : NO_SECURITY;
   sec2 = sec2 ? sec2 : NO_SECURITY;

   return -1 * strcasecmp(sec1, sec2);
}


NSAPI_EXTERN int  setupRevisionCompare(const char *rev1, const char *rev2)
{
   rev1 = rev1 ? rev1 : "0";
   rev2 = rev2 ? rev2 : "0";
   return strcasecmp(rev1, rev2);
}

NSAPI_EXTERN int setupBuildCompare(const char *b1, const char *b2)
{
   /* b1: buildNumber of new version */
   /* b2: buildNumber of existing version */
  int y1 = 90;  // default 1990
  int y2 = 90;  // default 1990
  char *num1, *num2;
  char *p1, *p2;
  int iRet;

  if (b1 == NULL && b2 == NULL)
    return 0;
  if (b1 == NULL)
    return -1;
  if (b2 == NULL)
    return 1;

  num1 = strdup(b1);
  num2 = strdup(b2);

  /* get year from build number  */
  if (p1 = strchr(num1, '.'))
  {
    *p1 = '\0';
    y1 = atoi(num1);
    p1 ++;
  }
  else
  {
    p1 = (char *)"001.0002";
  }

  if (p2 = strchr(num2, '.'))
  {
    *p2 = '\0';
    y2 = atoi(num2);
    p2 ++;
  }
  else
  {
    p2 = (char *)"001.0001";
  }

  /*  Converts 2 digits year to 4 digits year.
      90 is divide line between 2000+ and 1900+ 
   */
  if (y1 < 90)
    y1 += 2000; 
  else if (y1 < 100)
    y1 += 1900;
 
  if (y2 < 90)
    y2 += 2000; 
  else if (y2 < 100)
    y2 += 1900;

  if (y1 > y2)
  {
    iRet = 1;
  }
  else if (y1 < y2)
  {
    iRet = -1;
  }
  else
  {
    /*  Compares days + time
     */
    iRet = strcasecmp(p1, p2);
  }

  free(num1);
  free(num2);

  return iRet;
}

/*
 * call this function when registration = TRUE
 */
NSAPI_EXTERN void
setupUpdateRegistration (const char *nickName, const char *ldapURL, const char *ldapUser, const char *ldapPwd)
{
   LdapError err;

   if (ldapURL)
   {
      Ldap ldap(err, ldapURL, ldapUser, ldapPwd, NULL, NULL);
      if (err == OKAY)
      {
         LdapEntry ldapEntry(&ldap);

         err = ldapEntry.retrieve(DEFAULT_COMMON_PREFS_FILTER, LDAP_SCOPE_SUBTREE, ldap.baseSuffix());
          
         if (err == OKAY)
         {
            ldapEntry.addAttribute("nsnyr", nickName);
            ldapEntry.update(ldapEntry.entryDN());
         }
      }
   }
}

/*
 *  API: setupRawText2String:
 *    convert raw resource text to string. 
 *    carriage return is defined as " \n" in raw text
 *  Return NULL when input is NULL otherwise return char * which contains
 *         converted value.  
 *           
 *  Note:  caller needs to free the return memory.
 */
NSAPI_EXTERN char *
setupRawText2String (const char *text)
{
   int i, j;
   int len;
   char *new_text;

   if (text == NULL)
      return NULL;

   len = strlen(text);
   new_text = (char *)malloc(len + 1);

   for (i = 0, j = 0; i < len; i++, j++)
   {
      // Take care of backslash n for now.
      if (text[i] == ' ' && text[i+1] == '\\' && text[i+2] == 'n')
      {
         new_text[j] = '\n';
         i += 2;
      }
      else
      {
         new_text[j] = text[i];
      }
   }

   new_text[j] = '\0';

   return new_text;
}


} /* End extern "C" */
