/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
**
** NAME:
**   uninstall.cpp
**
** DESCRIPTION:
**   Contains logging routines for uninstallation use.
** NOTES:
**
** HISTORY:
** $Log: uninstall.cpp,v $
** Revision 1.2  2006/03/23 15:59:10  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:30  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:27  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:13  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.2  2001/11/06 01:51:32  ahakim
** XXX AOL/NSCP Copyright XXX
**
** Revision 1.1.1.1  1999/03/04 23:28:31  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.2  1998/03/28 03:28:08  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.1  1997/11/11 19:16:41  pvo
** Added support for logging uninstall info.
**
**
*********************************************************************/

extern "C" {
#include "stdio.h"
}

#include "nsdefs.h"
#include "uninstall.h"

extern "C"
{
void logUninstallInfo(const char *serverRoot, 
                      const char *packageNickName,
                      const char *moduleNickName,
                      const char *what)
{
   char tstr[MED_BUF];
   FILE *fp = NULL;

   if (serverRoot == NULL || packageNickName == NULL || what == NULL)
   {
      return;
   }

   if (moduleNickName)
   {
      snprintf(tstr, sizeof(tstr), "%s%c%s%c%s%c%s.log", 
                    serverRoot, 
		    PATH_DELIM,
                    "setup", 
		    PATH_DELIM,
                    packageNickName,
		    PATH_DELIM,
                    moduleNickName);
   }
   else
   {
      snprintf(tstr, sizeof(tstr), "%s%c%s%c%s%c%s.log", 
                    serverRoot, 
		    PATH_DELIM,
                    "setup", 
		    PATH_DELIM,
                    packageNickName,
		    PATH_DELIM,
                    packageNickName);
   }
   tstr[sizeof(tstr)-1] = 0;
   
   fp = fopen(tstr, "a");

   if ( fp )
   {
      fprintf(fp, "%s\n", what);
      fclose(fp);
   }

   return;
}

}
