#!/usr/bin/perl
#
# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation version
# 2.1 of the License.
#  
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#  
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# END COPYRIGHT BLOCK
#
use Getopt::Std;

# assuming unzip exists (XXX: check it!)
# c.f., $path = $ENV{PATH};

sub Usage() {
	print "Usage: $0 -z <zipfile> [-i -l <logfile> -r <serverroot>] [-x -d <dest>] [-s <sizefile>]\n";
	exit();
}

getopts("z:d:r:l:s:xi");

$zipFile = "";
$dest = "";
$serverRoot = "";
$logFile = "";
$sizeFile = "";
$extract = 0;
$list = 0;
$getsize = 0;
$size = 0;

if ($opt_z) {
	$zipFile = $opt_z;
}
if ($opt_d) {
	$dest = $opt_d;
}
if ($opt_r) {
	$serverRoot = $opt_r;
}
if ($opt_l) {
	$logFile = $opt_l;
}
if ($opt_x) {
	$extract = 1;
}
if ($opt_i) {
	$list = 1;
}
if ($opt_s) {
	$getsize = 1;
	$sizeFile = $opt_s;
}

if ("" eq $zipFile) {
	Usage();
}
if (1 == $list) {
	if ("" eq "$logFile" || "" eq $serverRoot) {
		Usage();
	}
}
if (1 == $extract) {
	if ("" eq "$dest") {
		Usage();
	}
}
if (1 == $getsize) {
	if ("" eq "$sizeFile") {
		Usage();
	}
}

if (1 == $list) {
	open(LOG, "> $logFile") or die "Couldn't open $logFile: $!\n";
	$pid = open(README, "unzip -l $zipFile |") or die "Couldn't run unzip -l $zipFile: $!\n";
	while ($l = <README>) {
		chop($l);
		if ($l =~ /[0-1][0-9]-[0-3][0-9]-[0-9][0-9]/) {
			($w0, $w1, $w2, $w3, $w4) = split(/ +/, $l, 5);
			if ($serverRoot =~ /.*\/$/) {
				print LOG "$serverRoot$w4 $w1\n";
			} else {
				print LOG "$serverRoot/$w4 $w1\n";
			}
		}
		$prevl = $l;
	};
	($w0, $w1, $w2) = split(/ +/, $prevl, 3);
	$size = $w1;
	close(README);
	close(LOG);
}

if (0 == $size && 1 == $getsize) {
	$pid = open(README, "unzip -l $zipFile |") or die "Couldn't run unzip -l $zipFile: $!\n";
	while ($l = <README>) {
		chop($l);
		$prevl = $l;
	};
	($w0, $w1, $w2) = split(/ +/, $prevl, 3);
	$size = $w1;
	close(README);
}

if (1 == $getsize) {
	open(SIZE, "> $sizeFile") or die "Couldn't open $sizeFile: $!\n";
	print SIZE "$size";
	close(SIZE);
}

if (1 == $extract) {
	exec("unzip -o -qq $zipFile -d $dest") or die "Couldn't run unzip $zipFile: $!\n";
}
