/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** NAME
**   dl-common.cc
**
** DESCRIPTION
**   Common dialog routines
**
** NOTES:
**
** HISTORY
**
** $Log: dl-common.cc,v $
** Revision 1.2  2006/03/23 15:59:17  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:30  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.5  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.4  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.3  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/03 18:24:09  nhosoi
** rebranded setupsdk as brandx; updated the version number to 7.0
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.5  2001/09/20 21:36:36  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.4  2000/07/18 02:28:23  peterz
** convert admin domain to UTF8 before display to the terminal
**
** Revision 1.1.1.3  2000/03/06 00:57:11  peterz
** change "Netscape" to "iPlanet"
**
** Revision 1.1.1.2  1999/11/16 01:35:53  cxue
** keep going if no components can be selected
**
** Revision 1.1.1.1  1999/03/04 23:27:00  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.45.2.8  1999/02/04 23:57:07  cxue
** resource changes
**
** Revision 1.1.2.45.2.7  1999/02/04 02:24:42  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.45.2.6  1998/11/16 22:40:31  cxue
** allow checked in general section
**
** Revision 1.1.2.45.2.5  1998/10/29 03:13:33  cxue
** move dialog resource from dl-common.cc to uxres_data.cc
**
** Revision 1.1.2.45.2.4  1998/10/28 19:27:16  cxue
** change resource api to take id as string
**
** Revision 1.1.2.45.2.3  1998/10/15 22:22:04  cxue
** resource framework for custom ui
**
** Revision 1.1.2.45.2.2  1998/10/09 21:27:10  cxue
** select invisible components if it has dependency on it and mandatory.
**
** Revision 1.1.2.45.2.1  1998/09/11 01:15:42  cxue
** 322263, Ctrl-B doesn't work at password input field
**
** Revision 1.1.2.45  1998/08/22 02:03:45  pvo
** Fixed seg fault
**
** Revision 1.1.2.44  1998/08/21 02:04:04  cxue
** getLdapAdminDomain --> verifyInstallPrivilege
**
** Revision 1.1.2.43  1998/08/21 00:20:32  cxue
** return DIALOG_SAME after wrong user/password warning.
**
** Revision 1.1.2.42  1998/08/20 21:07:54  cxue
** remove local admin domain check
**
** Revision 1.1.2.41  1998/08/20 18:08:21  cxue
** verifyInstallPrivilege after user/password
**
** Revision 1.1.2.40  1998/08/20 01:39:41  cxue
** Show striped LdapURL when asking for ldap user
**
** Revision 1.1.2.37  1998/08/19 17:49:52  cxue
** Change to support Config only directory server and user only directory server
**
** Revision 1.1.2.36  1998/07/25 06:31:24  cxue
** Bug 302072. add logic to auto-detect UserDirectoryURL from DS4.0. This can avoid to ask UserDirectoryURL many times during installation. Requested by Sunil. Reviewed by Phong.
**
** Revision 1.1.2.35  1998/07/16 17:12:21  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.34  1998/07/02 03:30:31  pvo
** Some minor name changes
**
** Revision 1.1.2.33  1998/06/27 03:02:23  pvo
** Name change
**
** Revision 1.1.2.32  1998/06/25 18:04:46  pvo
** Change Users/Groups DS to User DS
**
** Revision 1.1.2.31  1998/06/24 17:41:12  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.30  1998/06/16 02:04:57  pvo
** 124100 - Server selection problem
**
** Revision 1.1.2.29  1998/05/29 16:55:13  pvo
** 117356 - component selection accepts any selection
**
** Revision 1.1.2.28  1998/05/13 04:39:29  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.27  1998/05/01 23:24:40  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.26  1998/04/30 00:45:20  pvo
** Add dialogs to disallow ldaps
**
** Revision 1.1.2.25  1998/04/14 23:32:54  pvo
** 113703 - Fix default selection problem
**
** Revision 1.1.2.24  1998/04/01 03:12:39  pvo
** Fixed typo mistake
**
** Revision 1.1.2.23  1998/04/01 03:06:26  pvo
** 112122 - Used checked components for defaults
**
** Revision 1.1.2.22  1998/04/01 02:42:29  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.21  1998/03/28 03:29:01  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.20  1998/03/13 21:24:47  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.19  1998/03/13 05:00:35  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.18  1998/03/10 02:49:13  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.17  1998/03/02 23:59:25  pvo
** 106912 - Pickup the right defaults in dialogs.
** 105320 - Improve logging of silent installation.
**
** Revision 1.1.2.16  1998/02/26 04:01:57  pvo
** Fixed break in download.cc due to the call to set_ldap_option() 
** in libinstall.
** Also try to detect input error when asking for installDN and try to
** create installDN if possible in case the user has the privilege 
** to create it.
**
** Revision 1.1.2.15  1998/02/26 00:13:57  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.14  1998/02/25 05:15:07  pvo
** Removed unecessary uninstall question for installDN.
**
** Revision 1.1.2.13  1998/02/24 00:23:05  pvo
** Support for topology changes.
**
** Revision 1.1.2.12  1998/02/17 23:36:48  pvo
** Fixed problem with defaults for Yes/No answer.
**
** Revision 1.1.2.11  1998/02/15 22:06:38  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.10  1998/01/09 03:27:27  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.9  1997/12/19 07:04:47  pvo
** Fixed selection problem in download.
**
** Revision 1.1.2.8  1997/12/12 21:54:15  pvo
** Various bug fixes.
**
** Revision 1.1.2.7  1997/12/03 18:24:50  pvo
** Fixed bug in askLdapInfoNext.
**
** Revision 1.1.2.6  1997/12/02 20:39:06  pvo
** Moved suitespot.inf to ./inf directory.
** Fixed problem with ns-config being called twice.
** Silent Installation support.
**
** Revision 1.1.2.5  1997/12/02 04:28:26  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.4  1997/11/26 02:02:52  pvo
** Restructure files and add Download support.
**
**
***********************************************************************/
extern "C"
{
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
}

#include "ldapu.h"
#include "ux-util.h"
#include "dialog.h"
#include "dl-common.h"
#include "ux-setup.h"
#include "installer.h"
#include "utf8.h"
#include "nsutils.h"


/*
** Forward References
**/
Bool yesNoDefaultNo(const char *answer);
Bool yesNoDefaultYes(const char *answer);
DialogAction askLdapInfoSetup(Dialog *me);
DialogAction askLdapInfoNext(Dialog *me);
DialogAction askLdapUserSetup(Dialog *me);
DialogAction askLdapUserNext(Dialog *me);
DialogAction askInstallDNSetup(Dialog *me);
DialogAction askInstallDNNext(Dialog *me);

Bool mayAskUserDirectoryInfo(Dialog *me);

/////////////////////////////////////////////////////////////////////////
/// Some common routines
/////////////////////////////////////////////////////////////////////////
static Bool 
isDefault(const char *answer, const char *defaultAns)
{
   Bool ret = False;
   if (answer[0] == '\0' || answer[0] == '\n')
   {
      /* Enter always the defeault */
      ret = True;
   }   
   else if (answer[0] == 'y' || answer[0] == 'Y')
   {
      if (!strcasecmp(defaultAns, "Yes"))
      {
         ret = True;
      }
   }
   else
   {
      if (!strcasecmp(defaultAns, "No"))
      {
          ret = True;
      }
   }
   return ret;
}

Bool
yesNoDefaultNo(const char *answer)
{
   return isDefault(answer, "No");
}

Bool
yesNoDefaultYes(const char *answer)
{
   return isDefault(answer, "Yes");
}

void
dialogSetup (Dialog *me, const char *which)
{
   const char *ans = localToUTF8(me->manager()->getDefaultScript()->get(which));

   if (ans && ans[0] != 0)
      me->setDefaultAns(ans);
}

Dialog *
createSelectionDialog (Components *products, const char *prompt, DLG_CALLBACK cb, const char *selectionText, const char *whatProduct)
{
   char tstr[SML_BUF];

   DialogInput *dl = new DialogInput;

   if (whatProduct != NULL)
      snprintf(tstr, sizeof(tstr), selectionText, whatProduct);
   else
      snprintf(tstr, sizeof(tstr), selectionText);
   tstr[sizeof(tstr)-1] = 0;

   dl->setText(tstr);
   dl->setSetupAction(NULL);
   dl->setNextAction(cb);
   dl->setPrevAction(NULL);
   dl->setDefaultAns("All");
   dl->setPrompt(prompt);

   setupComponentDialog (dl, products);

   return dl;
}

int
getSelectedComponent(Dialog *me, Components *components, Bool express)
{
  char      temp[SML_BUF];
  const char *buf = me->input();
  Product *p;
  Product *products[MAX_PRODUCTS];
  int j = 0;
  int i, k=0;
  Bool  hasVisible = False;

  // First, build a list of selectable components
  // Those that are not visible will be automatically
  // selected but not included in selection list

  for (i = 0; i < components->numComponents(); i++)
  {
    p = components->getComponent(i);
    if (p->isVisible())
    {
      products[j++] = p;
      p->deSelectAll();
      hasVisible = True;
    }
  }
  if (hasVisible == False)
  {
    // if no visible component, we have to let 
    // user to move on.  99 is random big number
    return 99;
  }

  if (buf[0] == 0)
  {
     if (me->defaultAns())
        buf = me->defaultAns();
     else
        buf = "All";
  }

  if (buf[0] == 'a' || buf[0] == 'A')
  {
     me->setDefaultAns("All");

     for (i = 0; i < j; i++, k++)
     {
        if (express)
        {
           products[i]->selectDefaults();
        }
        else
        {
           products[i]->select();
        }
     }
  }
  else
  {
    for (int i = 0; i < j; i++)
    {
      products[i]->deSelect();
      sprintf(temp, "%d", i + 1);
      if (strstr(buf, temp))
      {
        if (express)
        {
           products[i]->selectDefaults();
        }
        else
        {
           products[i]->select();
        }
          k++;
      }
    }
    if (k)
       me->setDefaultAns(buf);
  }
  return k;
}

void
setupComponentDialog (Dialog *me, Components *components)
{
   int i,j = 0;
   Product     *p;
   NSString    text2;
   NSString    ans;
   char        tstr[SML_BUF];

   text2 = 
"Components with a number in () contain additional subcomponents\n"
"which you can select using subsequent screens.\n\n";

   for (i = 0; i < components->numComponents(); i++)
   {
      p = components->getComponent(i);
      if (p->isVisible())
      {
         if (p->numVisibleComponents() >= 1)
            snprintf(tstr, sizeof(tstr), "    %d. %s (%d)\n", ++j, p->get(PKG_NAME), p->numVisibleComponents());
         else
            snprintf(tstr, sizeof(tstr), "    %d. %s\n", ++j, p->get(PKG_NAME));
         tstr[sizeof(tstr)-1] = 0;

         text2 = text2 + tstr;

         if (p->isChecked() && p->numVisibleComponents() == 0)
         {
            sprintf(tstr, "%d", j);
            if (ans == (char *) NULL)
            {
               ans = tstr;
            }
            else
            {
               ans = ans + ", " + tstr;
            }
         }
      }
   }
   me->setText2(text2.data());
   me->setUserData("Components", (long) components);
   if (ans == (char *) NULL)
   {
      ans = "All";
   }
   me->setDefaultAns(ans);
}

/////////////////////////////////////////////////////////////////////////
/// askLdapInfo
/////////////////////////////////////////////////////////////////////////

DialogInput askLdapInfo(
NULL,
NULL,
NULL,
askLdapInfoSetup,
askLdapInfoNext,
NULL,
True
);

DialogAction
verifyLdapInfo (Dialog *me, const char *ldapURL, const char *ldapLabel)
{
   SetUp *setup = ((SetUp *)me->manager()->parent());
   DialogAction rc = DIALOG_NEXT;
   NSString errMsg;

   if (!strncasecmp(ldapURL, "ldaps", 5))
   {
      errMsg = getResource(RES_MESSAGE_VERIFYLDAPINFO1);
   }
   else
   {
      me->manager()->getDefaultScript()->set(ldapLabel,ldapURL);
      errMsg = setup->verifyLdapServer(ldapURL);
   }

   if (errMsg != (char *) NULL)
   {
      errMsg = NSString("ERROR: ") + errMsg;
      DialogAlert alert(errMsg);
      alert.execute();
      rc =  DIALOG_SAME;
   }

   return rc;
}
   
DialogAction
askLdapInfoSetup(Dialog *me)
{
   NSString hostPortURL;
   InstallInfo *installInfo = me->manager()->getDefaultScript();

   me->setText(getResource(RES_DIALOG_LDAPINFO_TEXT, RES_GLOBAL_BRAND));
   me->setPrompt(getResource(RES_DIALOG_LDAPINFO_PROMPT));

   hostPortURL = stripConfigLdapURL(installInfo->get(CONFIG_LDAP_URL));
   if (hostPortURL != (char *) NULL)
      me->setDefaultAns(hostPortURL);

   return DIALOG_SAME;
}

DialogAction
askLdapInfoNext(Dialog *me)
{
   const char *answer;
   char *ldapURL;
   const char *buf = me->input();
   NSString errMsg;
   LDAPURLDesc *ludpp;
   char *domain;
   DialogAction rc = DIALOG_NEXT;
   InstallInfo *installInfo = me->manager()->getDefaultScript();
   Installer *setup = ((Installer *)me->manager()->parent());

   answer = me->defaultAns();

   if (buf[0] != '\0')
     answer = buf;

   ldapURL = formConfigLdapURL(answer);

   if (me->isVisited() == False || strcasecmp(buf, installInfo->get(CONFIG_LDAP_URL)))
   {
      rc = verifyLdapInfo(me, ldapURL, CONFIG_LDAP_URL);

      if (rc == DIALOG_NEXT)
      {
         installInfo->set(CONFIG_LDAP_URL, ldapURL);

         if ((domain = guessAdminDomainName(ldapURL, NULL, NULL)) != NULL)
         {
            installInfo->set(DS_ADMIN_DOMAIN, domain);
            free (domain);
         }
         askInstallDN.registerDialogNext(me->manager());
         askLdapInfo.visited();
      }
   }
        
   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// AskInstallDN
/////////////////////////////////////////////////////////////////////////

DialogInput askInstallDN(
NULL,
NULL,

NULL,

askInstallDNSetup,
askInstallDNNext,
NULL,
True
);

DialogAction askInstallDNSetup(Dialog *me)
{
   me->setText(getResource(RES_DIALOG_INSTALLDN_TEXT));
   me->setPrompt(getResource(RES_DIALOG_INSTALLDN_PROMPT));

   dialogSetup(me, DS_ADMIN_DOMAIN);
   return DIALOG_SAME;
}

DialogAction askInstallDNNext(Dialog *me)
{
   NSString errMsg;
   Installer *setup = ((Installer *)me->manager()->parent());
   InstallInfo *installInfo = me->manager()->getDefaultScript();
   const char *ldapURL = installInfo->get(CONFIG_LDAP_URL);
   const char *ldapUser = installInfo->get(MC_ADMIN_ID);
   const char *ldapPwd  = installInfo->get(MC_ADMIN_PWD);
   const char *installDN;

   const char *buf = me->input();

   if (buf[0] == 0)
   {
      installDN = me->defaultAns();
   }
   else
   {
      installDN = buf;
   }

   if (installDN == NULL || *installDN == 0)
   {
      return DIALOG_SAME;
   }

   /*
    * First take care of the case where the user might enter a DN
    */
   if (strpbrk(installDN, "=,") != NULL)
   {
      DialogAlert alert(getResource(RES_MESSAGE_ASKINSTALLDNNEXT1));
      alert.execute();
      return DIALOG_SAME;
   }


   errMsg = setup->verifyInstallDomain(ldapURL, installDN);

   if (errMsg != (char *) NULL)
   {
       errMsg = NSString("ERROR: ") + errMsg;
       DialogAlert alert(errMsg);
       alert.execute();
       return DIALOG_SAME;
   }
   else
   {
      installInfo->set(DS_ADMIN_DOMAIN ,installDN);

      askLdapUser.registerDialogNext(me->manager());
      return DIALOG_NEXT;
   }

   return DIALOG_NEXT;
}



/////////////////////////////////////////////////////////////////////////
/// askLdapUser
/////////////////////////////////////////////////////////////////////////

DialogInput askLdapUser(
NULL,
NULL,

DEFAULT_ADMINUSER,

askLdapUserSetup,
askLdapUserNext,
NULL,
True
);

DialogAction
askLdapUserSetup(Dialog *me)
{
   char text2[SML_BUF];
   int rval;
   char *localLdapURL;
   NSString hostPortURL;
   const char *s;

   me->setText(getResource(RES_DIALOG_LDAPUSER_TEXT));
   me->setPrompt(getResource(RES_DIALOG_LDAPUSER_PROMPT));


   dialogSetup(me, MC_ADMIN_ID);

   localLdapURL = UTF8ToLocal(me->manager()->getDefaultScript()->get(CONFIG_LDAP_URL));
   hostPortURL = stripConfigLdapURL(localLdapURL);

   snprintf(text2, sizeof(text2), "    %s", (const char *)hostPortURL);
   text2[sizeof(text2)-1] = 0;

   free (localLdapURL);

   me->setText2(text2);

   return DIALOG_SAME;
}

DialogAction
getLdapPassword(Dialog *me, const char *ldapURL, const char *ldapUser)
{
   SetUp *setup = ((SetUp *)me->manager()->parent());
   DialogAction rc = DIALOG_NEXT;
   NSString errMsg;
   const char *ldapPwd;

   while (1)
   {
      me->showString("Password: ");
      if (me->getPassword () == 0)
      {
         return DIALOG_PREV;
      }
      else if (me->input()[0] == 0)
      {
         continue;
      }
      else
      {
         ldapPwd = me->input();
         break;
      }
   }

   // Got ldap User and Password, now authenticate
   errMsg = setup->verifyLdapUser(ldapURL, ldapUser, ldapPwd);
   if (errMsg != (char *) NULL)
   {
       errMsg = NSString("ERROR: ") + errMsg;
       DialogAlert alert(errMsg);
       alert.execute();
       rc = DIALOG_SAME;
   }

   return rc;
}

DialogAction
verifyInstallPrivilege(Dialog *me, const char *ldapURL, const char *ldapUser, const char *ldapPwd, const char *installDN)
{
   Installer *setup = ((Installer *)me->manager()->parent());
   DialogAction rc = DIALOG_NEXT;
   NSString errMsg;

   errMsg = setup->verifyInstallPrivilege(ldapURL, ldapUser, ldapPwd, installDN);

   if (errMsg != (char *) NULL)
   {
       errMsg = NSString("ERROR: ") + errMsg;
       DialogAlert alert(errMsg);
       alert.execute();
       return DIALOG_SAME;
   }

   return rc;
}


DialogAction
askLdapUserNext(Dialog *me)
{
   NSString    ldapUser;
   const char *ldapPwd;
   const char *buf = me->input();
   NSString errMsg;
   SetUp *setup = ((SetUp *)me->manager()->parent());
   InstallInfo *installInfo =  me->manager()->getDefaultScript();
   DialogAction rc = DIALOG_NEXT;
   const char *ldapURL = installInfo->get(CONFIG_LDAP_URL);
   const char *installDN = installInfo->get(DS_ADMIN_DOMAIN);

   if (buf[0] != 0)
   {
     ldapUser = buf;
   }
   else
   {
      ldapUser = me->defaultAns();
   }

   installInfo->set(MC_ADMIN_ID,ldapUser);

   rc = getLdapPassword(me, installInfo->get(CONFIG_LDAP_URL), ldapUser);

   if (rc == DIALOG_NEXT)
   {
      ldapPwd = me->input();
      rc = verifyInstallPrivilege(me, ldapURL, ldapUser, ldapPwd, installDN);

      if (rc == DIALOG_NEXT)
      {
         installInfo->set(MC_ADMIN_PWD, ldapPwd);
         me->visited();
         rc = DIALOG_EXIT;
         if (installInfo->get("SetupType"))
         {
            installInfo->set(UG_LDAP_URL, ldapURL);
            installInfo->set(UG_ADMIN_ID, ldapUser);

            if (mayAskUserDirectoryInfo(me) == True)
               rc = DIALOG_NEXT;
         }
      }
   }

   return rc;
}

DialogAction
welcomeSetup(Dialog *me)
{
  NSString welcome_text(getResource(RES_DIALOG_WELCOME_TEXT, RES_GLOBAL_BRAND));
  welcome_text = welcome_text +
                 getResource(RES_DIALOG_WELCOME_TEXT0, RES_GLOBAL_BRAND) +
                 getResource(RES_DIALOG_WELCOME_TEXT1, RES_GLOBAL_CONSOLE);
  me->setText(welcome_text);
  me->setText2(getResource(RES_DIALOG_WELCOME_TEXT2));
  me->setPrompt(getResource(RES_DIALOG_WELCOME_PROMPT));
  return DIALOG_SAME;
}

/////////////////////////////////////////////////////////////////////////
/// askUserDirectoryInfo
/////////////////////////////////////////////////////////////////////////

DialogInput askUserDirectoryInfo(
NULL,
NULL,

NULL,

askUserDirectoryInfoSetup,
askUserDirectoryInfoNext,
NULL,
True
);

DialogAction
askUserDirectoryInfoSetup(Dialog *me)
{
   me->setText(getResource(RES_DIALOG_USERDIRECTORYINFO_TEXT));
   me->setPrompt(getResource(RES_DIALOG_USERDIRECTORYINFO_PROMPT));

   dialogSetup (me, UG_LDAP_URL);
   return DIALOG_SAME;
}

DialogAction
askUserDirectoryInfoNext(Dialog *me)
{
   const char *ldapURL;
   const char *buf = me->input();
   NSString errMsg;
   LDAPURLDesc *ludpp;
   DialogAction rc = DIALOG_NEXT;

   ldapURL = me->defaultAns();

   if (buf[0] != 0)
     ldapURL = buf;

   rc = verifyLdapInfo(me, ldapURL, UG_LDAP_URL);

   if (rc == DIALOG_NEXT)
   {
      askUserDirectoryAdmin.registerDialogNext(me->manager());
      askUserDirectoryInfo.visited();
   }

   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// askUserDirectoryAdmin
/////////////////////////////////////////////////////////////////////////

DialogInput askUserDirectoryAdmin(
NULL,
NULL,

DEFAULT_ADMINUSER,

askUserDirectoryAdminSetup,
askUserDirectoryAdminNext,
NULL,
True
);

DialogAction
askUserDirectoryAdminSetup(Dialog *me)
{
   char text2[SML_BUF];
   int rval;
   char *localLdapURL;
   const char *s;

   me->setText(getResource(RES_DIALOG_USERDIRECTORYADMIN_TEXT));
   me->setPrompt(getResource(RES_DIALOG_USERDIRECTORYADMIN_PROMPT));

   dialogSetup(me, UG_ADMIN_ID);

   localLdapURL = UTF8ToLocal(me->manager()->getDefaultScript()->get(UG_LDAP_URL));

   snprintf(text2, sizeof(text2), "    %s", localLdapURL);
   text2[sizeof(text2)-1] = 0;

   free (localLdapURL);

   me->setText2(text2);

   return DIALOG_SAME;
}

DialogAction
askUserDirectoryAdminNext(Dialog *me)
{
   NSString    ldapUser;
   const char *ldapURL;
   const char *buf = me->input();
   NSString errMsg;
   SetUp *setup = ((SetUp *)me->manager()->parent());
   const char *installDN;
   InstallInfo *installInfo =  me->manager()->getDefaultScript();
   DialogAction rc;

   if (buf[0] != 0)
   {
     ldapUser = buf;
   }
   else
   {
      ldapUser = me->defaultAns();
   }

   installInfo->set(UG_ADMIN_ID,ldapUser);

   rc = getLdapPassword(me, installInfo->get(UG_LDAP_URL), ldapUser);

   if (rc == DIALOG_NEXT)
   {
      installInfo->set(UG_ADMIN_PWD, me->input());
      me->visited();
      rc = DIALOG_EXIT;;
   }
   return rc;
}

Bool
mayAskUserDirectoryInfo(Dialog *me)
{
   DialogAction rc = DIALOG_NEXT;
   NSString errMsg;
   InstallInfo *installInfo = me->manager()->getDefaultScript();
   Installer *setup = ((Installer *)me->manager()->parent());
   const char *ldapURL = installInfo->get(CONFIG_LDAP_URL);
   const char *ldapUser = installInfo->get(MC_ADMIN_ID);
   const char *ldapPwd  = installInfo->get(MC_ADMIN_PWD);
   const char *installDN = installInfo->get(DS_ADMIN_DOMAIN);

   if (setup->needUserDirAuth())
   {
      char *url = NULL;

      errMsg = setup->getUserDirectory(
            ldapURL, 
            ldapUser, 
            ldapPwd,
            installDN, 
            &url, 
            NULL,
            NULL);

      if (errMsg != (char *)NULL) 
      {
         askUserDirectoryInfo.registerDialogNext(me->manager());
      }
      else
      {
         installInfo->set(UG_LDAP_URL, url);
         askUserDirectoryAdmin.registerDialogNext(me->manager());
            if (url != NULL)
                free (url);
      }

      return True;
   }

   return False;
}
