/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** NAME
**   dl-uninst.cc
**
** DESCRIPTION
**   Common dialog routines Uninstaller
**
** NOTES:
**
** HISTORY
**
** $Log: dl-uninst.cc,v $
** Revision 1.2  2006/03/23 15:59:17  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:31  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.4.4.1  2002/04/13 01:02:13  djani
** Bug 601745: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.4  2001/11/15 22:52:17  ahakim
** 600423: remove iplanet reference
**
** Revision 1.1.1.3  2001/09/20 21:36:37  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.2  2000/03/06 00:59:41  peterz
** change "Netscape" to "iPlanet"
**
** Revision 1.1.1.1  1999/03/04 23:27:02  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.23.4.2  1999/02/04 23:57:08  cxue
** resource changes
**
** Revision 1.1.2.23.4.1  1998/10/29 03:13:35  cxue
** move dialog resource from dl-common.cc to uxres_data.cc
**
** Revision 1.1.2.23  1998/07/22 01:24:55  cxue
** bug 309301. setup dialog chain, such it only asks Ldap Info once for all
**
** Revision 1.1.2.22  1998/07/02 03:30:32  pvo
** Some minor name changes
**
** Revision 1.1.2.21  1998/06/01 21:20:08  pvo
** 122446 - Uninstall prematurely deletes itself
**
** Revision 1.1.2.20  1998/05/29 16:55:14  pvo
** 117356 - component selection accepts any selection
**
** Revision 1.1.2.19  1998/05/01 23:24:41  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.18  1998/04/30 00:45:21  pvo
** Add dialogs to disallow ldaps
**
** Revision 1.1.2.17  1998/04/14 03:28:55  pvo
** 110241 - Uninstall use wrong port
**
** Revision 1.1.2.16  1998/03/28 03:29:02  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.15  1998/03/13 21:24:47  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.14  1998/03/13 05:00:36  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.13  1998/03/10 02:49:15  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.12  1998/03/09 22:09:36  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.11  2000/03/03 21:39:57  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.10  1998/02/26 00:13:59  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.9  1998/02/15 22:06:40  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.8  1998/01/22 04:51:27  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.7  1998/01/09 03:27:28  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.6  1997/12/12 21:54:17  pvo
** Various bug fixes.
**
** Revision 1.1.2.5  1997/11/26 02:02:55  pvo
** Restructure files and add Download support.
**
**
***********************************************************************/

extern "C"
{
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
}

#include "ldapu.h"
#include "ux-util.h"
#include "dialog.h"
#include "uninstaller.h"
#include "dl-common.h"
#include "dl-uninst.h"

/*
** Forward References
**/
DialogAction askSeletionNext(Dialog *);
DialogAction askLocalUninstSetup(Dialog *me);
DialogAction askLocalUninstNext (Dialog *me);

static int
getUninstallComponents(Dialog *me, Components *components)
{
   char      temp[SML_BUF];
   const char *buf = me->input();
   Product *p;
   Product *products[MAX_PRODUCTS];
   int j = 0;
   int i, k=0;

   for (i = 0; i < components->numComponents(); i++)
   {
      p = components->getComponent(i);
      if (p->isVisible())
        products[j++] = p;
      else
        p->deSelect();
   }

   if (buf[0] == 0)
   {
      if (me->defaultAns())
         buf = me->defaultAns();
      else
         buf = "All";
   }

   if (buf[0] == 'a' || buf[0] == 'A')
   {
      me->setDefaultAns("All");
	  /** Bug : 601745 If all components are invisible return 1**/
	  if(j==0) return 1;

      for (i = 0; i < j; i++, k++)
      {
         if (components->isTop())
         {
            products[i]->selectAll();
         }
         else
         {
            products[i]->select();
         }
      }
   }
   else
   {
      for (int i = 0; i < j; i++)
      {
         products[i]->deSelect();
         sprintf(temp, "%d", i + 1);
         if (strstr(buf, temp))
         {
            if (components->isTop())
               products[i]->selectAll();
            else
               products[i]->select();
	    k++;
         }
      }
      if (k)
         me->setDefaultAns(buf);
   }
   return k;
}

DialogYesNo askLocalUninst(
NULL,
"Would you like to continue with local uninstallation?",
           "No",
           askLocalUninstSetup,
           askLocalUninstNext
);

DialogAction askLocalUninstSetup(Dialog *me)
{
   NSString text1 (
"The Netscape Directory Server at\n");

   NSString text1a(
"is down, unreachable, or the configured LDAP URL is not valid.");

   char *text2 =
"It is recommended that you fix the problem with the registry\n"
"then perform uninstallation again. To quit uninstallation, type\n"
"CTRL-C.\n\n"
"Alternatively, you can enter the correct URL or proceed with local\n"
"uninstallation of the selected servers.\n";

   text1 = text1 + "     " 
                 + me->manager()->getDefaultScript()->get(CONFIG_LDAP_URL) 
                 + "\n"
                 + text1a;

   me->setText(text1);
   me->setText2(text2);

   return DIALOG_SAME;
}

DialogAction askLocalUninstNext(Dialog *me)
{
   const char *buf = me->input();

   me->manager()->resetLast();

   if (buf[0] == 0)
   {
       buf = me->defaultAns();
   }

   if (yesNoDefaultNo(buf) == True)
   {
      askLdapInfo.registerDialogNext(me->manager());
   }

   return DIALOG_NEXT;
}

DialogAction
askSelectionNext(Dialog *me)
{
   DialogAction rc;
   UnInstaller  *uninstaller = (UnInstaller *) me->manager()->parent();
   InstallInfo *installInfo =  me->manager()->getDefaultScript();

   rc = verifySelection(me);

   if (rc == DIALOG_NEXT)
   {
      /*
       * Take care of module
       */
      Components *products;
      me->getUserData("Components", (long &) products);

      if (products->isTop())
         me->manager()->resetLast();

      for (int i = 0 ; i < products->numComponents(); i++)
      {
         Product *p = products->getComponent(i);

         if (p->isSelected() && p->numComponents() > 0 )
         {
            Dialog *selectionDialog;
            selectionDialog = createSelectionDialog(
               p->getComponents(),
               "Specify the components you wish to uninstall",
               askSelectionNext,
               getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND),
               p->get(PKG_NAME));
            selectionDialog->registerDialogLast(me->manager());
         }
      }

      me->visited();

      // Ask ldap info once at the top level. this avoids to register 
      // ldap dialog many times.
      if (products->isTop() &&
         uninstaller->getInstallableComponents()->useLdap())
      {
         if( uninstaller->existingLdapCheck() == 0)
         {
            const char *ldapURL = installInfo->get(CONFIG_LDAP_URL);

            NSString errMsg;

            if (!strncasecmp(ldapURL, "ldaps", 5))
            {
               char tmp[MED_BUF];
               LDAPURLDesc *ludpp = NULL;

               ldap_url_parse ((char *) ldapURL , &ludpp);
               snprintf(tmp, sizeof(tmp), "ldap://%s:389/%s", ludpp->lud_host, ludpp->lud_dn);
               tmp[sizeof(tmp)-1] = 0;
               ldap_free_urldesc(ludpp);

               installInfo->set(CONFIG_LDAP_URL, tmp);

               askLdapInfo.registerDialogLast(me->manager());

            }
            else if ((errMsg = uninstaller->verifyLdapServer(ldapURL)) != (char *) NULL)
            {
               askLocalUninst.registerDialogLast(me->manager());
            }
            else
            {
               askLdapUser.registerDialogLast(me->manager());
            }
         }
         else
         {
            ; // Just ignore, don't ask for Ldap Information
         }
      }
   }
   return rc;
}

DialogAction
verifySelection(Dialog *me)
{
   UnInstaller  *uninstaller = (UnInstaller *) me->manager()->parent();
   int i, j;
   char tstr[SML_BUF];
   Components *components;
   Components *all;
   NVPair *dp = NULL;

   me->getUserData("Components", (long &) components);

   if (getUninstallComponents (me, components) == 0)
   {
      return DIALOG_SAME;
   }

   all = uninstaller->getInstallableComponents();

   for (i = 0 ; i < components->numComponents(); i++)
   {
      Product *p = components->getComponent(i);

      if (p->isSelected())
      {
         const char *s = p->get(PKG_NICKNAME);

//         for (j = i + 1; j < components->numComponents(); j++)
         for (j = 0; j < all->numComponents(); j++)
         {
//            Product *q = components->getComponent(j);

	    Product *q = all->getComponent(j);

            if (dp)
               delete dp;

            dp = q->dependencies(PKG_DEPENDENCIES);

            if (dp && dp->get(s) && q->isSelected() == False)
            {
               // q is dependent on p and q is not selected
               snprintf(tstr, sizeof(tstr),
                  getResource(RES_MESSAGE_UNINST_VERIFYSELECTION1),
                  q->get(PKG_NAME), 
                  p->get(PKG_NAME),
                  q->get(PKG_NAME),
                  q->get(PKG_NAME));
               tstr[sizeof(tstr)-1] = 0;

               DialogAlert  alert(tstr);
               alert.execute();
               delete dp;
               return DIALOG_SAME;
            }
         }
      }
   }

   return DIALOG_NEXT;
}

