/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME:
**   installer.cc
**
** DESCRIPTION:
**   Installation Program for servers
**
** NOTES:
**
** HISTORY:
** $Log: installer.cc,v $
** Revision 1.5  2006/03/27 21:18:35  nhosoi
** [186642] Directory Server Makefile updates for Internal build
** Changed the createSIE so that caller (Admin Server and Directory Server) passes
** the brand to SetupUtil.
**
** Revision 1.4  2006/03/23 15:59:17  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.3  2005/12/13 20:58:22  nhosoi
** [175418] Admin Server ns-update crashes if necessary attribute-value is not given
** 1) Although Admin Server's PostInstall program ns-update expects it, PostInstall
** framework in SetupUtil does not take silent install option "-s".  Changed to pass the option.
** 2) PostInstall framework does not support logging. Changed to pass the option.
** 3) PostInstall failure is issued as WARNING and the installation finishes with
** Success message even though it really failed.  Increased the error level to FATAL.
**
** Revision 1.2  2005/10/26 16:38:16  rmeggins
** Bug(s) fixed: 171793
** Bug Description: Change setuputil version to 1.0
** Reviewed by: Noriko (Thanks!)
** Fix Description: Mostly just typical rebranding stuff.  I found a couple
** of places in installer.cc that were not listed in the branding.dat so I
** added them.  I also got rid of the webserv version and the enterprise
** stuff, which is apparently not used anywhere.
** Platforms tested: RHEL4
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:31  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.5  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.4  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.3  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/03 22:18:51  nhosoi
** 1) extracted hardcoded brandnames from error messages
** 2) <Brand> Console is changed to <Brand> Management Console (uxres_data.cc)
** modified files: dl-inst.cc,installer.cc,uxres.cc,uxres.h,uxres_data.cc
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.30.4.5  2003/09/23 21:59:15  djani
** Bug 623890: Before any prepreInstall program is executed, the program name will be prefixed with the directory path where the setup program lives. So now, the admin can mention
** 1) prepreInstall = relative_path_to_program_from_where_setup_lives (e.g. prepreInstall= dsktune - which we currently have) or
** 2) prepreInstall = absolute_path_to_program
** and the program will be executed successfully without requiring a '.' in the PATH.
**
** Revision 1.1.1.30.4.4  2003/08/28 18:13:22  djani
** Bug 601226: Now, the setup allows the programmer to run PrePreInstall programs on Unix platforms. The field PrePreInstall holds the location of the program relative to the setup program that will be executed before the dialogues appear. This is a new feature in the setupsdk.
**
** Revision 1.1.1.30.4.3  2003/08/15 23:42:06  djani
** Bug 408242: If LD_LIBRARY_PATH (SHLIB_PATH for hpux) is set when user runs the setup program, a notice is displayed now. Also, ENV varialble is unset during the installation process.
**
** Revision 1.1.1.30.4.2  2002/04/19 19:05:15  djani
** Now this version is called 6.2
**
** Revision 1.1.1.30.4.1  2002/04/13 00:31:12  djani
** changed version to 6.02
**
** Revision 1.1.1.30  2002/01/17 01:56:54  djani
** 601224 : changed version from 4.2 to 6.0
**
** Revision 1.1.1.29  2001/11/16 03:18:08  pkhincha
** Wrong fix. REverting back. Umask cannot be set at init. Will create all
** directories as 700.
**
** Revision 1.1.1.28  2001/11/16 00:18:57  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.27  2001/11/01 19:50:31  djani
** 520937: changed umask(022) to (077) in order to create tempfiles under /var/tmp with rw------- permissions
**
** Revision 1.1.1.26  2001/09/20 21:36:37  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.25  2000/03/06 01:00:54  peterz
** change "Netscape" to "iPlanet"
**
** Revision 1.1.1.24  1999/12/04 00:34:57  cxue
** SecurityCheck and RequireDomain support in setup.inf
**
** Revision 1.1.1.23  1999/11/15 21:16:52  cxue
** 4.2 to 4.5
**
** Revision 1.1.1.22  1999/11/12 01:57:59  cxue
** Keep going even no archive is defined in <product>.inf
**
** Revision 1.1.1.21  1999/10/27 00:30:17  cxue
** Allow cutomizing finish message
**
** Revision 1.1.1.20  1999/10/15 16:58:08  cxue
** take out extra diskspace checking
**
** Revision 1.1.1.19  1999/10/15 01:21:04  cxue
** log message
**
** Revision 1.1.1.18  1999/10/15 00:57:15  cxue
** more logs
**
** Revision 1.1.1.17  1999/10/13 00:19:25  cxue
** chnage <serverroot> and go forward
**
** Revision 1.1.1.16  1999/09/28 21:08:05  cxue
** without input, fgets returns \n
**
** Revision 1.1.1.15  1999/09/27 20:18:39  cxue
** check diskspace after selection is done
**
** Revision 1.1.1.14  1999/09/17 01:48:47  cxue
** fix long lasting cross-dependence bug
**
** Revision 1.1.1.13  1999/07/29 00:23:54  svbld
** Merged the admin42_branch back onto the admin_branch..
**
** Revision 1.1.1.11.2.4  1999/07/26 21:08:42  shihcm
** bug#353225, (Msg)Error message missing which component need to install
**
** Revision 1.1.1.11.2.3  1999/06/16 22:48:25  cxue
** Calling fgets instead of gets to get rid of warnings on Linux
**
** Revision 1.1.1.11.2.2  1999/06/15 04:25:51  cxue
** bug 353225, can't cast NSString to char * on AIX
**
** Revision 1.1.1.11.2.1  1999/06/14 20:46:56  cxue
** Get status for renamed file back from InfoZip
**
** Revision 1.1.1.11  1999/06/08 23:24:50  cxue
** Use user's login id and group as default SSUSER and SSGROUP if user id is not root
**
** Revision 1.1.1.10  1999/06/07 23:41:14  cxue
** Write out selected component list and installed component list to installation cache, so NAS and NES preInstall can base on it to ask different questions
**
** Revision 1.1.1.9  1999/06/04 06:04:37  cxue
** 354173, don't connect to DS if not required
**
** Revision 1.1.1.8  1999/05/27 16:49:58  cxue
** bug 344882, support UseLdap=TRUE in slapd section
**
** Revision 1.1.1.7  1999/05/21 22:10:18  cxue
** if version is defined in patch section, update version in its target section
**
** Revision 1.1.1.6  1999/05/20 19:01:00  cxue
** Patch database behaviors same as NT
**
** Revision 1.1.1.5  1999/05/05 18:01:22  cxue
** bug 351303, -p option
**
** Revision 1.1.1.4  1999/04/23 22:11:48  cxue
** debugger doesn't like NULL gets passed to strcasecmp
**
** Revision 1.1.1.3  1999/04/22 01:49:25  cxue
** version to 4.2
**
** Revision 1.1.1.2  1999/04/02 18:05:09  cxue
** bug 347418: invisible component check
**
** Revision 1.1.2.93.2.37  1999/02/04 23:57:09  cxue
** resource changes
**
** Revision 1.1.2.93.2.36  1999/02/04 02:24:43  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.93.2.35  1999/01/29 00:04:28  cxue
** Find memory problem, remove commented out code
**
** Revision 1.1.2.93.2.34  1999/01/21 00:15:27  cxue
** Stop supporting downgrade installation
**
** Revision 1.1.2.93.2.33  1999/01/20 01:08:23  cxue
** support UpgradeOnly installation
**
** Revision 1.1.2.93.2.32  1999/01/16 05:59:22  cxue
** change Server Family to Server Products
**
** Revision 1.1.2.93.2.31  1999/01/13 01:32:13  cxue
** remove askRemoveInstallInfo dialog
**
** Revision 1.1.2.93.2.30  1999/01/12 22:27:32  cxue
** remove install.inf by default
**
** Revision 1.1.2.93.2.28  1998/12/23 22:09:49  cxue
** if startconsole is not under serverroot, don't ask user to type startconsole
**
** Revision 1.1.2.93.2.27  1998/12/18 19:37:47  cxue
** change -r to -i, because -r is used for reconfiguration purpose
**
** Revision 1.1.2.93.2.26  1998/12/04 17:30:37  pvo
** Protect install.inf with 0666
**
** Revision 1.1.2.93.2.25  1998/12/04 02:49:21  cxue
** billx uninstallation and customizable destination support
**
** Revision 1.1.2.93.2.24  1998/11/12 21:49:14  cxue
** select parent if child is selected through invisible checking
**
** Revision 1.1.2.93.2.23  1998/11/12 18:35:46  cxue
** BillX:  hide all components for specific inf file (eg. base.inf)
**
** Revision 1.1.2.93.2.22  1998/11/12 02:56:14  cxue
** append o=netscaperoot to config_ldap_url upon reading silent.inf
**
** Revision 1.1.2.93.2.21  1998/11/10 23:59:51  cxue
** log message
**
** Revision 1.1.2.93.2.20  1998/11/10 02:40:37  cxue
** Call common setup api for logging, remove _installLog
**
** Revision 1.1.2.93.2.19  1998/11/06 20:24:56  cxue
** getUserDefault return reference to NVPair
**
** Revision 1.1.2.93.2.18  1998/11/04 19:42:33  pvo
** Enable option for skip build number checking
**
** Revision 1.1.2.93.2.17  1998/11/02 21:39:01  cxue
** compiler warnings
**
** Revision 1.1.2.93.2.16  1998/11/02 21:37:27  cxue
** product name from resource
**
** Revision 1.1.2.93.2.15  1998/10/28 23:47:45  cxue
** move dialog resource from dl-inst to uxres_data
**
** Revision 1.1.2.93.2.14  1998/10/28 19:08:40  pvo
** 310620 - update build number upon re-install
**
** Revision 1.1.2.93.2.13  1998/10/28 17:54:09  cxue
** setup -r infoFile
**
** Revision 1.1.2.93.2.12  1998/10/20 00:01:26  cxue
** select invisible component before pre-install
**
** Revision 1.1.2.93.2.11  1998/10/17 01:15:02  cxue
** read user defined setting from .nssetup-default
**
** Revision 1.1.2.93.2.10  1998/10/15 22:22:06  cxue
** resource framework for custom ui
**
** Revision 1.1.2.93.2.9  1998/10/10 00:35:05  yjh
** Fix build break caused by new unified setup SDK in AIX, HPUX and OSF1
**
** Revision 1.1.2.93.2.8  1998/10/09 21:27:10  cxue
** select invisible components if it has dependency on it and mandatory.
**
** Revision 1.1.2.93.2.7  1998/10/05 23:52:38  cxue
** return error in case setup.inf is not there
**
** Revision 1.1.2.93.2.6  1998/10/02 18:05:37  cxue
** Support mode=express|typical|custom|all modes in [general] section. New hidden attribute for Dialog class
**
** Revision 1.1.2.93.2.5  1998/09/29 00:52:46  cxue
** setup -v to show build number
**
** Revision 1.1.2.93.2.4  1998/09/24 01:57:40  cxue
** In the absence of components in current package, setup checks version number of dependency at server root
**
** Revision 1.1.2.93.2.3  1998/09/22 16:53:15  cxue
** Merge changes from admin4_branch
**
** Revision 1.1.2.93.2.2  1998/09/21 20:12:24  cxue
** 322887. Convert UserGroupURL from UTF8 to local when saving to install.inf
**
** Revision 1.1.2.93.2.1  1998/09/10 16:36:37  cxue
** Install to empty directory even we can't remove it
**
** Revision 1.1.2.93  1998/08/22 00:44:04  cxue
** call insertLdifEntries after calling post-install
**
** Revision 1.1.2.92  1998/08/20 23:44:01  cxue
** Add writable checking if directory exists
**
** Revision 1.1.2.91  1998/08/20 18:06:41  cxue
** getLocalLdapInfo
**
** Revision 1.1.2.90  1998/08/20 07:07:30  cxue
** formConfigLdapURL for silent installation
**
** Revision 1.1.2.89  1998/08/20 02:03:09  cxue
** warnings
**
** Revision 1.1.2.87  1998/08/19 17:49:53  cxue
** Change to support Config only directory server and user only directory server
**
** Revision 1.1.2.86  1998/08/19 01:32:14  pvo
** Add -b option for supporting binary only installation
**
** Revision 1.1.2.85  1998/08/14 23:25:29  cxue
** Common installer won't ask LdapURL question if someone reinstalls directory server
**
** Revision 1.1.2.84  1998/08/13 00:07:43  pvo
** 311803 - Silent install gives 2 paths to log file
**
** Revision 1.1.2.83  1998/07/31 02:56:59  pvo
** Fix version check logic
**
** Revision 1.1.2.82  1998/07/25 06:31:25  cxue
** Bug 302072. add logic to auto-detect UserDirectoryURL from DS4.0. This can avoid to ask UserDirectoryURL many times during installation. Requested by Sunil. Reviewed by Phong.
**
** Revision 1.1.2.81  1998/07/18 01:24:44  pvo
** 202937 - At end of install, inform user to use startconsole
**
** Revision 1.1.2.80  1998/07/16 17:12:23  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.79  1998/07/16 00:25:09  cxue
** Bug 301450. Change file mode for install.inf to 600, because it contains password information. Reviewed by Phong.
**
** Revision 1.1.2.78  1998/07/02 03:30:33  pvo
** Some minor name changes
**
** Revision 1.1.2.77  1998/06/29 22:09:39  pvo
** 225241 - Correct writing UserDirectory info to install.inf
**
** Revision 1.1.2.76  1998/06/28 23:11:23  pvo
** Keyword changes
**
** Revision 1.1.2.75  1998/06/27 03:02:24  pvo
** Name change
**
** Revision 1.1.2.74  1998/06/24 17:41:14  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.73  1998/06/01 21:04:04  pvo
** 118281 - support local encoding in install.inf
**
** Revision 1.1.2.72  1998/05/27 01:49:22  pvo
** 117351 - setup crashes
**
** Revision 1.1.2.71  1998/05/13 04:39:31  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.70  1998/05/07 02:11:25  pvo
** 110399 - Need to reset tty setting
**
** Revision 1.1.2.69  1998/05/07 01:49:35  pvo
** 116345 - Don't ask LDAP question if new DS is installed
**
** Revision 1.1.2.68  1998/05/01 23:24:42  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.67  1998/05/01 01:06:50  pvo
** Fix problem with skipping extraction
**
** Revision 1.1.2.66  1998/04/24 01:36:53  pvo
** Increase some buf size
**
** Revision 1.1.2.65  1998/04/14 23:30:38  pvo
** 113700 - Don't pause if not interactive mode
**
** Revision 1.1.2.64  1998/04/14 01:12:13  pvo
** 113369 - added pause
**
** Revision 1.1.2.63  1998/04/09 03:30:26  pvo
** 113213 - Installer quits after extraction
**
** Revision 1.1.2.62  1998/04/01 02:42:30  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.61  1998/03/28 03:29:02  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.60  1998/03/26 03:35:22  pvo
** 111380 - Setup fails silently if disk full.
**
** Revision 1.1.2.59  1998/03/24 22:29:45  pvo
** 110961 - Setup cannot handle long directory name.
**
** Revision 1.1.2.58  1998/03/13 21:24:48  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.57  1998/03/13 05:00:37  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.56  1998/03/10 02:49:16  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.55  1998/03/09 22:09:37  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.54  2000/03/03 21:39:57  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.53  1998/03/02 23:59:27  pvo
** 106912 - Pickup the right defaults in dialogs.
** 105320 - Improve logging of silent installation.
**
** Revision 1.1.2.52  1998/02/26 04:01:58  pvo
** Fixed break in download.cc due to the call to set_ldap_option() 
** in libinstall.
** Also try to detect input error when asking for installDN and try to
** create installDN if possible in case the user has the privilege 
** to create it.
**
** Revision 1.1.2.51  1998/02/26 00:13:59  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.50  1998/02/25 05:15:07  pvo
** Removed unecessary uninstall question for installDN.
**
** Revision 1.1.2.49  1998/02/24 21:33:36  pvo
** 107637 - Silent installation needs to read DS_INSTALL_DN from script
**
** Revision 1.1.2.48  1998/02/24 00:23:07  pvo
** Support for topology changes.
**
** Revision 1.1.2.47  1998/02/15 22:06:40  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.46  1998/02/08 21:52:04  pvo
** 105971 - Delegation support
**
** Revision 1.1.2.45  1998/02/04 00:01:52  pvo
** 104824 - DEC uninstall goes out of control
**
** Revision 1.1.2.44  1998/01/30 21:45:32  pvo
** 105219 - Need to retain DS info as specified in silent script
**
** Revision 1.1.2.43  1998/01/29 21:52:23  pvo
** Bug 105041 - add -h (help) screen for setup
** Bug 104968 - change default LDAP suffix to domain.com rather than example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.42  1998/01/23 03:10:53  pvo
** Additional code for writing nyr.
**
** Revision 1.1.2.41  1998/01/23 01:42:09  pvo
** Support writing NYR into DS.
**
** Revision 1.1.2.40  1998/01/22 04:51:28  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.39  1998/01/15 18:13:36  pvo
** Fixed problem with Silent install not writing answer cache.
**
** Revision 1.1.2.38  1998/01/09 03:27:29  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.37  1997/12/18 02:10:22  pvo
** Fixed problem on HPUX with tmpnam.
**
** Revision 1.1.2.36  1997/12/13 02:40:08  pvo
** Support for new LDAP schema.
**
** Revision 1.1.2.35  1997/12/12 21:54:20  pvo
** Various bug fixes.
**
** Revision 1.1.2.34  1997/12/04 00:03:12  pvo
** Prepare support for DS installation.
**
** Revision 1.1.2.33  1997/12/03 00:50:36  pvo
** Package suitespot.dif.
**
** Revision 1.1.2.32  1997/12/02 20:39:08  pvo
** Moved suitespot.inf to ./inf directory.
** Fixed problem with ns-config being called twice.
** Silent Installation support.
**
** Revision 1.1.2.31  1997/12/02 04:28:28  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.30  1997/11/26 02:02:57  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.29  1997/11/17 00:02:31  pvo
** Added umask(022);
**
** Revision 1.1.2.28  1997/11/16 22:34:07  pvo
** Added back installType.
**
** Revision 1.1.2.27  1997/11/14 09:11:31  pvo
** Various changes to strengthen code.
**
** Revision 1.1.2.26  1997/11/13 19:28:40  pvo
** More new stuffs. Changed .pi to .inf.
**
** Revision 1.1.2.25  1997/11/11 23:17:34  pvo
** Implemented preUninstall and postUninstall.
**
** Revision 1.1.2.24  1997/11/11 05:41:09  pvo
** Added dependency checking for uninstallation.
**
** Revision 1.1.2.23  1997/11/09 19:30:11  pvo
** More code for uninstallation. Changed ns-remove.cc to uninstall.cc.
** Removed second password prompt when asking Ldap user.
**
** Revision 1.1.2.22  1997/11/08 16:52:02  pvo
** UnInstallation support.
**
** Revision 1.1.2.21  1997/10/29 01:43:01  pvo
** Modular Installation support.
**
** Revision 1.1.2.20  1997/10/24 03:39:33  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.19  1997/10/23 03:23:34  pvo
** More change to support other platforms.
**
** Revision 1.1.2.18  1997/10/22 23:58:51  pvo
** Warning removal project!
**
** Revision 1.1.2.17  1997/10/22 02:40:48  pvo
** Too many changes to list out here.
**
** Revision 1.1.2.16  1997/10/16 04:24:10  pvo
** Quite a lot of code changes for support updating and ACIs.
**
** Revision 1.1.2.15  1997/10/15 01:02:22  pvo
** Fixed include problem.
**
** Revision 1.1.2.14  1997/10/14 05:01:57  pvo
** More code.
**
** Revision 1.1.2.13  1997/10/10 04:25:35  pvo
** More supported for zip.
**
** Revision 1.1.2.12  1997/10/10 00:36:00  pvo
** Added support for zip format.
**
** Revision 1.1.2.11  1997/10/02 01:34:20  pvo
** More lenient on certain errors.
**
** Revision 1.1.2.10  1997/10/01 19:56:28  pvo
** Fixed bug with executing preInstall program.
**
** Revision 1.1.2.9  1997/09/29 22:51:34  pvo
** Additional code.
**
** Revision 1.1.2.8  1997/09/27 02:47:16  pvo
** Checked in.
**
*********************************************************************/

extern "C" {
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>

}

#include "ldapu.h"
#include "ux-util.h"
#include "installer.h"
#include "wizard.h"
#include "dl-common.h"
#include "setupapi.h"
#include "setupinst.h"

extern Bool unzipRename;

Installer::Installer (int argc, char **argv)
   :_installMode(Interactive), _installType(Typical),_checkBuildNum(True),_keepInstallCache(False),_preInstallOnly(False),_checkDomainName(True)
{
   /* 
    * Set up the Installer
    */

   _baseDir = InstUtil::dirName(argv[0]);
   _baseFile = SUITESPOT_INF;

   if (_baseDir == ".")
   {
     _baseDir = _workDir;
   }
   else if (_baseDir[0] != '/')
   {
      // Not a full path name -- asume from the currentDir
      _baseDir = _workDir + "/" + _baseDir;
   }

   getOptions(argc, argv);
}

Installer::~Installer()
{
   if (_dialogManager)
      _dialogManager->disableWinMode();
}
   
void
Installer::getOptions(int argc, char **argv)
{
   int opt;
   char *t;

   while ((opt = getopt(argc,argv, "?hb:sf:vi:Bkp")) != -1)
   {
      switch (opt)
      {
         case 's':
           _installMode = Silent;
           break;
         case 'g':
           _installMode = ScriptGen;
           break;
         case 'b':
           _installMode = BinaryOnly;
           _serverRoot  = optarg;
            break;
         case 'B':
           _checkBuildNum = False;
           break;
         case 'f':
           t = strchr(optarg,'/');
           if (!t)
           {
              _inputFile = _workDir + "/" + optarg;
           }
           else
           {
              _inputFile = optarg;
           }
           break;
          case 'h':
            printf("%s Setup Program Version 1.0\n", (const char *) getResource(RES_GLOBAL_PRODUCT));
            printf("Usage: setup [-options] [argument]\n");
           printf("Options:\n");
           printf("   -h            print out this message\n");
           printf("   -s            silent installation mode\n");
           printf("   -f <file>     specify <file> as silent installation script\n");
           printf("   -b <dir>      only install binaries, but no configuration\n");
           printf("   -k            keep installation cache\n");
           exit(0);
         case 'v':
            printf("Setup Program Version 1.0 build %s\n", BUILD_NUM);
           exit(0);
            break;
         case 'i':  
           // HIDDEN feature for automation testing
           // load setup.inf from specified directory
           if (optarg[strlen(optarg) - 1] == '/')
           {
              optarg[strlen(optarg) - 1] = '\0';
              _baseDir = optarg;
              optarg[strlen(optarg) - 1] = '/';
           }
           else
              _baseDir = optarg;

           if (setupDirExists(_baseDir) == False && setupFileExists(_baseDir) == True)
           {
               _baseFile = InstUtil::fileName(_baseDir);
               _baseDir = InstUtil::dirName (_baseDir);
           }

           if (_baseDir == ".")
              _baseDir = _workDir;
           else if (_baseDir[0] != '/')
              _baseDir = _workDir + "/" + _baseDir;


            break;
         case 'k':
            _keepInstallCache = True;
            break;            
         case 'p':
            _preInstallOnly = True;
            break;            
         default:
           exit(1);
           break;
       }
   }
}


int
Installer::init()
{
   umask(022);
   char buf[SML_BUF];
   int err;

   if ((err = getNewProducts(_baseDir, _baseFile)) != 0)
   {
     if (err == -1)
        cerr << "ERROR: Information file (setup.inf) not found" << endl;
     else
        cerr << "ERROR: Failed to retrieve product information file" << endl;

      return -1;
   }

   DialogManager::enableWinMode();

   if (_installMode == Silent)
   {
      if (_inputFile == (char *) NULL || InstUtil::fileExists(_inputFile) == False)
      {
         cerr << "ERROR:Silent Installation script not found" << endl;
         return -1;
      }

      // Load Install Script
      _installInfo = new InstallInfo (_inputFile);

      if (_installInfo->get(CONFIG_LDAP_URL))
      {
         char *s;
         s = formConfigLdapURL(_installInfo->get(CONFIG_LDAP_URL));
         _installInfo->set(CONFIG_LDAP_URL, s);
         free(s);
      }
      _installInfo->toUTF8(CONFIG_LDAP_URL);
      _installInfo->toUTF8(DS_ADMIN_DOMAIN);
      _installInfo->toUTF8(MC_ADMIN_ID);
      _installInfo->toUTF8(UG_LDAP_URL);
      _installInfo->toUTF8(UG_ADMIN_ID);

      // Change _infoFile so we could modify it if necessary
      _infoFile = tmpnam(buf);
   }
   else
   {
      _installInfo = new InstallInfo();
      _infoFile = tmpnam(buf);
      initDefaultAnswer (_installInfo);
   }

   chmod(_infoFile.data(), S_IRUSR | S_IWUSR);

   startLogging();


   // initialize resource, so we can show custom dialogs

   NVPair customResource;
   const char *filename;
   const char *p;

   filename = _newSuiteSpot.get("resource");

   if (filename)
   {
      NSString resourceFile;
      resourceFile = _baseDir + "/" + filename;
      customResource.read(resourceFile);
   }

   // Even without resource file defined, we still want to
   // customize product and vendor name 
   if (p = _newSuiteSpot.get(SETUP_INF_VENDOR))
      customResource.set(SETUP_INF_VENDOR, p);
   if (p = _newSuiteSpot.get(SETUP_INF_NAME))
      customResource.set(SETUP_INF_NAME, p);

   initResource(&customResource);

   initDialog();

   return 0;
}

void
Installer::initDefaultAnswer(InstallInfo *ans)
{
   const char *p;
   NVPair *userInfo;

   if (ans == NULL)
     return;

   ans->set("SetupType", "Setup"); // Indicate this is install, not uninstall

   ans->set(MACHINE_NAME, InstUtil::guessHostname());

   p = _newSuiteSpot.get(SETUP_INF_MODE);
   if (p != NULL  && strcasecmp(p, SETUP_INF_ALL_MODES))
   {
      if (strcasecmp(p, "Express") == 0)         
         ans->set(INSTALL_TYPE, EXPRESS_TYPE);
      else if (strcasecmp(p, "custom") == 0)         
         ans->set(INSTALL_TYPE, CUSTOM_TYPE);
      else         
         ans->set(INSTALL_TYPE, TYPICAL_TYPE);
   }
   else
   {   
      ans->set(INSTALL_TYPE, DEFAULT_INSTALL_TYPE);
   }

   ans->set(CONFIG_LDAP_URL, NSString("ldap://")
                      + ans->get(MACHINE_NAME) 
                      + ":389/" 
                      + "o=NetscapeRoot");
   if (InstUtil::isSelfRootUser() == True)
   {
      ans->set(SS_USER_ID, DEFAULT_SSUSER);
      ans->set(SS_GROUP, DEFAULT_SSGROUP);
   }
   else
   {
      ans->set(SS_USER_ID, InstUtil::getSelfUserID());
      ans->set(SS_GROUP, InstUtil::getSelfGroupID());
   }

   ans->set(MC_ADMIN_ID, DEFAULT_ADMINUSER);
   ans->set(MC_ADMIN_PWD, DEFAULT_ADMINPWD);

   if (p = _newSuiteSpot.get(SETUP_INF_INSTALL_DIR))
      ans->set(SERVER_ROOT, p);

   p = _newSuiteSpot.get(SETUP_INF_SECURITYCHECK);
   if (p != NULL  && strcasecmp(p, "True"))
   {
     Product::_needSecurity = False;
     ans->set(SETUP_INF_SECURITYCHECK, p);
   }

   p = _newSuiteSpot.get(SETUP_INF_REQUIREDOMAIN);
   if (p != NULL  && strcasecmp(p, "True"))
   {
     _checkDomainName = False;
      ans->set(SETUP_INF_REQUIREDOMAIN, p);
   }

   /* hidden feature for type saving:
      read default setting from HOME/.nssetup-default 
    */
   userInfo = getUserDefault();

   if (p = userInfo->get(SS_USER_ID))
      ans->set(SS_USER_ID, p);
   if (p = userInfo->get(SS_GROUP))
      ans->set(SS_GROUP, p);
   if (p = userInfo->get(MC_ADMIN_ID))
      ans->set(MC_ADMIN_ID, p);
   if (p = userInfo->get(CONFIG_LDAP_URL))
      ans->set(CONFIG_LDAP_URL, p);
   if (p = userInfo->get(INSTALL_TYPE))
      ans->set(INSTALL_TYPE, p);
   if (p = userInfo->get(MACHINE_NAME))
      ans->set(MACHINE_NAME, p);
   if (p = userInfo->get(SERVER_ROOT))
      ans->set(SERVER_ROOT, p);
}

int
Installer::initDialog()
{
   if (_installMode == Silent || _installMode == BinaryOnly)
   {
      DialogManager::disableWinMode();
   }
   else
   {
      
      Dialog::initDisplay((_newSuiteSpot.package())->get(SETUP_INF_NAME),
                          (_newSuiteSpot.package())->get(SETUP_INF_VENDOR));

      // Create the dialog manager
      _dialogManager = new Wizard();
      _dialogManager->setInstallMode(_installMode);
      _dialogManager->setInstallType(_installType);
      _dialogManager->setParent(this);

      _dialogManager->setDefaultScript(_installInfo);
   }

   return 0;
}

void
Installer::run()
{
   int rc = 0;
   const char *s;

   /*
    * Start the first stage - common installer's questions
    */
   
   printNotice();
   prepreInstall();

   if (_installMode == Silent)
   {
      setupLogMessage(INFO, "Setup", "Start silent installation...");     
      rc = doSilentInstall();
   }
   else if (_installMode == Interactive)
   {
      setupLogMessage(INFO, "Setup", "Start interactive installation...");
      rc = doInteractiveInstall();
   }
   else if (_installMode == BinaryOnly)
   {
      setupLogMessage(INFO, "Setup", "Start binary installation...");
      rc = doBinaryInstall();
   }

   if (rc != 0)
   {
      setupLogMessage(FATAL, "Setup", "Can't initialize installation!");
   }

   if (rc == 0)
   {
      /*
         For server installation:
           select component on other's dependency list if not selected 
      */

      if (_installInfo->get(SETUP_INF_INSTALLWHICH) &&
          strcasecmp(_installInfo->get(SETUP_INF_INSTALLWHICH), SETUP_INF_INSTALLCLIENTS))
      {
         selectComponentDependency(getInstallableComponents(), getInstalledComponents());
      }

      setupLogMessage(INFO, "Setup", "PreInstall phrase...");
      if (rc = preInstall())
      {
        setupLogMessage(FATAL, "Setup", "PreInstall failed!");
      }

      /*
       * Fourth stage - installing server binaries
       */
      if (rc == 0 && _preInstallOnly == False)
      {
         setupLogMessage(INFO, "Setup", "Unzip component binaries...");
         if (rc = install())
         {
            setupLogMessage(FATAL, "Setup", "Unzip failed!");
         }
      }

      /*
       * Fifth stage - configure servers
       */
      if (rc == 0 && _preInstallOnly == False)
      {
        setupLogMessage(INFO, "Setup", "PostInstall phrase...");
        if (rc = postInstall())
        {
            setupLogMessage(FATAL, "Setup", "PostInstall failed!");
        }
      }

      if (_installMode == Interactive)
         Dialog::pause();

      cleanUp();

   }

   stopLogging();

   if (_installMode == Silent)
   {
      printf("\nInstallation is complete.\n");
      printf("See %s for possible errors.\n\n", _logFile.data());
   }
   else if (rc == 0 && _installMode == Interactive)
   {
      NSString startconsole;
      const char *s;
      _dialogManager->disableWinMode();

      /*
       * if there is customized finish message, take it.
       */
      if ((s = _installInfo->get(SETUP_INF_FINISHMESSAGE)) == NULL)
      {
        s = _newSuiteSpot.get(SETUP_INF_FINISHMESSAGE);
      }
      if (s != NULL)
      {
        char *msg;
        msg = setupRawText2String(s);        
        printf("\n%s\n", msg);
      }
      else
      {
        startconsole = _serverRoot + "/startconsole";
        if (setupFileExists(startconsole))
        {
          printf("\nGo to %s and type startconsole to begin\n", _serverRoot.data());
          printf("managing your servers.\n\n");
        }
      }
   }

   return;
}


void
Installer::cleanUp()
{
   // Restore files that were backed up
   Components *q = getInstallableComponents();

   for (int i = 0; i < q->numComponents(); i++)
   {
      Product *p = q->getComponent(i);

      if (p->isSelected() && !p->skipped())
      {
         const char *s = p->get("Destination");
         if (s != NULL)
         {
            p->restoreFiles(s);
         }
         else
         {
            p->restoreFiles(_serverRoot);
         }
      }
   }

   // Now relocate install.inf to
   if (_installMode != Silent)
   {
      // Relocate install.inf to server root setup dir
      NSString newInstallInfo = _serverRoot + "/" + SETUP_DIR + "/" + INSTALL_INF;
      NSString setupDirectory = _serverRoot + "/" + SETUP_DIR;
      char *stripedURL;

      if (!setupDirExists(setupDirectory))
      {
         setupCreateDir(setupDirectory, 0755);
      }

      _installInfo->remove("SetupType");
      _installInfo->remove(INSTALL_TYPE);
      _installInfo->remove("InstallWhich");
      _installInfo->remove("SelectedComponents");
      _installInfo->remove("InstalledComponents");

      _installInfo->toLocal(CONFIG_LDAP_URL);
      /*
       * Strip LdapURL before saving to installation cache file
       */
      stripedURL = stripConfigLdapURL(_installInfo->get(CONFIG_LDAP_URL));
      _installInfo->set(CONFIG_LDAP_URL, stripedURL);
      free(stripedURL);

      _installInfo->toLocal(DS_ADMIN_DOMAIN);
      _installInfo->toLocal(MC_ADMIN_ID);

      if (needUserDirAuth() == False)
      {
         Product *ds = getInstallableComponents()->getComponent("slapd");

         // We only want to remove this if ds is not being installed
         if (ds == NULL || ds->isSelected() == False)
         {
            _installInfo->remove(UG_LDAP_URL);
            _installInfo->remove(UG_ADMIN_ID);
            _installInfo->remove(UG_ADMIN_PWD);
         }
      }

      _installInfo->toLocal(UG_LDAP_URL);
      _installInfo->toLocal(UG_ADMIN_ID);

      _installInfo->setFormat(1);

      _installInfo->write(newInstallInfo);

      // change mode to 600 (read/write only by server installer)
      chmod(newInstallInfo.data(), S_IRUSR | S_IWUSR);

      if (_installMode == Interactive)
      {
         if (_keepInstallCache == False)
            unlink(newInstallInfo.data());
      }
   }
   unlink(_infoFile.data());
}

int
Installer::doInteractiveInstall()
{
   welcome.registerDialogNext(_dialogManager);
   return _dialogManager->start();
}

int
Installer::doSilentInstall()
{
   char **servers;
   char **modules;
   const char *s;
   const char *ldapURL = NULL, *ldapUser = NULL, *ldapPwd=NULL, *installDN=NULL;
   NSString errMsg;
   int i, j;
   int err = 0;
   Product *p;

   if (_newSuiteSpot.numComponents() == 0)
   {
      setupLogMessage(FATAL, "Setup", "Either setup.inf is missing or no installable components found");
      err = -1;
   }

   servers = _installInfo->getListItems(SERVER_LIST);
   if (servers == NULL)
   {
      setupLogMessage(FATAL, "Setup", "No components are specified to be installed.");
      err = -1;
   }

   // Get server root
   if ((s = _installInfo->get(SERVER_ROOT)) == NULL)
   {
      setupLogMessage(FATAL, "Setup", "ServerRoot is not specified.");
      err = -1;
   }
   else
   {
      _serverRoot = s;
      errMsg = checkServerRoot(s);
      if (errMsg != (char *) NULL)
      {
         setupLogMessage(FATAL, "Setup", errMsg.data());
         err = -1;
      }
   }

   if (err)
   {
      return err;
   }

   // Now select all the components to install
   Components *components = _newSuiteSpot.getComponents();

   i = 0;
   while (servers[i] != NULL)
   {
      p = components->getComponent(servers[i]);
      if (p)
      {
         p->select();
         if (p->numComponents())
         {
            Components *sc = p->getComponents();
            NVPair *sp = _installInfo->getSection(p->get(PKG_NICKNAME));
            if (sp != NULL)
            {
               modules = sp->getListItems(PKG_COMPONENTS);
               if (modules == NULL)
               {
                  setupLogMessage(WARN, "Setup", "No selected module for %s", p->get(PKG_NICKNAME));
                  setupLogMessage(WARN, "Setup", "Skip %s", p->get(PKG_NICKNAME));
                  p->deSelect();
               }
               else
               {
                  j = 0;
                  while (modules[j] != NULL)
                  {
                     Product *q = sc->getComponent(modules[j]);
                     if (q)
                     {
                        q->select();
                     }
                     ++j;
                  }
               }
            }
         }
      }
      i++;
   }

   // Verify the selections
   errMsg = checkInstallable(components);
   if (errMsg == (char *) NULL)
   {
      errMsg = checkComponentsDependency(getInstallableComponents());
   }

   if (errMsg != (char *) NULL)
   {
      setupLogMessage(FATAL, "Setup", errMsg.data());
      err = -1;
      return err;
   }

   // Verify host name
   if ((s = _installInfo->get(MACHINE_NAME)) == NULL)
   {
      err = -1;
      setupLogMessage(FATAL, "Setup", "FullMachineName is not specified");
   }

   // Verify SSUser and Group
   if ((s = _installInfo->get(SS_USER_ID)) == NULL)
   {
      setupLogMessage(FATAL, "Setup", "SuiteSpotUserID is not specified");
      return -1;
   }

   errMsg = verifySSUser(s);
   if (errMsg != (char *) NULL)
   {
      setupLogMessage(FATAL, "Setup", errMsg.data());
      return -1;
   }

   if ((s = _installInfo->get(SS_GROUP)) == NULL)
   {
      setupLogMessage(FATAL, "Setup", "SuiteSpotGroup is not specified");
      return -1;
   }

   errMsg = verifySSGroup(_installInfo->get(SS_USER_ID),s);
   if (errMsg != (char *) NULL)
   {
      err = -1;
      setupLogMessage(FATAL, "Setup", errMsg.data());
   }

   // Ldap stuffs and we are finito

   if (needLdapInfo() == False)
   {
      ;
   }
   else
   {
      if (_installInfo->get(CONFIG_LDAP_URL) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "ConfigDirectoryLdapURL is not specified");
         return -1;
      }
      ldapURL = formConfigLdapURL(_installInfo->get(CONFIG_LDAP_URL));
      _installInfo->set(CONFIG_LDAP_URL, ldapURL);

      errMsg = verifyLdapServer(ldapURL);
      if (errMsg != (char *) NULL)
      {
         setupLogMessage(FATAL, "Setup", errMsg.data());
         return -1;
      }

      if ((ldapUser = _installInfo->get(MC_ADMIN_ID)) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "ConfigDirectoryAdminID is not specified");
         return -1;
      }
      if ((ldapPwd = _installInfo->get(MC_ADMIN_PWD)) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "ConfigDirectoryAdminPwd is not specified");
         return -1;
      }
      errMsg = verifyLdapUser(ldapURL, ldapUser, ldapPwd);
      if (errMsg != (char *) NULL)
      {
         setupLogMessage(FATAL, "Setup", errMsg.data());
         return -1;
      }
      if ((installDN = _installInfo->get(DS_ADMIN_DOMAIN)) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "AdminDomain is not specified");
         return -1;
      }
      errMsg = verifyInstallPrivilege(ldapURL, ldapUser, ldapPwd, installDN);
      if (errMsg != (char *) NULL)
      {
         setupLogMessage(FATAL, "Setup", errMsg.data());
         return -1;
      }
   }

   if (needUserDirAuth() == False)
   {
      ;
   }
   else
   {
      if ((ldapURL = _installInfo->get(UG_LDAP_URL)) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "UserDirectoryLdapUrl not specified");
         return -1;
      }

      errMsg = verifyLdapServer(ldapURL);
      if (errMsg != (char *) NULL)
      {
         setupLogMessage(FATAL, "Setup", errMsg.data());
         return -1;
      }

      if ((ldapUser = _installInfo->get(UG_ADMIN_ID)) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "UserDirectoryAdminID is not specified");
         return -1;
      }
      if ((ldapPwd = _installInfo->get(UG_ADMIN_PWD)) == NULL)
      {
         setupLogMessage(FATAL, "Setup", "UserDirectoryAdminPwd is not specified");
         return -1;
      }

      errMsg = verifyLdapUser(ldapURL, ldapUser, ldapPwd);

      if (errMsg != (char *) NULL)
      {
         setupLogMessage(FATAL, "Setup", errMsg.data());
         return -1;
      }
   }

   return err;
   
}   

int
Installer::doBinaryInstall()
{
   const char *s;
   NSString errMsg;
   int err = 0;

   if (_newSuiteSpot.numComponents() == 0)
   {
      Dialog::showString("Nothing to install. Quit");
      err = -1;
   }

   // Get server root
   if (_serverRoot == (char *) NULL)
   {
      Dialog::showString("Please specify the install location");
      err = -1;
   }
   else
   {
      errMsg = checkServerRoot(_serverRoot);
      if (errMsg != (char *) NULL)
      {
         Dialog::showString(errMsg);
         err = -1;
      }
   }

   if (err)
   {
      return err;
   }

   _installInfo->set(SERVER_ROOT, _serverRoot);

   // Now select all the components to install
   _newSuiteSpot.selectAll();

   // Verify the selections
   errMsg = checkInstallable(_newSuiteSpot.getComponents());
   if (errMsg == (char *) NULL)
   {
      errMsg = checkComponentsDependency(getInstallableComponents());
   }

   if (errMsg != (char *) NULL)
   {
      Dialog::showString(errMsg);
      err = -1;
   }

   return err;
}   

int
Installer::getNewProducts(const NSString & sourceDir, const char *infName)
{
  NSString infFile;

  infFile = sourceDir + "/";
  infFile = infFile + infName;

  if (InstUtil::fileExists(infFile) == False)
  {
     return -1;
  }

  if (sourceDir == _sourceDir && _newSuiteSpot.numComponents())
  {
     return 0;
  }

  if (_newSuiteSpot.retrieveSuiteSpot(sourceDir, infName) == 0 && _newSuiteSpot.numComponents())
  {
     _newSuiteSpot.getComponents()->sortComponents();
     _newSuiteSpot.select();
     _newSuiteSpot.getAllComponents(*_newComponents); // Flatten the structure
     _newComponents->sortComponents();
     _sourceDir = sourceDir;
     return 0;
  }
  return -2;
}

void
Installer::refreshNewProducts()
{
  /*
   *  Setup skips products which are not compatible with the ones in 
   *  existing server root. Refresh will return it back.
   */

   Product *pNew;
   int i;

   for (i = 0; i < _newComponents->numComponents(); i++)
   {
      pNew = _newComponents->getComponent(i);
      if (pNew->skipped() == True)
      {
        pNew->skip(False);
      }
   } 
}


void
Installer::checkInstalled()
{
   // Go through all found products, check them
   // against current installation if any
   
   Product *newComponent;
   Product *oldComponent;
   const   char *name;

   for (int i = 0; i < _newComponents->numComponents(); i++)
   {

      newComponent = _newComponents->getComponent(i);
      newComponent->exists(False);
      newComponent->installed(False);
      newComponent->compatible(True);
      newComponent->securityChecked(True);

      if (_oldComponents->numComponents() == 0)
      {
         // If no existing installed product, it's OKAY
         continue;
      }

      name = newComponent->get(PKG_NICKNAME);

      oldComponent = _oldComponents->getComponent(name);
      if (oldComponent == NULL)
      {
         continue;
      }

      newComponent->exists(True);

      // Compare two product packages to see if they are compatible
      if (newComponent->isIdentical(oldComponent))
      {
         newComponent->installed(True);
      } 
      else 
      {
         int compatible;
         compatible = newComponent->checkCompatibility(oldComponent,_checkBuildNum);
         if (compatible < 0 )
         {
            newComponent->compatible(False);
            if (compatible == -2)
            {
               newComponent->downgrade(True);
            }
         }
         if (newComponent->checkSecurity(oldComponent) < 0)
         {
            newComponent->securityChecked(False);
         }
      }
   }
}
   
int
Installer::doPreOrPostInstall(Product &product, const char *which)
{
   int err = 0;
   NSString program;
   NSString sourceDir;
   char tmp[XSM_BUF];

   if (product.skippedConfig())
   {
      if (strcasecmp(which, PKG_POSTINSTALL) == 0)
      {
         /* start program (server instance) here
          */
         const char *s;
         if ((s = product.get(PKG_START_PROGRAM)) != NULL)
         {
            NSString program;
            if (s[0] == '/')
            {
               program = s;
            }
            else
            {
               program = _serverRoot + "/" +s;
            }

            InstUtil::execProgram(program);
         }      
      }
      return err;
   }

   if (product.numComponents() == 0)
   {
      // This is a terminal package
      _dialogManager->disableWinMode();

      if (!strcasecmp(which, PKG_PREINSTALL))
      {
         err = product.preInstall(_serverRoot, _sourceDir,
                   _installInfo->get(INSTALL_TYPE),
                   _installMode, _logFile,
                   _infoFile);
         err = WEXITSTATUS(err);
         if (err == 7)
         {
            err = 0;
            product.skipConfig();
         }
      }
      else
      {
         err = product.postInstall(_serverRoot, _sourceDir,
						 _infoFile, _installInfo, _installMode, _logFile);
      }

      _dialogManager->disableWinMode();

      if (err)
      {
         // Deselect this component and all components that
         // depend on it
         product.skip(True);
         if (_installMode == Silent)
         {
            setupLogMessage(FATAL, "Setup", "Failure installing %s\n.", product.get(PKG_NAME));
            return err;
         }
         else
         {
            printf("ERROR. Failure installing %s. Do you want to continue [n]? ", product.get(PKG_NAME));
            fflush(stdout);
            fgets(tmp, sizeof(tmp), stdin);
            if (tmp[0] == 'n' || tmp[0] == 'N' || tmp[0] == '\0' || (tmp[0] < 0x20))
            {
               terminate(1);
            }
         }
      }
   }

   for (int i = 0; i < product.numComponents(); i++)
   {
      Product *p = product.getComponent(i);
      if (p->isSelected() && !p->skipped())
      {
         err = doPreOrPostInstall(*p, which);
         if (err)
            return err;
      }
   }

   return err;
}
   
void
Installer::prepreInstall()
{
   const char *s, *program_array[MAX_ALLOWED_PREPREINSTALL];
   int MAXNUM=0;
   char tmp[XSM_BUF];

   memset(tmp, '\0', XSM_BUF);
   memset(program_array, '\0', (MAX_ALLOWED_PREPREINSTALL*sizeof(char *)));

   DialogManager::disableWinMode();
   for (int i = 0; i < _newSuiteSpot.numComponents(); i++)
   {
      Product *p = _newSuiteSpot.getComponent(i);
      if (s = p->get(PKG_PREPREINSTALL))
      {
         if (MAXNUM == MAX_ALLOWED_PREPREINSTALL)
         {
            printf("ERROR: Too many PrePreInstall programs found.\n");
            exit(1);
         }
         program_array[MAXNUM++] = s;
         program_array[MAXNUM] = NULL;
      }

      for (int j = 0; j < p->numComponents(); j++)
      {
         Product *pp = p->getComponent(j);
         if (s = pp->get(PKG_PREPREINSTALL))
         {
            if (MAXNUM == MAX_ALLOWED_PREPREINSTALL)
            {
               printf("ERROR: Too many PrePreInstall programs found.\n");
               exit(1);
            }
            program_array[MAXNUM++] = s;
            program_array[MAXNUM] = NULL;
         }
      }
   }

   if (MAXNUM)
   {
      NSString program;
      for ( int i = 0; i < MAXNUM ; i++)
      {
         if (program_array[i][0] == '/')
         {
            program = program_array[i];
         }
         else
         {
            program = _sourceDir + "/" + program_array[i];
         }

         printf("\nExecuting %s...\n", program.data());
         fflush(stdout);
         InstUtil::execProgram(program);
         if (_installMode != Silent) 
         {
            printf("%s", "Continue [n]? ");
            fflush(stdout);
            fgets(tmp, sizeof(tmp), stdin);
            if (tmp[0] == 'n' || tmp[0] == 'N' || tmp[0] == '\0' || (tmp[0] < 0x20))
            {
               exit(0);
            }
         }
      } // end of for;; for MAXNUM
   } // end of if (MAXNUM)
}

int
Installer::preInstall()
{
   int err = 0;

   if (_installMode == BinaryOnly) return 0;

   /*
    * Write answers so others can read it
    */
   _installInfo->remove("SetupType");
   _installInfo->setFormat(1);

   _installInfo->toLocal(CONFIG_LDAP_URL);
   _installInfo->toLocal(DS_ADMIN_DOMAIN);
   _installInfo->toLocal(MC_ADMIN_ID);
   _installInfo->toLocal(UG_LDAP_URL);
   _installInfo->toLocal(UG_ADMIN_ID);

   _installInfo->write(_infoFile);

   for (int i = 0; i < _newSuiteSpot.numComponents(); i++)
   {
      Product *p = _newSuiteSpot.getComponent(i);
      if (p->isSelected())
      {
         if (_installMode != Interactive)
         {
            setupLogMessage(START, "Setup", "Installing %s", p->get(PKG_NAME));
            cout << "Installing " << p->get(PKG_NAME) << endl;
         }

         err = doPreOrPostInstall(*p, PKG_PREINSTALL);
         if (err)
         {
            p->skip(True);
         }
      }
   }

   /*
    * Re-read cache
    */

   delete _installInfo;
   _installInfo = new InstallInfo (_infoFile);

   _installInfo->toUTF8(CONFIG_LDAP_URL);
   _installInfo->toUTF8(DS_ADMIN_DOMAIN);
   _installInfo->toUTF8(MC_ADMIN_ID);
   _installInfo->toUTF8(UG_ADMIN_ID);
   _installInfo->toUTF8(UG_LDAP_URL);

   return err;
}


Bool Installer::checkDiskSpace()
{
  long availSpace;
  long needSpace;

  availSpace = setupGetDiskFreeSpace((const char *) _serverRoot);

  needSpace =  requiredDiskSpace();

  if (needSpace > availSpace)
  {
     return False;
  }

  return True;
}

  
int
Installer::install()
{
   int err;

   err = installSuiteSpot();

   updateSuiteSpotInfo();

   return err;
}

long Installer::requiredDiskSpace()
{
  long total;
  Product *p;

  total = 0;

  for (int i = 0; i < _newComponents->numComponents(); i++)
  {
    p = _newComponents->getComponent(i);
    if (p->isSelected())
    {      
      total += p->requiredDiskSpace(_sourceDir, _installInfo);
    }
  }

  return total;
}

int
Installer::installSuiteSpot()
{
   int err = 0;
   NSString ssFlag = _serverRoot + "/.suitespot";
   Product & suitespot = _newSuiteSpot;
   NSString errMsg;
   NSString dest;
   NVPair *section;

   DialogManager::newPage();

   // Mark this server root as 4.0 server root
   fclose(fopen(ssFlag.data(), "w"));

   for (int i = 0; i < suitespot.numComponents(); i++)
   {
      Product *p = suitespot.getComponent(i);

      if (p->isSelected())
      {
         if (p->skipped())
         {
            if (_installMode == Interactive)
            {
               Dialog::showString(NSString("Skipping ") + p->get(PKG_NAME) + "...\n");
            }
            else
            {
               setupLogMessage(WARN, "Setup", "Skipping %s ...\n", p->get(PKG_NAME));
               printf("Warning: skipping %s ...\n", p->get(PKG_NAME));
               fflush(stdout);
            }
         }
         else if (p->numComponents() == 0)
         {
             // This is a standalone package

            if (_installMode == Interactive)
            {
               Dialog::showString(NSString("Extracting ") + p->get(PKG_NAME) + "...\n");
            }
            else
            {
               setupLogMessage(INFO, "Setup", "Extracting %s ...\n", p->get(PKG_NAME));
               printf("Extracting %s ...\n", p->get(PKG_NAME));
               fflush(stdout);
            }
            errMsg = p->install(_serverRoot, _sourceDir, _installInfo);
            if (errMsg != (char *) NULL)
            {
               err = -1;
               if (_installMode == Interactive)
               {
                  Dialog::showString(NSString("ERROR: ") + errMsg);
               }
               else
               {
                  printf("ERROR: %s\n", errMsg.data());
                  fflush(stdout);
                  setupLogMessage(FATAL, "Setup", 
                      "Error extracting %s (%s)", p->get(PKG_NAME), errMsg.data());

               }
               terminate(1);
            }
                
         }
         else
         {
             err = doInstall(*p);
         }

      }

      // If err, PVO, deselect this package an all dependent
      if (err)
      {
         p->skip(True);
      }
   }

   /*
    * warning messages about *.deleteme
    */
   if (unzipRename == True)
   {
      errMsg = NSString("Warning: ") + 
         "The Setup program cannot delete locked files at this time,\n"
         "so it may rename files with the extension .deleteme. You can\n"
         "delete all files with the extension .deleteme at a later time.\n";


      if (_installMode == Interactive)
      {
         DialogAlert alert(errMsg);
         alert.execute();
      }
      else
      {
         setupLogMessage(WARN, "Setup", errMsg);
      }
   }
   return err;
}

int
Installer::doInstall(const Product &product)
{
   Product *p;
   int err = 0;
   NSString installedDir;
   NSString errMsg;
   Components *products = product.getComponents();

   for (int i = 0; i < product.numComponents(); i++)
   {
      err = 0;
      p = product.getComponent(i);
      if (p->isSelected())
      {
         // Skip this server if one or more servers it depends on
         // had been deselected
         if (p->skipped())
         {
            if (_installMode == Interactive)
            {
               Dialog::showString(NSString("Skipping ") + p->get(PKG_NAME) + "...\n");
            }
            else
            {
               setupLogMessage(WARN, "Setup", "Skipping %s ...", p->get(PKG_NAME));
               printf("Warning: skipping %s ...\n", p->get(PKG_NAME));
                fflush(stdout);
            }
            err = -1;
         }
         else
         {
            const char *archive = p->getInfo(_installInfo, PKG_ARCHIVE);
            if (archive && *archive)
            {
               if (_installMode == Interactive)
               {
                  Dialog::showString(NSString("Extracting ") + p->get(PKG_NAME) + "...\n");
               }
               else
               {
                  printf("Extracting %s ...\n", p->get(PKG_NAME));
                   fflush(stdout);
               }
               errMsg = p->install(_serverRoot, _sourceDir, _installInfo);
               if (errMsg != (char *) NULL)
               {
                  err = -1;
                  if (_installMode == Interactive)
                  {
                     Dialog::showString(NSString("ERROR: ") + errMsg);
                  }
                  else
                  {
                     printf("ERROR: %s\n", errMsg.data());
                      fflush(stdout);
                     setupLogMessage(FATAL, "Setup",
                     "Error extracting %s (%s)", p->get(PKG_NAME), errMsg.data());
                  }
                    terminate(1);
               }
            }
         }
      }
      if (err)
      {
         p->skip(True);
      }
   }
   return err;
}


int
Installer::postInstall()
{
   int err = 0;

   if (_installMode == BinaryOnly) return 0;

   _installInfo->setFormat(1);

   _installInfo->toLocal(CONFIG_LDAP_URL);
   _installInfo->toLocal(DS_ADMIN_DOMAIN);
   _installInfo->toLocal(MC_ADMIN_ID);
   _installInfo->toLocal(UG_ADMIN_ID);
   _installInfo->toLocal(UG_LDAP_URL);

   _installInfo->write(_infoFile);
   for (int i = 0; i < _newSuiteSpot.numComponents(); i++)
   {
      Product *p = _newSuiteSpot.getComponent(i);
      if (p->isSelected() && !p->skipped())
      {

         err = doPreOrPostInstall(*p, PKG_POSTINSTALL);
         if (err)
         {
            p->skip(True);
         }
         else if (_installMode != Interactive)
         {
            setupLogMessage(SUCCESS, "Setup", "Installing %s.", p->get(PKG_NAME));
         }
      }
   }

   /*
    * Re-read cache
    */

   delete _installInfo;
   _installInfo = new InstallInfo (_infoFile);
   _installInfo->toUTF8(CONFIG_LDAP_URL);
   _installInfo->toUTF8(DS_ADMIN_DOMAIN);
   _installInfo->toUTF8(MC_ADMIN_ID);
   _installInfo->toUTF8(UG_ADMIN_ID);
   _installInfo->toUTF8(UG_LDAP_URL);

   return err;
}

void
Installer::updateModuleInfo(const Product &product, NVPair *moduleInfo)
{
   Product *p;
   Components *comp;
   NVPair *module = NULL;
   NSString moduleDir;
   NSString moduleInf = NULL;

   if (product.isSelected() == False || product.skipped())
   {
      return;
   }

   comp = product.getComponents();

   for (int i = 0; i < comp->numComponents(); i++)
   {
      p = comp->getComponent(i);

      if (p->isSelected() && p->skipped() == False)
      {
         if (p->packageType() == Patch)
         {
            updatePatchInfo (p, moduleInfo);
         }
         else
         {
            moduleInf = _serverRoot + "/" + SETUP_DIR + "/" + p->get("InfFile");

        /*
             * ModuleInfo parameter is NULL if updateModuleInfo is
             * called from updateSuiteSpotInfo. This is to 
             * differentiate between the first level packages and
             * the real package components. If moduleInfo is NULL
             * module.inf takes nickname.inf. Sub-components of
             * this package is to use the same inf file.
             */
            if (moduleInfo == NULL)
            {
               module = new NVPair(moduleInf);
               module->addSection(NULL, p->package());
            }
            else
            {
               module = moduleInfo;
               module->addSection(p->get(PKG_NICKNAME), p->package());
               _installInfo->add(product.get(PKG_NICKNAME), PKG_COMPONENTS, p->get(PKG_NICKNAME));
            }

            updateModuleInfo(*p, module);

            updateRegistrationInfo(p);

            if (moduleInfo == NULL)
            {
               module->setFormat(1);
               module->write(moduleInf);
               delete module;
            }
         }
     }
   }
}

void
Installer::updatePatchInfo(Product *patch, NVPair *moduleInfo)
{
   NVPair *applyTo = patch->dependencies(PKG_APPLYTO);
   const char *applyToName = applyTo->name(0);
   Components *installedComponents = getInstalledComponents();
   Product *applyToComponent = installedComponents->getComponent(applyToName);
   NSString moduleInf;
   NSString patchInf;
   NVPair *patchInfo;

   if (applyToComponent == 0)
   {
      // Something wrong, ignore for now PVO
   }
   else
   {
      char *newList;
      const char *s;
      NSString id;
      id = patch->get(PKG_NICKNAME);
      if (patch->get(PKG_NICKNAME))
      {
         id = id + "/" + patch->get(SETUP_INF_PATCHID);
      }

      moduleInfo->getSection(applyToName)->add(PKG_PATCHES, id);

      // update version info if it's defined in patch
      if (s = patch->get("Version"))
         moduleInfo->getSection(applyToName)->set(PKG_VERSION, s);

      patchInf = _serverRoot + "/" + SETUP_DIR + "/" + 
         moduleInfo->get("SetupDir") + "/" + moduleInfo->get("nickname") + "-patch.inf";
      patchInfo = new NVPair(patchInf);

      newList = setupAddNameToList(patchInfo->get(PKG_COMPONENTS), patch->get(PKG_NICKNAME));
      patchInfo->set(PKG_COMPONENTS, newList);
      free (newList);

      patchInfo->addSection(patch->get(PKG_NICKNAME), patch->package());
      patchInfo->setFormat(1);
      patchInfo->write(patchInf);
   }
}

void
Installer::updateRegistrationInfo (const Product *p)
{
   const char *s;
   LdapError err;
   const char *ldapURL = _installInfo->get(CONFIG_LDAP_URL);
   const char *ldapUser = _installInfo->get(MC_ADMIN_ID);
   const char *ldapPwd = _installInfo->get(MC_ADMIN_PWD);
   Bool regNYR = False;
   Bool regProduct = False;

   s = p->get(PKG_REGISTRATION);
   if (s && !strcasecmp(s, "True"))
      regNYR = True;

   s = p->get(PKG_USELDAP);
   if (s && !strcasecmp(s, "True"))
      regProduct = True;

   if (regProduct == False && regNYR == False)
      return;

   if (ldapURL)
   {
      Ldap ldap(err, ldapURL, ldapUser, ldapPwd, _bindDN, _bindPwd);
      if (err == OKAY)
      {
         LdapEntry ldapEntry(&ldap);
         if (regNYR == True && !p->installed())
         {
            err = ldapEntry.retrieve(DEFAULT_COMMON_PREFS_FILTER, LDAP_SCOPE_SUBTREE, ldap.baseSuffix());

            if (err == OKAY)
            {
               ldapEntry.addAttribute("nsnyr", p->get(PKG_NICKNAME));
               ldapEntry.update(ldapEntry.entryDN());
            }
         }

         if (regProduct == True)
         {
            /* Update the ISIE */
            ldapEntry.clear();
            ldapEntry.setAttribute("nsVendor", p->get(PKG_VENDOR));
            ldapEntry.setAttribute("nsProductVersion", p->get(PKG_VERSION));
            ldapEntry.setAttribute("nsRevisionNumber", p->get(PKG_REVISION));
            ldapEntry.setAttribute("nsBuildNumber", p->get(PKG_BUILDNUMBER));
            ldapEntry.setAttribute("nsSerialNumber", p->get(PKG_SERIALNUMBER));
            ldapEntry.setAttribute("nsExpirationDate", p->get(PKG_EXPIRY));
            ldapEntry.setAttribute("nsBuildSecurity", p->get(PKG_SECURITY));
            ldapEntry.setAttribute("nsLdapSchemaVersion", p->get(PKG_SCHEMAVERSION));

            updateISIEVersionInfo(&ldap, &ldapEntry, p->get(PKG_NAME), _installInfo->get(MACHINE_NAME), _installInfo->get(SERVER_ROOT), _installInfo->get(DS_ADMIN_DOMAIN));
         }
      }
   }            
}

void
Installer::updateSuiteSpotInfo()
{
   NSString uninstInfoFile;
   Product *product;
   PackageInfo *suitespot;
   NVPair *pp;
   int i;
   char **components = NULL;

   suitespot  = _newSuiteSpot.package();
   uninstInfoFile = _serverRoot + "/" + SETUP_DIR + "/" + SUITESPOT_INF;
   Components *comp = _newSuiteSpot.getComponents();

   /*
    * Retrieve existing uninstall info file
    * It's OK if there is none
    */
   NVPair uninstInfo(uninstInfoFile);

   components = uninstInfo.getListItems(PKG_COMPONENTS);

   // Replace the [general] section, in case of an upgrade
   uninstInfo.addSection(NULL, suitespot);

   uninstInfo.delList(PKG_COMPONENTS);

   /*
    * Restore the list of existing components
    */
   uninstInfo.add(PKG_COMPONENTS, (const char **) components);

   for (i = 0; i < comp->numComponents(); i++)
   {
      product = comp->getComponent(i);
      if (product->isSelected() && product->skipped() == False &&
          product->packageType() != Patch &&
          product->packageType() != Update)
      {
         pp = suitespot->getSection(product->get(PKG_NICKNAME));
         uninstInfo.addSection(product->get(PKG_NICKNAME), pp);
         uninstInfo.add(PKG_COMPONENTS, product->get(PKG_NICKNAME));
         _installInfo->add(SERVER_LIST, product->get(PKG_NICKNAME));
      }
   }

   uninstInfo.setFormat(1);
   uninstInfo.write(uninstInfoFile);

   updateModuleInfo(_newSuiteSpot, NULL);

   if (components)
   {
      NVPair::freeListItems(components);
   }
}


NSString 
Installer::checkServerRoot(const char *sroot)
{
   int err = 0;
   char errMsg[SML_BUF];

   rmdir (sroot);

   errMsg[0] = 0;

   if (InstUtil::dirExists(sroot) == True)
   {
      if (InstUtil::isServerRoot(sroot) == False && InstUtil::dirEmpty(sroot) == False)
      {
         err = -4;
      }

      if (InstUtil::dirWritable(sroot) == False)
      {
         err = -3;
      }
   }
   else
   {
      err = InstUtil::createDirectory(sroot, 0755);
   }

   if (!err)
   {
      getInstalledProducts(sroot);
      // skip invalid components. Invalid components are those not compatible
      // with same components existing on server root.
      skipInvalidComponents();

   }
         
   if (err)
   {

      if (err == -1)
      {
         snprintf(errMsg, sizeof(errMsg), "Can't create directory %s\n", sroot);
      }
      else if (err == -2)
      {
         snprintf(errMsg, sizeof(errMsg), "%s exists and is not a directory.\n", sroot);
      }
      else if (err == -3)
      {
         snprintf(errMsg, sizeof(errMsg), "Can't write to %s\n", sroot);
      }
      else if (err == -4)
      {
         snprintf(errMsg, sizeof(errMsg),
            getResource(RES_MESSAGE_CHECKSERVERROOT1),
            sroot);
      }
      else
      {
         snprintf(errMsg, sizeof(errMsg),
            getResource(RES_MESSAGE_CHECKSERVERROOT2),
            sroot);
      }
      errMsg[sizeof(errMsg)-1] = 0;
   }
   return errMsg;
}

NSString
Installer::checkInstallable(Components *components)
{
   char errMsg[SML_BUF];
   Product *p;
   int i;

   errMsg[0] = 0;

   for (i = 0; i < components->numComponents(); i++)
   {
      p = components->getComponent(i);
      if (p->isSelected() && !p->skipped())
      {
         if (p->compatible() && p->securityChecked())
         {
            continue;
         }
         else if (!p->securityChecked())
         { 
            snprintf(errMsg, sizeof(errMsg),
"You are installing %s over an\n"
"existing installation that is of a higher security level.\n"
"Installation over an incompatible security level is not\n"
"supported.\n", p->get(PKG_NAME));
            break;
         }
         else
         {
            snprintf(errMsg, sizeof(errMsg),
"You are installing %s over an\n" 
"existing installation that is incompatible.\n"
"Installation over an incompatible version is not\n"
"allowed.\n", p->get(PKG_NAME));
         }
      }
   }
   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

#if 0
NSString
Installer::checkComponentsDependency(Components *components)
{
   char errMsg[SML_BUF];
   Product *p;
   int i;
   int err;

   errMsg[0] = 0;

   for (i = 0 ; i < components->numComponents(); i ++)
   {
       p = components->getComponent(i);

       if (p->isSelected() && !p->skipped())
       {
          NSString s;
          PackageType packageType;

          packageType = p->packageType();

          if (packageType == Update || packageType == Patch)
          {
            Bool ok;
            if (packageType == Update)
            {
               if ((ok = checkUpdate(p)) == False)
               {
                  snprintf(errMsg, sizeof(errMsg),
                     getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY1),
                     p->get(PKG_NAME));
               }
            }
            else
            {
               const char *msg;
               ok = True;
               if (err = checkPatch(p))
               {
                  ok = False;
                  switch (err)
                  {
                  case 1:
                     msg = getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY3);
                     break;
                  case 2:
                     msg = getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY5);
                     break;
                  case 3:
                     msg = getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY6);
                     break;
                  };
                  snprintf(errMsg, sizeof(errMsg), msg, p->get(PKG_NAME));
               }

            }
          }
          else if (s = components->checkInstallDependency(p, getInstalledComponents()))
          {
             // This product depends on another product alias
             Product *q = components->getComponent(s.data());

/* PVO
             // Check whether s is already installed  or at least
             // a compatible version already exists
             if (q->installed() || (q->exists() && q->compatible()))
             {
                continue;
             }
             else
*/
             {
                 // Not compatible
                 snprintf(errMsg, sizeof(errMsg),
                    getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY2),
                    p->get(PKG_NAME),
                    q ? q->get(PKG_NAME) : s.data(),
                    q ? q->get(PKG_NAME) : s.data());
            }
         }

         if (errMsg[0])
         {
            break;
         }
      }
   }

   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

#else

NSString
Installer::checkComponentsDependency(Components *components)
{
   char errMsg[BIG_BUF];
   Product *p;
   int i;
   int err = 0;

   errMsg[0] = '\0';
   for (i = 0 ; i < components->numComponents(); i ++)
   {
      NSString msg;
      
      p = components->getComponent(i);
      if (p->isSelected() && !p->skipped())
      {
         char nickname[SML_BUF];
         err = components->checkProductDependencies(p, nickname);
         if (err)
         {
            if (err == -1)
            {
               snprintf(errMsg, sizeof(errMsg), "Error: Can't find component %s in the package\n", (const char *) nickname);
            }
            else
            {
               int loc1, loc2;
               loc1 = getUILocation(p->get(PKG_NICKNAME));
               loc2 = getUILocation(nickname);
               if (loc2 < loc1)
               {
                  snprintf(errMsg, sizeof(errMsg),
                    getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY2),
                    p->get(PKG_NAME),
                    nickname,
                    nickname);
               }
               else
               {
                  snprintf(errMsg, sizeof(errMsg),
                    getResource(RES_MESSAGE_CHECKCOMPONENTDEPENDENCY2),
                    p->get(PKG_NAME),
                    nickname,
                    nickname);
               }
            }
         }
      }
   }

   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

NSString
Installer::checkComponentsDependency(Components *components, int checkAll)
{
   char errMsg[BIG_BUF];
   Product *p;
   int i;
   int err = 0;

    errMsg[0] = '\0';
   for (i = 0 ; i < components->numComponents(); i ++)
   {
      NSString msg;
      
      p = components->getComponent(i);
      if (p->isSelected() && !p->skipped())
      {
         char nickname[SML_BUF];
         err = components->checkProductDependencies(p, nickname);
         if (err)
         {
            if (err == -1)
            {
               snprintf(errMsg, sizeof(errMsg), "Can't find component: %s\n", (const char *) nickname);
            }
            else
            {
               int loc1, loc2;
               loc1 = getUILocation(p->get(PKG_NICKNAME));
               loc2 = getUILocation(nickname);
               if (loc2 < loc1)
                  snprintf(errMsg, sizeof(errMsg),
                           "%s's prior component %s is not selected\n", p->get(PKG_NICKNAME), nickname);
               else if (checkAll)
                  snprintf(errMsg, sizeof(errMsg),
                           "%s's later component %s is not selected\n", p->get(PKG_NICKNAME), nickname);
            }
         }
      }
   }

   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

#endif


Bool
Installer::checkUpdate(Product *p)
{
   if (p->installed())
   {
      return True; 
   }
   else
   {
      return False;
   }
}

int 
Installer::checkPatch(Product *p)
{
   NVPair *applyTo = p->dependencies(PKG_APPLYTO);

   const char *s = applyTo->name(0); /* Get name of component */
   const char *r = applyTo->get(s);  /* Get version of component */

   /* We'll need to get revision also PVO */
   Components *oldComponents = getInstalledComponents();

   Product *applyToComponent = oldComponents->getComponent(s);
   
   if (applyToComponent == NULL)
   {
      return 1;
   }
   else 
   {
      if (strcasecmp(r, applyToComponent->get("Version")))
         return 2;
         
      if (p->checkSecurity(applyToComponent))
         return 3;

      return 0;
   }
}

         
NSString
Installer::verifySSUser(const char *ssUser)
{
   int err;
   char errMsg[SML_BUF];

   err = InstUtil::availUser(ssUser);

   switch (err)
   {
      case -1:

         snprintf(errMsg, sizeof(errMsg),
            getResource(RES_MESSAGE_VERIFYSSUSER1),
            ssUser);
         break;
      case -2:
      {
         NSString err_text(getResource(RES_MESSAGE_VERIFYSSUSER2, RES_GLOBAL_BRAND));
         err_text = err_text + getResource(RES_MESSAGE_VERIFYSSUSER3);
         snprintf(errMsg, sizeof(errMsg), err_text, ssUser);
         break;
      }
      default:
         errMsg[0] = 0;
         break;
   }

   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

NSString
Installer::verifySSGroup(const char *ssUser, const char *ssGroup)
{
   int err;
   char errMsg[SML_BUF];

   err = InstUtil::availGroup(ssUser, ssGroup);

   switch (err)
   {
      case -1:
        snprintf(errMsg, sizeof(errMsg),
         getResource(RES_MESSAGE_VERIFYSSGROUP1),
         ssUser);
        break;

      case -2:
        snprintf(errMsg, sizeof(errMsg),
         getResource(RES_MESSAGE_VERIFYSSGROUP2),
         ssGroup);
        break;

      case -3:
        snprintf(errMsg, sizeof(errMsg),
         getResource(RES_MESSAGE_VERIFYSSGROUP3),
         ssUser,ssGroup);
        break;
      case -4:
      {
        NSString err_text(getResource(RES_MESSAGE_VERIFYSSGROUP4, RES_GLOBAL_BRAND));
        err_text = err_text + getResource(RES_MESSAGE_VERIFYSSGROUP5);

        snprintf(errMsg, sizeof(errMsg), err_text, ssUser,ssGroup);
        break;
      }
      default:
        errMsg[0] = 0;
        break;
   }

   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

Bool
Installer::hasClients()
{
   Components *q = getInstallableComponents();

   for (int i = 0; i < q->numComponents(); i++)
   {
      Product *p = q->getComponent(i);
      const char *s = p->get(PKG_ISMCC);
      if (s && !strcasecmp(s, "True"))
      {
         return True;
      }
   }
   return False;
}

Bool
Installer::allClients()
{
   Components *q = getInstallableComponents();
   int count = 0;

   for (int i = 0; i < q->numComponents(); i++)
   {
      Product *p = q->getComponent(i);
      const char *s = p->get(PKG_ISMCC);
      if (s &&  !strcasecmp(s, "True"))
      {
         ++count;
      }
   }
   if (count == q->numComponents())
   {
      return True;
   }
   else
   {
      return False;
   }
}


void
Installer::selectAllClients()
{
   Components *q = getInstallableComponents();
   Product *p;
   int i;
   const char *s;

   for (i = 0; i < _newSuiteSpot.numComponents(); i++)
   {
      p = _newSuiteSpot.getComponent(i);
      if (p->numComponents())
      {
         p->select();
      }
   }
   for (i = 0; i < q->numComponents(); i++)
   {
      p = q->getComponent(i);
      s = p->get(PKG_ISMCC);
      if (s && !strcasecmp(p->get(PKG_ISMCC), "True"))
      {
          p->select();
      }
   }
} 
   
Bool
Installer::needLdapInfo()
{
   Bool needLdap = True;

   // Now determine whether we should skip the LDAP question
   Components *q = getInstallableComponents();
   Product *ds = q->getComponent("slapd");
   Components *r = newSuiteSpot()->getComponents();

   if (q->useLdap() == False)
   {
      needLdap = False;
   }
   else
   {
      needLdap = needLdapCheck(ds, r);
      _needLdap = needLdap;
   }

   return needLdap;
}

Bool
Installer::needUserDirAuth()
{
   Bool needUserDir = True;

   // Now determine whether we should skip the LDAP question
   Components *q = getInstallableComponents();
   Product *ds = q->getComponent("slapd");
   Components *r = newSuiteSpot()->getComponents();

   if (q->userDirAuth() == False)
   {
      needUserDir = False;
   }
   else
   {
      needUserDir = _needLdap; // Do not check twice
   }

   return needUserDir;
}

Bool
Installer::needLdapCheck(const Product *ds, const Components *r)
{

   Bool needLdap;

   // At least one server uses LDAP
   if(existingLdapCheck() == 0 )
   {
      // The server root is using some LDAP server
      // then this is a re-installation
      // If DS is not selected 
      //   ask for LDAP URL
      // else DS is selected
      //   If DS is the only thing being installed
      //      don't ask for LDAP info
      //   else if (DS is brand new)
      //         do not ask for LDAP info
      //   else this is a re-installation of DS
      //      PVO - do something here (ask for LDAP info but it 
      //      might be the same one which could be brought down 
      //      at this time.

      if (!ds || ds->isSelected() == False)
      {
         // If ds is not selected
         needLdap = True;
      }
      else
      {
         if (r->numSelectedComponents() == 1)
         {
            // ldap is the only thing installed
            needLdap = False;
         }
         else if (ds->exists() == False)
         {
            // This is a brand new installation of LDAP
            // the server root is using some other LDAP
            // then DS will ask the question
            needLdap = False;
         }
         else
         {
             // This is a re-installation of DS. 
            if (ds->get(PKG_USELDAP) && (strcasecmp(ds->get(PKG_USELDAP), "True") == 0))
               needLdap = True;
            else
               needLdap = False;
         }
      }
   }
   else 
   {
      // Brand new installation (ldapCheck failed)
      if (ds && ds->isSelected())
      {
         // LDAP question will be asked by DS installation
         needLdap = False;
      }
      else
      {
         needLdap = True;
      }
   }

   return needLdap;
}

NSString
Installer::verifyInstallPrivilege (const char *ldapURL, const char *ldapUser, const char *ldapPwd,
            const char *installDN)
{
   LdapError err;
   char errMsg[MED_BUF];

   errMsg[0] = 0;

   Ldap ldap(err, ldapURL, ldapUser, ldapPwd, _bindDN, _bindPwd);

   err = setupInstallFramework (&ldap, installDN, getResource(RES_GLOBAL_BRAND));

   if (err == INVALID_ROOT)
   {
      snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYINSTALLPRIVILEGE1, RES_GLOBAL_BRAND));
   }
   else if (err == INSTALL_DN_ERR)
   {
      if (installDN == NULL)
      {
         NSString err_text(getResource(RES_MESSAGE_VERIFYINSTALLPRIVILEGE2, RES_GLOBAL_BRAND));
         err_text = 
            err_text + getResource(RES_MESSAGE_VERIFYINSTALLPRIVILEGE22);
         snprintf(errMsg, sizeof(errMsg), err_text, 
            DEFAULT_SS_CONTAINER_RDN, DEFAULT_ROOT_DN);
      }
      else
      {
         NSString err_text(getResource(RES_MESSAGE_VERIFYINSTALLPRIVILEGE3, RES_GLOBAL_BRAND));
         err_text =
            err_text + getResource(RES_MESSAGE_VERIFYINSTALLPRIVILEGE32);
         snprintf(errMsg, sizeof(errMsg), err_text, installDN);
      }
   }
   else if (err != OKAY)
   {
      snprintf(errMsg, sizeof(errMsg),
              getResource(RES_MESSAGE_VERIFYINSTALLPRIVILEGE4), ldapUser);
   }
   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

NSString
Installer::getUserDirectory (const char *ldapURL, const char *ldapUser, const char *ldapPwd,
            const char *installDN, char **url, char **bindDN, char **bindPwd)
{
   LdapError err;
   char errMsg[MED_BUF];

   errMsg[0] = 0;

   // somehow _bindDN gets corrupted if I pass to ldap constructor 
   // Ldap ldap(err, ldapURL, ldapUser, ldapPwd, _bindDN, _bindPwd);
   Ldap ldap(err, ldapURL, ldapUser, ldapPwd, NULL, NULL);


   err = getLocalUserDirectory(&ldap, 
      installDN, 
      ldap.host(),
      _serverRoot,
      url, 
      bindDN, 
      bindPwd);

   // try to get GlobalUserDirectory if no LocalUserDirectory 
   if(err != OKAY) {
          err = getGlobalUserDirectory(&ldap, 
                  installDN, 
                  url, 
                  bindDN, 
                  bindPwd);
    }

   if (err != OKAY)
   {
      snprintf(errMsg, sizeof(errMsg), "Setup is unable to detect User Directory Informations from the Server\n");
      errMsg[sizeof(errMsg)-1] = 0;
   }

   return errMsg;
}


NSString
Installer::verifyInstallDomain (const char *ldapURL, const char *installDN)
{
   LdapError err;
   char errMsg[MED_BUF];

   errMsg[0] = 0;

   Ldap ldap(err, ldapURL, NULL, NULL, NULL, NULL);


   err = checkAdminDomain (&ldap, installDN);


   if (err != OKAY)
   {
      snprintf(errMsg, sizeof(errMsg),      
"The Server Configuration directory subtree appears not to be set up\n"
"correctly for the installation of Netscape Server products. Please contact\n"
"your directory administrator to verify that the you have write\n"
"access to %s\n", installDN);
      errMsg[sizeof(errMsg)-1] = 0;
   }

   return errMsg;
}


Bool 
Installer::getLocalLdapInfo (char ** ldapURL, char ** ldapUser, char **installDN)
{
   static char *url = NULL, *user = NULL, *dn = NULL;
   static Bool localStatus = True;

   if (localStatus == False) 
   {
      *ldapURL = *ldapUser = *installDN = NULL;
      return False;
   }

   if (url || user || dn)
   {
      if (ldapURL)
         *ldapURL = url;
      if (ldapUser)
         *ldapUser = user;
      if (installDN)
         *installDN = dn;
      return True;
   }

   getDefaultLdapInfo(_serverRoot, &url, &user, &dn);

   if (url == NULL && user == NULL && dn == NULL)
      return False;

   if (ldapURL != NULL && url != NULL)
      *ldapURL = url;

   if (ldapUser != NULL && user != NULL)
      *ldapUser = user;

   if (installDN != NULL && dn != NULL)
      *installDN = dn;   

    return True;
}

void
Installer::selectComponentDependency(Components *new_components, Components *old_components)
{
   DialogAlert *msg;
   Product & suitespot = _newSuiteSpot;
   int i, j;

   // select invisible mandatory components
   new_components->selectMandatoryComponents();

   // select invisible and checked=True component if parent is selected.
   // it needs to be done here becasue _newSuiteSpot knows parent info
   for (i = 0; i < suitespot.numComponents(); i++)
   {
      Product *p = suitespot.getComponent(i);

      if (p->isSelected() == True && p->numComponents())
      {
         for (j = 0; j < p->numComponents(); j++)
         {
            Product *childProduct;
            childProduct = p->getComponent(j);
            if (childProduct->isChecked() == True && childProduct->isVisible() == False &&
                childProduct->compatible())
            {
               childProduct->select();
            }
         }
      }
   }

   // select component dependencies
   new_components->selectInstallDependency();

   // select parent if it's child is selected
   for (i = 0; i < suitespot.numComponents(); i++)
   {
      Product *p = suitespot.getComponent(i);

      if (p->isSelected() == False && p->numComponents())
      {
         for (j = 0; j < p->numComponents(); j++)
         {
            Product *childProduct;
            childProduct = p->getComponent(j);
            if (childProduct->isSelected())
            {
               p->select();
               break;
            }
         }
      }
   }

#if 1
   /*
    * Write out selected component and installed component list
    * for some product to decide what questions to ask.
    *
    */
   if (_installMode == Interactive)
   {
      NSString list;
      
      for (i = 0; i < new_components->numComponents(); i++)
      {
         Product *p = new_components->getComponent(i);
      
         if (p->isSelected() && !p->skipped())
         {
            if (list != (char *) NULL)
               list = list + "," + p->get("NickName");
            else
               list = p->get("NickName");
         }
      }
      if (list != (char *) NULL)
      {
         _installInfo->set("SelectedComponents", (const char *) list);
         list = NULL;
      }
      
      for (i = 0; i < old_components->numComponents(); i++)
      {
         Product *p = old_components->getComponent(i);
      
         if (list != (char *) NULL)
            list = list + "," + p->get("NickName");
         else
            list = p->get("NickName");
      }
      
      if (list != (char *) NULL)
      {
         _installInfo->set("InstalledComponents", (const char *) list);
      }
   }

#else
   // debug mode: show selected components list
   if (_installMode == Interactive && _newSuiteSpot.get("_debug"))
   {
      char names[2000];

      strcpy(names, "The following components will be installed\n\n");

      for (i = 0; i < new_components->numComponents(); i++)
      {
         Product *p = new_components->getComponent(i);

         if (p->isSelected() && !p->skipped())
         {
            strcat(names, "\n");
            strcat(names, p->get("name"));
            strcat(names, " [");
            strcat(names, p->get("NickName"));
            strcat(names, " ");
            strcat(names, p->get("Version"));
            strcat(names, "]");

            
            for (j = 0; j < old_components->numComponents(); j++)
            {
               Product *q = old_components->getComponent(j);

               if (q && strcasecmp(q->get("NickName"), p->get("NickName")) == 0)
               {
                  strcat(names, "[overwrite existing ");
                  strcat(names, q->get("version"));
                  strcat(names, "]");
               }
            }

         }
      }

      if (i != 0)
      {
         strcat(names, "\n");
         strcat(names, "\n");

         msg = new DialogAlert(names);
         msg->execute();
      }
   }
#endif
}

/*
 Currently we are using two level selection UI
 each level takes max 2**4 elements.  
 This function returns two dimension number of UI location. 
 The return value is internal number before page displaying. It tells
 relative location between components.  As we support hidden components, 
 user might not see it on screen at all.
 */
int Installer::getUILocation(const char *nickname)
{
   int n;
   int i, j;

   for (i = 0; i < _newSuiteSpot.numComponents(); i++)
   {
      Product *p = _newSuiteSpot.getComponent(i);

      n = i << 4;

      if (p->numComponents())
      {
         for (j = 0; j < p->numComponents(); j++)
         {
            Product *childProduct;
            childProduct = p->getComponent(j);
            if (strcasecmp(childProduct->get(PKG_NICKNAME), nickname) == 0)
            {
               return n + j;     
            }
         }
      }
   }
   
   return -1;
}


void Installer::startLogging()
{
   char buf[SML_BUF];

#if 0
   _logFile = _tmpDir + "/" +  INSTALL_LOG;
   _installLog = new InstallLog (_logFile);
   if (_installLog->isValid() == False)
   {
      delete _installLog;
      _logFile = tmpnam(buf);
      _installLog = new InstallLog(_logFile);
   }
#endif
   _logFile = tmpnam(buf);
   setupSetLogFile(_logFile);
   setupLogMessage(INFO, "Setup", "Start...");
}

void Installer::stopLogging()
{
   NSString newLogFile = _serverRoot + "/" + SETUP_DIR + "/" + INSTALL_LOG;

   setupLogMessage(INFO, "Setup", "DONE");

   if (InstUtil::copyFile(newLogFile, _logFile) == 0)
   {
      unlink(_logFile.data());
      _logFile = newLogFile;
   }

}

void Installer::skipInvalidComponents()
{
   Product *pNew;
   Product *pOld;
   int i;

   for (i = 0; i < _newComponents->numComponents(); i++)
   {
      pNew = _newComponents->getComponent(i);

      NSString s;
      PackageType packageType;
      packageType = pNew->packageType();

      if (packageType == Update || packageType == Patch)
      {
         if (packageType == Update && checkUpdate(pNew) == False)
         {
            pNew->skip(True);            
         }
         else if (packageType == Patch && checkPatch(pNew) != 0)
         {
            pNew->skip(True);
        }
      }
      else
      {
         pOld = _oldComponents->getComponent(pNew->get(PKG_NICKNAME));

         if (pOld && ((pNew->checkCompatibility(pOld, _checkBuildNum) < 0) || 
                      (pNew->checkSecurity(pOld) < 0)))
         {
            pNew->skip(True);
         }
      }
   }
}

Bool
Installer::needDomainName()
{
  return _checkDomainName;
}



void
Installer::printNotice()
{
   /* Bug 408242: Check if the LD_LIBRARY_PATH/SHLIB_PATH is set.
    * If so, display a warning. Also, unset ENV variable.
    */

    char *lib_name = NULL, *lib_path = NULL, tmp[XSM_BUF], *env_path = NULL;

#if defined (HPUX)
    lib_name = strdup("SHLIB_PATH");
#else
    lib_name = strdup("LD_LIBRARY_PATH");
#endif

    memset(tmp, '\0', XSM_BUF);
    if (lib_path = getenv(lib_name))
    {
        DialogManager::disableWinMode();
        printf("\n%s %s %s\n\n", "Notice: Your", lib_name, "variable is set to:");
        printf("\t%s\n\n", lib_path);
        printf("%s %s %s\n", "Please unset", lib_name, "or verify that the");
        printf("%s\n", "above setting is correct before continuing with");
        printf("%s\n", "the installation. An incorrect setting may cause");
        printf("%s\n", "problems during the installation such as the wrong");
        printf("%s\n\n", "copy of a shared library being used.");
        if (_installMode != Silent) 
        {
            printf("%s", "Continue [n]? ");
            fflush(stdout);
            fgets(tmp, sizeof(tmp), stdin);
            if (tmp[0] == 'n' || tmp[0] == 'N' || tmp[0] == '\0' || (tmp[0] < 0x20))
            {
                free(lib_name);
                exit(0);
            }
        }
    }

    free(lib_name);

    env_path = strdup("ENV=");
    putenv(env_path);
}
