/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME:
**   instutil.cc
**
** DESCRIPTION:
**   SuiteSpot Installation Program Utilities
**
** NOTES:
**   UNIX Only
** HISTORY:
** $Log: ux-util.cc,v $
** Revision 1.2  2006/03/23 15:59:22  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:33  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:30  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/03/01 01:45:18  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/09 01:33:38  nhosoi
** bugzilla 147447: De-brand the Setup SDK as "brandx"; attachment (110842)
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.7.4.1  2004/02/26 17:18:25  kaining
** Bug 624241 - modified guessHostName() to make a copy of hn before doing sprintf()
**
** Revision 1.1.1.7  2001/11/16 03:18:09  pkhincha
** Wrong fix. REverting back. Umask cannot be set at init. Will create all
** directories as 700.
**
** Revision 1.1.1.6  2001/11/16 00:18:59  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.5  1999/12/31 02:05:58  rweltman
** Do not define getdomainname on AIX 4.3
**
** Revision 1.1.1.4  1999/10/12 20:01:19  cxue
** change getdomainname to sysinfo(SI_SRPC_DOMAIN,...) on Solaris
**
** Revision 1.1.1.3  1999/06/08 23:23:08  cxue
** APIs to get current user and group name
**
** Revision 1.1.1.2  1999/03/19 22:40:53  svbld
** Solaris 2.6 compiler doesn't like sysent.h
**
** Revision 1.1.1.1  1999/03/04 23:27:12  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.38.2.4  1999/01/29 23:43:15  cxue
** treat lost+found only directory as empty directory
**
** Revision 1.1.2.38.2.3  1998/11/09 17:43:21  cxue
** move InstallLog to setupapi.[h|cpp]
**
** Revision 1.1.2.38.2.2  1998/10/28 17:52:28  cxue
** fileName
**
** Revision 1.1.2.38.2.1  1998/09/10 16:33:03  cxue
** Add dirEmptry for checking empty directory
**
** Revision 1.1.2.38  1998/09/03 19:52:17  pvo
** Fix availGroup problem as reported by Sunil
**
** Revision 1.1.2.37  1998/08/20 23:14:44  cxue
** remvoe formHostPortURL, calling stripedLdapURL in nsutils.cpp
**
** Revision 1.1.2.36  1998/08/19 17:45:25  cxue
** Add formHostPortURL to InstUtil
**
** Revision 1.1.2.35  1998/08/18 04:47:51  cxue
** Move getDefaultLdapInfo to lib/common.cpp
**
** Revision 1.1.2.34  1998/06/17 20:48:35  atulb
** Commented out the call to setsockopt for setting SO_REUSEADDR to true since it allowed multiple binds to same port on HPUX11 and Irix6.2/6.3
**
** Revision 1.1.2.33  1998/05/13 04:39:34  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.32  1998/03/31 22:18:18  pvo
** 112007 - Installer support for Admin's bind address.
**
** Revision 1.1.2.31  1998/03/26 03:20:38  pvo
** 111470 - Uninstall core dumps on IRIX.
**
** Revision 1.1.2.30  1998/03/24 22:29:53  pvo
** 110961 - Setup cannot handle long directory name.
**
** Revision 1.1.2.29  1998/03/20 04:27:27  pvo
** Removed osfcn.h so it would build on HP using aCC.
**
** Revision 1.1.2.28  1998/03/13 21:24:51  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.27  1998/03/10 02:49:32  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.26  1998/03/04 02:29:36  pvo
** Changes due to change in common time format.
**
** Revision 1.1.2.25  1998/02/24 00:22:47  pvo
** Support topology change
**
** Revision 1.1.2.24  1998/01/29 21:53:01  pvo
** Bug 104982 - in verifyLdap, return error if url does not have port.
** Bug 104968 - change default LDAP suffix to domain.com rather than 
** example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.23  1998/01/27 02:33:16  pvo
** Changed getAdmPwd to retrieve the pwd from somewhere else.
**
** Revision 1.1.2.22  1998/01/22 04:52:03  pvo
** Added getAdmPwd() and doUUEncode()
**
** Revision 1.1.2.21  1998/01/15 18:16:14  pvo
** Fixed compilation problem in AIX4.2
**
** Revision 1.1.2.20  1998/01/09 03:24:48  pvo
** Added copyFile() and logTime() functions.
**
** Revision 1.1.2.19  1997/12/12 21:54:35  pvo
** Various bug fixes.
**
** Revision 1.1.2.18  1997/12/03 19:03:43  pvo
** Check for uname >=0 instead of just > 0 since on some OS (like IRIX), 0
** means OK!
**
** Revision 1.1.2.17  1997/12/02 04:28:33  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.16  1997/11/26 04:43:19  pvo
** Fixed up for cross-platform compilation.
**
** Revision 1.1.2.15  1997/11/26 01:14:30  pvo
** Added getSysVersion and getSelfUserID
**
** Revision 1.1.2.14  1997/11/15 19:57:48  pvo
** In guessDomain(), fixed problem with AIX not being able to guess the FQDN.
**
** Revision 1.1.2.13  1997/11/14 09:11:37  pvo
** Various changes to strengthen code.
**
** Revision 1.1.2.12  1997/11/13 19:28:55  pvo
** More new stuffs.
**
** Revision 1.1.2.11  1997/11/11 05:41:25  pvo
** Added dependency checking for uninstallation.
**
** Revision 1.1.2.10  1997/11/09 19:26:56  pvo
** Various changes due to nsdefs and ux-util API changes.
**
** Revision 1.1.2.9  1997/11/08 16:58:17  pvo
** isServerRoot now only detects a 4.0 server root
**
** Revision 1.1.2.8  1997/10/29 01:43:09  pvo
** Modular Installation support.
**
** Revision 1.1.2.7  1997/10/24 03:39:44  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.6  1997/10/23 03:23:44  pvo
** More change to support other platforms.
**
** Revision 1.1.2.5  1997/10/22 23:59:03  pvo
** Warning removal project!
**
** Revision 1.1.2.4  1997/10/10 22:51:09  pvo
** Writing ACI to netscapeRoot.
**
** Revision 1.1.2.3  1997/10/06 21:06:49  pvo
** Additional code. Pirated routine time_stamp from util_ctime.
**
** Revision 1.1.2.2  1997/10/06 20:38:16  pvo
** Restructured dependencies.
**
** Revision 1.1.2.1  1997/09/27 02:47:32  pvo
** Checked in.
**
*********************************************************************/

/*********************************************************************
**
** Includes
**********************************************************************
*/

extern "C" {
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <time.h>               /* time */
#include <pwd.h>                /* getpwnam */
#include <ctype.h>
#include <grp.h>                /* getgrgid */
#include <fcntl.h>              /* creat */
#include <sys/stat.h>           /* stat */
#include <sys/wait.h>           /* wait */
#include <sys/utsname.h>	/* utsname */

#include <sys/types.h>
#include <sys/socket.h>         /* socket, bind */
#if defined(SOLARIS)
  #include <sys/systeminfo.h>
#endif

#include <netinet/in.h>         /* htonl */
#include <netdb.h>              /* gethostbyname */
#include <arpa/inet.h>          /* inet_ntoa */
#include <dirent.h>
#include <assert.h>
}

const char *PATH_RESCONF = "/etc/resolv.conf";

/*
extern "C" {
#include "ldaputil/dbconf.h"
#include "ldaputil/ldapdb.h"
#include "ldaputil/ldaputil.h"
}
*/

#include "ux-util.h"
#include "ldapu.h"

const char *INSTALL_TYPE = "InstallType";
const char *EXPRESS_TYPE = "1";
const char *TYPICAL_TYPE = "2";
const char *CUSTOM_TYPE  = "3";

const char *DEFAULT_LDAP_SUFFIX = "o=Example.com";

#if (defined(OSF1) || defined(AIX) || defined(HPUX) || defined(SOLARIS))
extern "C" {
#if (defined(OSF1))
int  gethostname (char *, int);
#endif
#if !(defined(AIX4_3))
void getdomainname(char *, int);
#endif
}
#endif

/*********************************************************************
**
** FUNCTION:    getDefaultHostname
** DESCRIPTION:
**   Get the default hostname as previously configured.
**
** INPUTS:      hn  -  input buffer
** OUTPUTS:     hn  -  host name, if found
** RETURN:      0 (good), -1 (failure)
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**      hn is expected to point to a valid memory location.
**********************************************************************
*/

NSString 
InstUtil::getDefaultHostName(const char *sroot)
{
   char tstr[BIG_BUF];
   NSString hn;
   NVPair *admconf;

   snprintf(tstr, sizeof(tstr), "%s/%s", sroot, DEFAULT_ADMINCONF);
   tstr[sizeof(tstr)-1] = 0;

   admconf = new NVPair(tstr); 

   if (admconf->isEmpty() == False)
   {
      hn = admconf->get("ServerName");
   }

   delete admconf;
   return hn;
}

/*********************************************************************
**
** FUNCTION:    getHostIpAddress
** DESCRIPTION:
**   Get the IP address associated with a given host name
**
** INPUTS:      const char *hostName. If NULL, default to current host.
** OUTPUTS:     None
** RETURN:      NULL, or IP address
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**      
**********************************************************************
*/
NSString
InstUtil::getHostIpAddress(const char *host)
{
   struct hostent *hn;
   NSString hostName;

   if (host == NULL)
   {
      hostName = guessHostname();
      host = hostName.data();
   }

   hn = gethostbyname(host);

   if (!hn)
   {
      return NULL;
   }
   else
   {
      return NSString(inet_ntoa(*((struct in_addr *) hn->h_addr)));
   }
}

/*********************************************************************
**
** FUNCTION:    getDefaultUser
** DESCRIPTION:
**   Tries to get the SuiteSpot User ID previously configured
**   for the existing installation.
**
** INPUTS:      sroot   - server root
** OUTPUTS:
** RETURN:      ssUser
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**      ssUser and ssGroup are expected to point to valid memory buf.
**********************************************************************
*/

NSString
InstUtil::getDefaultUser(const char *sroot)
{
   char tstr[BIG_BUF];
   NVPair *ssconf;
   NSString ssUser;

   snprintf(tstr, sizeof(tstr), "%s/%s", sroot, DEFAULT_SSUSERCONF);
   tstr[sizeof(tstr)-1] = 0;

   ssconf = new NVPair(tstr);

   if (ssconf->isEmpty() == False)
   {
      ssUser = ssconf->get("SuiteSpotUser");
   }

   delete ssconf;

   return ssUser;

}
/*********************************************************************
**
** FUNCTION:    getDefaultGroup
** DESCRIPTION:
**   Tries to get the SuiteSpot User ID previously configured
**   for the existing installation.
**
** INPUTS:      sroot   - server root
** OUTPUTS:
** RETURN:      default Group
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**      
**********************************************************************
*/

NSString
InstUtil::getDefaultGroup(const char *sroot)
{
   char tstr[BIG_BUF];
   NVPair *ssconf;
   NSString ssGroup;

   snprintf(tstr, sizeof(tstr), "%s/%s", sroot, DEFAULT_SSUSERCONF);
   tstr[sizeof(tstr)-1] = 0;

   ssconf = new NVPair(tstr);

   if (ssconf->isEmpty() == False)
   {
      ssGroup = ssconf->get("SuiteSpotGroup");
   }

   delete ssconf;

   return ssGroup;
}

/*********************************************************************
**
** FUNCTION:    getAdmPwd
** DESCRIPTION:
**   
**  
**
** INPUTS:   
** OUTPUTS:
** RETURN:      
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**
**********************************************************************
*/
NSString
InstUtil::getAdmPwd(const char *serverRoot)
{
   char temp[BIG_BUF];
   NVPair admpw;
   const char *pwd = NULL;

   snprintf(temp, sizeof(temp), "%s/admin-serv/config/adm.conf", serverRoot);
   temp[sizeof(temp)-1] = 0;
   admpw.setFormat(2);

   admpw.read(temp);
   pwd = admpw.get("siepid");

   return pwd;
}




/*********************************************************************
**
** FUNCTION:    getSysVersion
** DESCRIPTION:
**   Get the system version in the form of <sysname><version>
**   per the output of uname -svr as appropriate.
**
** INPUTS:      
** OUTPUTS:
** RETURN:      NSString containing the verison
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**
**********************************************************************
*/
NSString
InstUtil::getSysVersion()
{
   struct utsname sysname;
   char tstr[XSM_BUF];

   if (uname(&sysname) >= 0)
   {
#if defined(IRIX) /* I want IRIX, not IRIX64 */
      snprintf(tstr, sizeof(tstr), "%s%s", "IRIX", sysname.release);
#elif defined(SOLARIS) || defined(OSF1) || defined(HPUX)
      snprintf(tstr, sizeof(tstr), "%s%s", sysname.sysname, sysname.release);
#elif defined(AIX)
      snprintf(tstr, sizeof(tstr), "%s%s.%s", sysname.sysname, sysname.version, sysname.release);
#endif
      tstr[sizeof(tstr)-1] = 0;
      return tstr;
   }
   else
   {
      return (char *) NULL;
   }
}

/*********************************************************************
**
** FUNCTION:    guessDomain()
** DESCRIPTION:
**   Try to guess the domain name of this machine.
**
** INPUTS:      none
** OUTPUTS:     none
** RETURN:      Guessed domain
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**      
**********************************************************************
*/

NSString
InstUtil::guessDomain(void)
{
   FILE *f = fopen(PATH_RESCONF, "r");
   char line[SML_BUF];
   char domain[SML_BUF];
   const char *dm;

   /* See if there's a domain entry in their resolver configuration */
   if (f)
   {
      while (fgets(line, sizeof(line), f))
      {
         if (!strncasecmp(line, "domain ", 7))
         {
            sscanf(&line[7], "%s", domain);
            domain[sizeof(domain)-1] = 0;
            dm = (domain[0] == '.' ? &domain[1] : domain);
            return dm;
         }
      }
      fclose(f);
   }

#ifndef NO_DOMAINNAME
   /* No domain found. Try getdomainname. */
#ifdef SOLARIS
  sysinfo(SI_SRPC_DOMAIN, domain, SML_BUF);
#else
   getdomainname(domain, sizeof(domain));
#endif
   dm = (domain[0] == '.' ? &domain[1] : domain);
#endif

   return dm;
}

/*********************************************************************
**
** FUNCTION:    guessHostname
** DESCRIPTION:
**   Guess the fully-qualified domain name of this machine.
**
** INPUTS:      none
** OUTPUTS:     none
** RETURN:      pointer to guessed hostname
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**      Returned memory to be treated as Readonly
**********************************************************************
*/

NSString
InstUtil::guessHostname(void)
{
   char hn[SML_BUF];
   NSString dn = InstUtil::guessDomain();

   gethostname(hn, sizeof(hn));

   /* Make FQDN */
   if (!strchr(hn, '.') && dn.data())
   {
      /* Bug 624241 - sprintf() will wipe out target var first on Linux */
      char hncopy[SML_BUF];
      strcpy(hncopy, hn);
      snprintf(hn, sizeof(hn), "%s.%s", hncopy, dn.data());
      hn[sizeof(hn)-1] = 0;
   }

   /* Return whatever we ended up with. */
   return hn;
}

Bool
InstUtil::portAvailable(int port)
{
   struct sockaddr_in sa_server;
   int sd, ret;
   int one = 1;

   if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      return False;
/* atulb: 6/17/98 - bug#135631 : commenting out this as it is unnecessary and
   allowed multiple binds to the same port on the localhost
   if ((setsockopt(sd, SOL_SOCKET, SO_REUSEADDR,
                   (const char *) &one, sizeof(one))) == -1)
      return False;
*/
   memset((char *) &sa_server, 0, sizeof(sa_server));
   sa_server.sin_family = AF_INET;
   sa_server.sin_addr.s_addr = htonl(INADDR_ANY);
   sa_server.sin_port = htons(port);
   ret = bind(sd, (struct sockaddr *) & sa_server, sizeof(sa_server));
   close(sd);
   return (ret == -1) ? False : True;
}

/*********************************************************************
**
** FUNCTION:    guessPort
** DESCRIPTION:
**   Guess a port
**
** INPUTS:      none
** OUTPUTS:     none
** RETURN:      pointer to guessed hostname
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**      Returned memory to be treated as Readonly
**********************************************************************/
int
InstUtil::guessPort()
{
   int port;

   srand(time(NULL));
   while (1)
   {
      port = MINPORT + rand() % (MAXPORT-MINPORT);

      if (portAvailable(port) == True)
        return port;
   }
}

/*********************************************************************
**
** FUNCTION:    availUser
** DESCRIPTION:
**   Giving a user id, determine whether the user is valid.
**   Moved here from ux-config.c
**
** INPUTS:      user
** OUTPUTS:     none
** RETURN:
**    0: User is okay
**   -1: User not found
**   -2: chown failed
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/

/* XXXrobm should not be copy-and-pasted from libadmin! */
int
InstUtil::availUser(const char  *user)
{
   struct passwd *pw;
   char fn[SML_BUF];
   int fd, ret;
   uid_t u;

   setpwent();
   if (isalpha((int) user[0]))
   {
      if (!(pw = getpwnam(user)))
      {
         return -1;
      }
   }
   else
   {
      if (!(pw = getpwuid(atoi(user))))
      {
         return -1;
      }
   }
   endpwent();

/* PVO
   (void) strcpy(user, pw->pw_name);
*/

   if ((u = geteuid()))
   {
      if (u != pw->pw_uid)
         return -2;
      else
         return 0;
   }

   snprintf(fn, sizeof(fn), "/tmp/trychown.%ld", (long) getpid());
   fn[sizeof(fn)-1] = 0;
   if ((fd = creat(fn, 0777)) == -1)
      return 0;                 /* Hmm. */
   ret = chown(fn, pw->pw_uid, pw->pw_gid);
   close(fd);
   unlink(fn);
   return (ret == -1 ? -2 : 0);
}

/*********************************************************************
**
** FUNCTION:    availGroup
** DESCRIPTION:
**   Giving a user id and a group name, determine whether the group
**   and the combination is valid.
**
** INPUTS:      user - the user id
**              group - the group name
** OUTPUTS:     none
** RETURN:
**    0 : group is okay
**   -1 : user not found
**   -2 : group not found
**   -3 : user not in group
**   -4 : chown on file with user/group failed
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/

int
InstUtil::availGroup(const char *user, const char *group)
{
   struct passwd *pw;
   struct group *grp;
   char *grp_member;
   char fn[SML_BUF];
   int i, fd, ret;
   uid_t u;

   setpwent();
   if (isalpha((int) user[0]))
   {
      if (!(pw = getpwnam(user)))
      {
         return -1;
      }
   }
   else
   {
      if (!(pw = getpwuid(atoi(user))))
      {
         return -1;
      }
   }
   endpwent();

   if (isalpha((int) group[0]))
   {
      grp = getgrnam(group);
   }
   else
   {
      grp = getgrgid(atoi(group));
   }
   if (grp == (struct group *) NULL)
   {
      return (-2);
   }

   if (pw->pw_gid != grp->gr_gid)
   {
      for (i = 0, grp_member = grp->gr_mem[0];
           grp_member != (char *) NULL;
           i++, grp_member = grp->gr_mem[i])
      {
         if (isalpha((int) grp_member[0]))
         {
            if (!strcmp(grp_member, user))
            {
               goto userINgroup;
            }
         }
         else
         {
            if (atoi(grp_member) == pw->pw_uid)
            {
               goto userINgroup;
            }
         }
      }
      return (-3);
   }

/* PVO
   (void) strcpy(group, grp->gr_name);
*/

userINgroup:
   if (u = geteuid())
   {                            /* if not root, then */
      if (u != pw->pw_uid)
      {
         return (-4);           /* admserv with no right to start user

                                 * progc */
      }
      else
      {
         return (0);
      }
   }

   snprintf(fn, sizeof(fn), "/tmp/trychown_grp.%ld", (long) getpid());
   fn[sizeof(fn)-1] = 0;
   if ((fd = creat(fn, 0777)) == -1)
   {
      return 0;                 /* Hmm. */
   }
   ret = chown(fn, pw->pw_uid, grp->gr_gid);
   close(fd);
   unlink(fn);
   return (ret == -1 ? -4 : 0);
}

/*********************************************************************
**
** FUNCTION:    isSelfRootUser
** DESCRIPTION:
**   check whether the effective user is root of the running process
**
** INPUTS:      None
** OUTPUTS:
** RETURN:
**   
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
Bool 
InstUtil::isSelfRootUser()
{
   if (getuid() == 0)
      return True;
   else
      return False;
}


/*********************************************************************
**
** FUNCTION:    getSelfUserID
** DESCRIPTION:
**   Get the effective user ID of the running process
**
** INPUTS:      None
** OUTPUTS:
** RETURN:
**   
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/

NSString 
InstUtil::getSelfUserID()
{
   return getpwuid(geteuid())->pw_name;
}

/*********************************************************************
**
** FUNCTION:    getSelfGroupID
** DESCRIPTION:
**   Get the effective group ID of the running process
**
** INPUTS:      None
** OUTPUTS:
** RETURN:
**   
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/

NSString 
InstUtil::getSelfGroupID()
{
   struct group *gr;

   gr = getgrgid(getgid());

   return gr->gr_name;
}

/*********************************************************************
**
** FUNCTION:    getCurrentDirectory
** DESCRIPTION:
**   
**  
** INPUTS:      None
** OUTPUTS:     
** RETURN:
**   Current Directory
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/

NSString
InstUtil::getCurrentDir(void)
{
   char curdir[MED_BUF];

   return getcwd(curdir, sizeof(curdir));
}

/*********************************************************************
**
** FUNCTION:    dirWritable()
** DESCRIPTION:
**
** INPUTS:      None
** OUTPUTS:
** RETURN:
**   Current Directory
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
Bool
InstUtil::dirWritable(const char *dirName)
{
   if (access(dirName, W_OK) == -1)
   {
      return False;
   }
   else
   {
      return True;
   }
} 

Bool
InstUtil::fileExists(const char *fileName)
{
   struct stat fi;

   if (stat (fileName, &fi) == -1)
   {
      return False;
   }
   else
   {
      return True;
   }
}

Bool
InstUtil::dirExists(const char *dirName)
{
   struct stat fi;

   if (stat (dirName, &fi) == -1 || !S_ISDIR(fi.st_mode))
   {
      return False;
   }
   else 
   {
      return True;
   }
}

Bool
InstUtil::dirEmpty(const char *dirName)
{
   DIR *dirp;
   struct dirent *dp;
   Bool rc = True;

   dirp = opendir(dirName);
   while ((dp = readdir(dirp)) != NULL)
   {
      if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, "..") && strcmp(dp->d_name, "lost+found"))
      {
         rc = False;
         break;
      }
   } 
   closedir(dirp);
   return rc;
}

int
InstUtil::copyFile(const char *dest, const char *src)
{
  FILE          *sf;
  FILE          *df;
  char          buf[BUFSIZ];
  size_t        charsRead;

  if ( ( sf = fopen( src, "r" ) ) == NULL ) {
    return errno;
  }
  if ( ( df = fopen( dest, "w" ) ) == NULL ) {
    return errno;
  }
  while ( ( charsRead = fread( buf, 1, BUFSIZ, sf ) ) != 0 ) {
    if ( fwrite( buf, 1, charsRead, df ) != charsRead ) {
      return errno;
    }
  }
  fclose( sf );
  fclose( df );

  return 0;     /* everything's good */
}


NSString
InstUtil::dirName (const char *path)
{
   char *p;
   static char s[BIG_BUF];

   if (path == NULL || *path == 0 || !strcmp(path,".") || !strcmp(path,".."))
   {
      return (".");
   }
   else if (!strcmp(path,"/"))
   {
      return ("/");
   }

   strncpy(s, path, sizeof(s));
   s[sizeof(s)-1] = 0;

   /* Strip off trailing / */

   p = strrchr(s,'/');
   if (p != NULL && *(p+1) == 0)
   {
     *p = 0;
   }

   p = strrchr(s,'/');

   if (p == NULL)
   {
      return (".");
   }
   else
   {
      *p = 0;
      if (*s == 0)
      {
         return ("/");
      }
      else
      {
         return s;
      }
   }
}

NSString
InstUtil::fileName (const char *path)
{
   NSString s;
   const char *p;

   p = strrchr(path,'/');
   if (p != NULL)
   {
      s = p + 1;
   }
   else
   {
      s = path;
   }

   return s;
}



/*
 * Returns:
 *   0 : OK
 *  -1 : Can't create directory
 *  -2 : input exists and is not a directory
 *  -3 : Can't write to directory
 */

int 
InstUtil::createDirectory(const char *dir, int mode)
{
   struct stat fi;
   char *s;
   char *t;

   s = strdup(dir);
   t = s + 1;

   while (1)
   {
      t = strchr(t, '/');

      if (t)
         *t = '\0';
      if (stat(s, &fi) == -1)
      {
         if (mkdir(s, mode) == -1)
         {
            return -1;
         }
         chmod(dir, mode);
      }
      else if (!S_ISDIR(fi.st_mode))
      {
         return -2;
      }
      if (t)
         *t++ = '/';
      else
         break;
   }

   if (access(dir, W_OK) == -1)
   {
      return -3;
   }
   return 0;
}

Bool
InstUtil::isServerRoot(const char *dir)
{
   NSString ssFile = dir;

   ssFile = ssFile + "/.suitespot";
   if (InstUtil::fileExists(ssFile))
   {
      return True;
   }
   else
   {
      return False;
   }
}

Bool
InstUtil::isLegacyServerRoot(const char *dir)
{
   Bool ret = False;
   NSString sroot = dir;

   // dir should be something like
   //   /opt/fedora/

   if (InstUtil::dirExists(sroot + "/admin-serv") ||
       InstUtil::dirExists(sroot + "/admserv") ||
       InstUtil::dirExists(sroot + "/setup") ||
       InstUtil::dirExists(sroot + "/userdb"))
   {
      ret =  True;
   }

   return ret;
}

int
InstUtil::execProgram(const char *program, const char *where)
{
   NSString curDir = getCurrentDir();

   int ret;

   if (where)
   {
      chdir(where);
   }

   ret = system(program);

   if (where)
   {
      chdir(curDir.data());
   }
   
   return ret;
}


/* The magic set of 64 chars in the uuencoded data */
static unsigned char uuset[] = {
'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T',
'U','V','W','X','Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n',
'o','p','q','r','s','t','u','v','w','x','y','z','0','1','2','3','4','5','6','7',
'8','9','+','/' };
 
int 
InstUtil::doUUEncode(const unsigned char *src, unsigned char *dst, int srclen)
{
   int  i, r;
   unsigned char *p;
 
/* 
 * To uuencode, we snip 8 bits from 3 bytes and store them as
 * 6 bits in 4 bytes.   6*4 == 8*3 (get it?) and 6 bits per byte
 * yields nice clean bytes
 *
 * It goes like this:
 *      AAAAAAAA BBBBBBBB CCCCCCCC
 * turns into the standard set of uuencode ascii chars indexed by 
 * numbers:
 *      00AAAAAA 00AABBBB 00BBBBCC 00CCCCCC
 *
 * Snip-n-shift, snip-n-shift, etc....
 */
 
   for (p=dst,i=0; i < srclen; i += 3) {
                /* Do 3 bytes of src */
                register char b0, b1, b2;
 
                b0 = src[0];
                if (i==srclen-1)
                        b1 = b2 = '\0';
                else if (i==srclen-2) {
                        b1 = src[1];
                        b2 = '\0';
                }
                else {
                        b1 = src[1];
                        b2 = src[2];
                }
 
                *p++ = uuset[b0>>2];
                *p++ = uuset[(((b0 & 0x03) << 4) | ((b1 & 0xf0) >> 4))];
                *p++ = uuset[(((b1 & 0x0f) << 2) | ((b2 & 0xc0) >> 6))];
                *p++ = uuset[b2 & 0x3f];
                src += 3;
   }
   *p = 0;      /* terminate the string */

   r = (unsigned char *)p - (unsigned char *)dst;
 
   /* Always do 4-for-3, but if not round threesome, have to go
          clean up the last extra bytes */
 
   for( ; i != srclen; i--)
                *--p = '=';
 
   return r;
}

