/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME
**   ux-util.h
**
** DESCRIPTION
**   Various util functions used during installation.
** NOTES
**   UNIX Only
** HISTORY
**
** $Log: ux-util.h,v $
** Revision 1.1.1.1  2005/07/29 22:16:33  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:30  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:18  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.2  1999/06/08 23:23:07  cxue
** APIs to get current user and group name
**
** Revision 1.1.1.1  1999/03/04 23:27:13  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.20.2.3  1998/11/09 17:43:22  cxue
** move InstallLog to setupapi.[h|cpp]
**
** Revision 1.1.2.20.2.2  1998/10/28 17:52:29  cxue
** fileName
**
** Revision 1.1.2.20.2.1  1998/09/10 16:33:03  cxue
** Add dirEmptry for checking empty directory
**
** Revision 1.1.2.20  1998/08/20 23:14:43  cxue
** remvoe formHostPortURL, calling stripedLdapURL in nsutils.cpp
**
** Revision 1.1.2.19  1998/08/19 17:45:24  cxue
** Add formHostPortURL to InstUtil
**
** Revision 1.1.2.18  1998/04/09 23:58:36  pvo
** restructure ux-util.h
**
** Revision 1.1.2.17  1998/03/31 22:18:18  pvo
** 112007 - Installer support for Admin's bind address.
**
** Revision 1.1.2.16  1998/03/13 21:24:51  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.15  1998/02/24 00:40:32  pvo
** Change to support change from globals.{h,cpp} to global.{h, cpp}
**
** Revision 1.1.2.14  1998/02/24 00:22:47  pvo
** Support topology change
**
** Revision 1.1.2.13  1998/01/27 02:33:16  pvo
** Changed getAdmPwd to retrieve the pwd from somewhere else.
**
** Revision 1.1.2.12  1998/01/22 04:52:03  pvo
** Added getAdmPwd() and doUUEncode()
**
** Revision 1.1.2.11  1998/01/09 03:24:48  pvo
** Added copyFile() and logTime() functions.
**
** Revision 1.1.2.10  1997/12/12 21:54:36  pvo
** Various bug fixes.
**
** Revision 1.1.2.9  1997/12/02 16:28:52  terencek
** temporary build fix
**
** Revision 1.1.2.8  1997/12/02 04:28:34  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.7  1997/11/26 01:14:31  pvo
** Added getSysVersion and getSelfUserID
**
** Revision 1.1.2.6  1997/11/16 22:34:15  pvo
** Added back installType.
**
** Revision 1.1.2.5  1997/11/13 19:28:56  pvo
** More new stuffs.
**
** Revision 1.1.2.4  1997/11/09 19:26:56  pvo
** Various changes due to nsdefs and ux-util API changes.
**
** Revision 1.1.2.3  1997/11/08 16:58:17  pvo
** isServerRoot now only detects a 4.0 server root
**
** Revision 1.1.2.2  1997/10/06 20:38:16  pvo
** Restructured dependencies.
**
** Revision 1.1.2.1  1997/09/27 02:47:33  pvo
** Checked in.
**
**********************************************************************
*/
#ifndef INST_UTIL_H
#define INST_UTIL_H

#include <errno.h>
#include "global.h"
#include "ldapu.h"
#include "nsutils.h"
#include "setupapi.h"

extern const char *INSTALL_TYPE;
   extern const char  *EXPRESS_TYPE;
   extern const char  *TYPICAL_TYPE;
   extern const char  *CUSTOM_TYPE;

/*
 * Package Type
 */
typedef enum {Normal=0, Update=1, Patch=2} PackageType;

/*
 ************************************************************************
 * Install Log Error Levels
 ***********************************************************************
 */

extern const char *FATAL;
extern const char *INFO;
extern const char *WARN;
extern const char *START;
extern const char *SUCCESS;

/*
 ********************************************************
 * Common C functions for those who use C as well as C++
 ********************************************************
 */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Giving a server root, get the default Ldap information
 */
extern void getDefaultLdapInfo (
         const char *sroot, 
         char **ldapURL, 
         char **ldapUser,
         char **installDN);

void unescape_ldap_basedn(char *str);


#ifdef __cplusplus
}
#endif


#ifndef __cplusplus

/*
 ************************************************************************
 * Nice to have C-wrapped utilities
 *   Refer to the C++ sections for more explanations
 ***********************************************************************
 */

extern char *guessHostname();
extern char *guessDomain();
extern int  guessPort();
extern Bool portAvailable(int port);
extern int  availUser(const char *user);
extern int  availGroup(const char *user, const char *group);
extern char *getDefaultHostName(const char *sroot);
extern char *getDefaultUser(const char *sroot);
extern char *getDefaultGroup(const char *sroot);
extern char *getSysVersion();
extern char *getCurrentDir(void);
extern Bool dirWritable(const char *dirName);
extern int  createDirectory(const char *dirName, int mode);
extern Bool fileExists(const char *fileName);
extern Bool dirExists(const char *dirName);
extern Bool isServerRoot(const char *directoryName);
extern Bool isLegacyServerRoot (const char *dirName);
extern int  execProgram(const char *program, const char *where);

/*
 ************************************************************************
 * Silent installation log
 ***********************************************************************
 */
typedef void * InstallLog;

InstallLog *createInstallLog(const char *fileName);
void       logMessage(InstallLog *, const char *level, const char *who, const char *msg, ...);

#else

/////////////////////////////////////////////////////////////////////////
// C++ starts here - For those who prefer this language.
/////////////////////////////////////////////////////////////////////////

#include "nsdefs.h"

/////////////////////////////////////////////////////////////////////////
// Various nice to have utility routines 
/////////////////////////////////////////////////////////////////////////

class InstUtil
{
public:

  /*
   * MEMBER:
   *   InstUtil::guessHostname
   *   InstUtil::guessDomain
   * DESCRIPTION:
   *   Obviously, try to guess the FQDN or just the Domain name portion
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static NSString guessHostname();
   static NSString guessDomain();

  /*
   * MEMBER:
   *   InstUtil::guessPort
   *   InstUtil::portAvailable
   * DESCRIPTION:
   *   guessPort - randomly select an available port
   *   portAvailable - determine whether the given port is available
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   static int      guessPort();

   static Bool     portAvailable(int port);

  /*
   * MEMBER:
   *   InstUtil::availUser
   * DESCRIPTION:
   *   Giving a UNIX userID, determine whether the user is available
   *
   * INPUTS:
   *   UNIX user ID
   * OUTPUTS:
   *   
   * RETURN:
   *   0 : user is okay
   *  -1 : user is not found
   *  -2 : chown failed
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static int availUser(const char *user);

  /*
   * MEMBER:
   *   InstUtil::availGroup
   * DESCRIPTION:
   *   Giving a user ID and a group, determine whether the combination
   *   is OK.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *    0 : group is OK
   *   -1 : user not found
   *   -2 : group not found
   *   -3 : user not in group
   *   -4 : chown on file with user/group failed
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static int availGroup (const char *user, const char *group);

  /*
   * MEMBER:
   *   InstUtil::getSysVersion
   * DESCRIPTION:
   *   Get the version of the current OS, in the form of
   *    <OS><version>.<release>
   *   e.g. SunOS5.5
   *
   * INPUTS:
   *
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   static NSString getSysVersion();

  /*
   * MEMBER:
   *   InstUtil::getDefaultHostName
   * DESCRIPTION:
   *   Giving a server root, get the configured FQDN of my machine
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static NSString getDefaultHostName(const char *sroot);

  /*
   * MEMBER:
   *   InstUtil::getHostIpAddress
   * DESCRIPTION:
   *   Giving a host name, get the associated IP address.
   * INPUTS:
   *
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   static NSString getHostIpAddress(const char *hostName);

  /*
   * MEMBER:
   *   InstUtil::getDefaultUser
   *   InstUtil::getDefaultGroup
   * DESCRIPTION:
   *   Giving a server root, get the configured UNIX user ID/group
   *   to run servers as.
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static NSString getDefaultUser(const char *sroot);
   static NSString getDefaultGroup (const char *sroot);

  /*
   * MEMBER:
   *   InstUtil::getAdmPwd
   * DESCRIPTION:
   *   Giving a server root, get the Admin's password that
   *   could be used to authenticate with DS
   * INPUTS:
   *   server root
   * OUTPUTS:
   *   Admin's password
   * RETURN:
   *   the admin's password (allocated)
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */

   static NSString getAdmPwd(const char *sroot);

  /*
   * MEMBER:
   *   InstUtil::isSelfRootUser()
   * DESCRIPTION:
   *   Check if current login ID is root of the running process
   *
   * INPUTS:
   *
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   static Bool isSelfRootUser();

  /*
   * MEMBER:
   *   InstUtil::getSelfUserID()
   * DESCRIPTION:
   *   Get the (effective) login ID of the running process
   *
   * INPUTS:
   *
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   static NSString getSelfUserID();

  /*
   * MEMBER:
   *   InstUtil::getSelfGroupID()
   * DESCRIPTION:
   *   Get the (effective) group ID of the running process
   *
   * INPUTS:
   *
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   static NSString getSelfGroupID();


  /*
   * MEMBER:
   *   InstUtil::getCurrentDir
   * DESCRIPTION:
   *   Get the current working directory
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static NSString getCurrentDir(void);

  /*
   * MEMBER:
   *   InstUtil::dirWritable
   * DESCRIPTION:
   *   Giving a full path of a directory, check if dir exists and
   *   writable.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   static Bool dirWritable(const char *dirName);

  /*
   * MEMBER:
   *   InstUtil::dirName
   * DESCRIPTION:
   *   Get the Directory name portion of a filename
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   static NSString dirName(const char *path);

  /*
   * MEMBER:
   *   InstUtil::fileName
   * DESCRIPTION:
   *   Get the file name portion of a filename
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   static NSString fileName(const char *path);

   /*
   * MEMBER:
   *   InstUtil::createDirectory
   * DESCRIPTION:
   *   Giving a full path of a directory, create it.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   static int createDirectory(const char *dirName, int mode);

  /*
   * MEMBER:
   *   InstUtil::fileExists
   *   InstUtil::dirExists
   * DESCRIPTION:
   *   Determine if the given filename or dirname exists
   *   Note that fileExists determines whether a file or 
   *    a directory exists or not while dirExists determines
   *    whether a directory exists.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static Bool fileExists(const char *fileName);
   static Bool dirExists(const char *dirName);
   static int  copyFile(const char *dest, const char *src);
   static Bool dirEmpty(const char *dirName);

  /*
   * MEMBER:
   *   InstUtil::isServerRoot
   *   InstUtil::isLegacyServerRoot
   * DESCRIPTION:
   *   Try to guess as much as we can whether the given directory
   *   is a 4.0 server root. isLegacyServerRoot detects whether
   *   the directory is a pre-4.0 server root or not.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   static Bool isServerRoot(const char *directory);
   static Bool isLegacyServerRoot(const char *directory);

  /*
   * MEMBER:
   *   InstUtil::execProgram
   * DESCRIPTION:
   *   Execute the given program.
   *
   * INPUTS:
   *   program - the program to execute
   *   where   - directory to go to before executing the program. 
   *             execProgram will then return to the current dir after
   *             execution.
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   static int  execProgram(const char *program, const char *where = 0);

   static int doUUEncode (const unsigned char *, unsigned char *, int);
};


#endif /* cplusplus */


#endif /* INST_UTIL_H */
