/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME:
**   wrapper.cc
**
** DESCRIPTION:
**   C wrapper for those who can't handle C++ API.
**
** NOTES:
**   UNIX Only
**
** HISTORY:
**
** $Log: ux-wrapper.cc,v $
** Revision 1.3  2006/03/30 00:38:29  rmeggins
** Add using namespace std; after the include <iostream>
** Remove #include <iostream.h> in files that also include nsdefs.h
** Fix a potential buffer non-termination
**
** Revision 1.2  2006/03/23 15:59:22  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:33  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:30  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:18  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.1  1999/03/04 23:27:13  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.3  1997/11/26 01:14:30  pvo
** Added getSysVersion and getSelfUserID
**
** Revision 1.1.2.2  1997/11/09 19:26:57  pvo
** Various changes due to nsdefs and ux-util API changes.
**
** Revision 1.1.2.1  1997/09/27 02:47:33  pvo
** Checked in.
**
*********************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "ux-util.h"

extern "C"
{
char *guessHostname()
{
   NSString hn;

   hn = InstUtil::guessHostname();
   return strdup(hn.data());
}
 
char *guessDomain()
{
   NSString dn;

   dn = InstUtil::guessDomain();
   return strdup(dn.data());
}

int 
guessPort()
{
   return InstUtil::guessPort();
}

Bool portAvailable(int port)
{
   return InstUtil::portAvailable(port);
}

int  availUser(const char *user)
{
   return InstUtil::availUser(user);
}

int  availGroup(const char *user, const char *group)
{
   return availGroup(user,group);
}

char *getSysVersion()
{
   NSString sysVersion = InstUtil::getSysVersion();
   return strdup(sysVersion.data());
}

char *getDefaultHostName(const char *sroot)
{
   NSString hn = InstUtil::getDefaultHostName(sroot);
   return strdup(hn.data());
}

char *getDefaultUser(const char *sroot)
{
   NSString us = InstUtil::getDefaultUser(sroot);
   return strdup(us.data());
}

char *getDefaultGroup(const char *sroot)
{
   NSString ug = InstUtil::getDefaultGroup(sroot);
   return strdup(ug.data());
}
char *getCurrentDir(void)
{
  NSString cd = InstUtil::getCurrentDir();
  return strdup(cd.data());
}

Bool dirWritable(const char *dirName)
{
   return InstUtil::dirWritable(dirName);
}
int  createDirectory(const char *dirName, int mode)
{
   return InstUtil::createDirectory(dirName, mode);
}

Bool fileExists(const char *fileName)
{
   return InstUtil::fileExists(fileName);
}

Bool dirExists(const char *dirName)
{
   return InstUtil::dirExists(dirName);
}

Bool isServerRoot(const char *directoryName)
{
   return InstUtil::isServerRoot(directoryName);
}

int
execProgram(const char *program, const char *where)
{
   return InstUtil::execProgram(program, where);
}

void * 
createInstallLog(const char *fileName)
{
   InstallLog *log = new InstallLog(fileName);

   return log;
}

void 
logMessage(void *log, const char *level, const char *who , const char *msg, ...)
{
   char buf[BIG_BUF];
   va_list arg;
   FILE *fp;
   InstallLog *p = (InstallLog *) log;

   if (p->_logFile == NULL || 
       (fp = fopen(p->_logFile, "a")) == NULL)
   {
      return;
   }
   snprintf(buf, sizeof(buf), "[%s] %s", who, level);
   buf[sizeof(buf)-1] = 0;   

   va_start (arg, msg);
   vsnprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), msg, arg);
   va_end(arg);

   buf[sizeof(buf)-1] = 0;
   fputs(buf,fp);
   fputs("\n",fp);

   fclose(fp);

   return;
}



} /* End extern "C" */
