/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME
**   product.h
**
** DESCRIPTION
**   Definition for Product Info object
**
** NOTES
**
** HISTORY
**
** $Log: product.h,v $
** Revision 1.2  2005/12/13 20:58:22  nhosoi
** [175418] Admin Server ns-update crashes if necessary attribute-value is not given
** 1) Although Admin Server's PostInstall program ns-update expects it, PostInstall
** framework in SetupUtil does not take silent install option "-s".  Changed to pass the option.
** 2) PostInstall framework does not support logging. Changed to pass the option.
** 3) PostInstall failure is issued as WARNING and the installation finishes with
** Success message even though it really failed.  Increased the error level to FATAL.
**
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.9  2001/11/16 00:18:58  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.8  1999/12/04 00:34:59  cxue
** SecurityCheck and RequireDomain support in setup.inf
**
** Revision 1.1.1.7  1999/11/12 01:58:01  cxue
** Keep going even no archive is defined in <product>.inf
**
** Revision 1.1.1.6  1999/10/13 00:19:27  cxue
** chnage <serverroot> and go forward
**
** Revision 1.1.1.5  1999/09/27 20:18:42  cxue
** check diskspace after selection is done
**
** Revision 1.1.1.4  1999/09/17 01:48:49  cxue
** fix long lasting cross-dependence bug
**
** Revision 1.1.1.3  1999/05/25 18:04:07  cxue
** bug 353186, allow preInstall to customize archive files
**
** Revision 1.1.1.2  1999/04/02 18:05:10  cxue
** bug 347418: invisible component check
**
** Revision 1.1.1.1  1999/03/04 23:27:07  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.19.2.10  1999/02/04 02:24:46  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.19.2.9  1999/01/21 00:15:29  cxue
** Stop supporting downgrade installation
**
** Revision 1.1.2.19.2.8  1999/01/20 01:08:25  cxue
** support UpgradeOnly installation
**
** Revision 1.1.2.19.2.7  1998/12/18 19:37:10  cxue
** bad memory: reference memory which is released
**
** Revision 1.1.2.19.2.6  1998/12/04 02:49:22  cxue
** billx uninstallation and customizable destination support
**
** Revision 1.1.2.19.2.5  1998/11/04 19:42:35  pvo
** Enable option for skip build number checking
**
** Revision 1.1.2.19.2.4  1998/10/28 17:55:51  cxue
** support userdefined default setting
**
** Revision 1.1.2.19.2.3  1998/10/09 21:27:12  cxue
** select invisible components if it has dependency on it and mandatory.
**
** Revision 1.1.2.19.2.2  1998/10/03 00:25:39  cxue
** move version related APIs to nsinst.cpp
**
** Revision 1.1.2.19.2.1  1998/09/24 01:57:41  cxue
** In the absence of components in current package, setup checks version number of dependency at server root
**
** Revision 1.1.2.19  1998/08/22 00:44:05  cxue
** call insertLdifEntries after calling post-install
**
** Revision 1.1.2.18  1998/07/16 17:12:24  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.17  1998/06/24 17:41:15  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.16  1998/06/01 21:20:09  pvo
** 122446 - Uninstall prematurely deletes itself
**
** Revision 1.1.2.15  1998/05/01 01:06:51  pvo
** Fix problem with skipping extraction
**
** Revision 1.1.2.14  1998/04/01 02:42:31  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.13  1998/03/09 22:09:38  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.12  2000/03/03 21:39:59  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.11  1998/02/26 00:14:01  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.10  1998/02/15 22:06:41  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.9  1998/01/29 21:52:23  pvo
** Bug 105041 - add -h (help) screen for setup
** Bug 104968 - change default LDAP suffix to domain.com rather than example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.8  1998/01/22 04:51:30  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.7  1997/12/12 21:54:22  pvo
** Various bug fixes.
**
** Revision 1.1.2.6  1997/12/04 00:03:14  pvo
** Prepare support for DS installation.
**
** Revision 1.1.2.5  1997/11/26 02:03:01  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.4  1997/11/13 19:28:42  pvo
** More new stuffs. Changed .pi to .inf.
**
** Revision 1.1.2.3  1997/11/11 23:17:36  pvo
** Implemented preUninstall and postUninstall.
**
** Revision 1.1.2.2  1997/11/11 05:41:10  pvo
** Added dependency checking for uninstallation.
**
** Revision 1.1.2.1  1997/11/08 16:47:48  pvo
** New. Uninstallation support.
**
**
**********************************************************************/

/*********************************************************************
**
** Includes
**********************************************************************
*/

#ifndef NS_PRODUCT_H
#define NS_PRODUCT_H

extern "C" {
#include <stdio.h>
}

#include "nsdefs.h"
#include "ux-util.h"
// #include "net_if.h"

extern const char *EXTRACT_FLAGS;
extern const char *EXTRACT_PROGRAM;
typedef NVPair PackageInfo;

class Components;

class Product
{
public:

   Product ();

   Product (PackageInfo *);

   ~Product();

   void init();

   int retrieveSuiteSpot(const NSString & sourceDir, const char *);
   int retrieveProduct(const NSString & sourceDir, const char *fileName, const char *nickName);
   int retrieveDownloadProduct(NVPair *downloadInfo);

   const char *get(const char *attr) const;

   void  set(const char *attr, const char *value);

   PackageInfo *package() const
     { return _packageInfo; }

   int checkCompatibility(const Product *, Bool checkBuildNum) const;

   int checkSecurity(const Product *) const;

   Bool isIdentical (const Product *) const;

   Bool exists () const
      { return _exists; }

   void exists(Bool val)
      { _exists = val; }

   PackageType packageType() const;

   Bool installed() const
      { return _installed; }
   void installed(Bool val)
      { _installed = val; }

   Bool compatible() const
      { return _compatible; }
   void compatible(Bool val)
      { _compatible = val; }

   Bool securityChecked() const
      { return _securityChecked; }
   void securityChecked(Bool val)
      { _securityChecked = val; }

   Bool downgrade() const
      { return _downgrade; }
   void downgrade(Bool val)
      { _downgrade = val; }

   Bool isChecked() const;
   Bool isMandatory() const;
   Bool isVisible() const;

   Bool isSelected() const
   { return _skipped ? False : _selected; }
   void select()
      { _selected = True; }

   void deSelect()
      { _selected = False; }

   Bool skipped() const
      { return _skipped; }
   void skip(Bool val) 
      { _skipped = val; }

   Bool skippedConfig() const
      { return _skippedConfig; }
   void skipConfig()
      { _skippedConfig = True; }

   void selectDefaults();
   void selectAll(); 
   void deSelectAll();

   /*
    * Retrieve dependency information for this product
    */
   NVPair *dependencies(const char *dependencyName) const;

   /*
    * Retrieve all components and place them in the input Components
    * structure
    */
   void getAllComponents(Components &);

   /*
    * Get the components associated with this product
    */
   Components *getComponents() const
      { return _components; }

   Product *getComponent(int i) const;

   Product *getComponent(const char *nickName) const;

   /*
    * Number of components associated with this product
    */
   int numComponents() const;

   int numVisibleComponents() const;

   /*
    * Installation stuffs
    */
   NSString install(const NSString & serverRoot, const NSString & source, NVPair *installInfo);

   int preInstall(const NSString & serverRoot, const NSString &sourceDir, 
                  const char *installType, 
                  InstallMode installMode, const char *logFile, 
                  const char *cache) const;

   int postInstall(const NSString & serverRoot, const NSString & sourceDir,
                   const char *infoFile, const InstallInfo *nvpair,
                   InstallMode installMode, const char *logFile) const;

   NSString extract(const char *dest, const char *archive, 
                    const char *extractProgram=NULL,
                    const char *whichFile = NULL,
                    NVPair *installInfo = NULL);

   // int download(Inet &, const char *source, const char *destination);

   long requiredDiskSpace(const NSString & sourceDir, NVPair *installInfo);

   /*
    * Uninstallation stuffs
    */
   int unInstall(const NSString &serverRoot) const;
   int preUnInstall(const NSString &serverRoot, const char *infoFile) const;
   int postUnInstall(const NSString &serverRoot, const char *infoFile) const;
   int insertLdifFiles(const NSString & sourceDir, const NVPair *nvpair) const;
   int insertSchemaFiles(const NSString & sourceDir, const NVPair *nvpair) const;

   NVPair *logFile() const
      { return _logFile; }

   void updateInstalledInfo(NVPair *, const NSString &, const NSString &);

   void restoreFiles(const NSString &);

   void setArchiveSize(long size);
   long getArchiveSize();

   /*
    * Get customized info if it's defined in installation cache
    */
  const char * getInfo(NVPair *installInfo, const char *name);

  static Bool _needSecurity;

private:
   PackageInfo *_packageInfo;

   Bool _selected;
   Bool _skipped;
   Bool _skippedConfig;

   /*
    * A product of the same type is already installed
    */
   Bool _exists;
   /*
    * Server root is compatible
    */
   Bool _compatible;

   Bool _securityChecked;

   Bool _downgrade;

   /*
    * Identical and is already installed
    */
   Bool _installed;

   NSString untar (const char *dest, const char *archive, const char *program, const char *arg, const char *whichFile=NULL);
   NSString unzip (const char *dest, const char *archive, const char *whichFile=NULL, NVPair *installInfo=NULL);

   void backupFiles(const char *, const NSString &, const NSString &);

   /*
    * The components associated with this product
    */
   Components *_components;

   void clean();

   NVPair *_logFile;
   NSString _logDest;

   void relocate(const char *serverRoot, NSString & file) const;


  /*
   * archive size in kilobyte
   */
  long _archiveSize;
  void unzipInfo(const char *zipFile, NVPair *installInfo);

};

#define MAX_PRODUCTS 25

class Components
{
public:
   Components();

   ~Components();

   int numComponents() const
      { return _numComponents; }

   int numSelectedComponents() const;

   void selectComponent(const char *alias);

   void deselectComponent(const char *alias);

   void addComponent(Product *);

   int searchProduct(const char *nickName, const char *version);

   Product *getComponent(const char *nickName);

   Product *getComponent(int i);

   void sortComponents();

   NSString checkInstallDependency(Product *product, Components *);

   int checkProductDependencies(Product *product, char *nickname);

   void selectMandatoryComponents();

   void  selectInstallDependency();


   Bool  useLdap();

   Bool  userDirAuth();

   Bool  hasVisible();

   void  top()
     { _top = True; }

   Bool isTop()
     { return _top; }


private:
   int _numComponents;
   Product **_products;
   Bool _top;
};

int dependencySort (Product **);
void setUnzipSigHandler(void (*handler)(int));

#endif
