/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/**********************************************************************
** NAME
**   uninstaller.h
**
** DESCRIPTION
**
** NOTES
**
** HISTORY
**
** $Log: uninstaller.h,v $
** Revision 1.2  2006/03/30 00:38:29  rmeggins
** Add using namespace std; after the include <iostream>
** Remove #include <iostream.h> in files that also include nsdefs.h
** Fix a potential buffer non-termination
**
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.5  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.4  2005/07/27 03:01:39  nhosoi
** [161408] Open Sourcing Setup Util
** Replacing "#include <iostream>" with "#include <iostream.h>" for HP-UX.
**
** Revision 1.3  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.2  1999/04/16 17:12:09  cxue
** bug 347119: uninstaller doesn't call pre- post- uninstall program
**
** Revision 1.1.1.1  1999/03/04 23:27:08  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.9.6.2  1999/02/04 02:24:47  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.9.6.1  1998/09/09 18:17:00  cxue
** Silent uninstall support
**
** Revision 1.1.2.9  1998/06/01 21:20:10  pvo
** 122446 - Uninstall prematurely deletes itself
**
** Revision 1.1.2.8  1998/03/03 21:39:59  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.7  1998/02/04 00:01:54  pvo
** 104824 - DEC uninstall goes out of control
**
** Revision 1.1.2.6  1998/01/09 03:27:32  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.5  1997/12/12 21:54:23  pvo
** Various bug fixes.
**
** Revision 1.1.2.4  1997/11/26 02:03:02  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.3  1997/11/13 19:28:46  pvo
** More new stuffs. Changed .pi to .inf.
**
** Revision 1.1.2.2  1997/11/11 05:41:11  pvo
** Added dependency checking for uninstallation.
**
** Revision 1.1.2.1  1997/11/08 16:47:49  pvo
** New. Uninstallation support.
**
**
***********************************************************************/

#ifndef NS_UNINSTALLER_H
#define NS_UNINSTALLER_H

/**********************************************************************
**
** Includes
**
**********************************************************************/
extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
}

#include "nsdefs.h"
#include "ux-util.h"
#include "dialog.h"
#include "product.h"
#include "ux-setup.h"

extern const char *selectionText;
/*
** Forward Reference
*/
class UnInstaller;

/*
** CLASS:
**   UnInstaller
**
** DESCRIPTION:
**   The SuiteSpot UnInstaller
**
*/
class UnInstaller:public SetUp
{
public:

   UnInstaller(int, char **);
   int  init();
   void run();

protected:
	Bool _silent;

private:
   InstallMode _unInstallMode;   /* Silent or Interactive */

   /*
    * Private functions
    */
   void getOptions(int argc, char **argv);
   void initDefaultAnswer(InstallInfo *);
   int  initDialog();

   int  unInstall();
   int  doUnInstall(Product &product);
   void unInstallHiddenComponents(Product &product);
   void unInstallComponent(Product *product);
   void updateSuiteSpotInfo();

   Bool serverRootEmpty() const;

};

#endif
