/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** NAME
**   ux-setup.cc
**
** DESCRIPTION
**   Base class for intallation programs
**
** NOTES:
**
** HISTORY
**
** $Log: ux-setup.cc,v $
** Revision 1.2  2006/03/23 15:59:17  rmeggins
** Bug(s) fixed: 186280
** Bug Description: Close potential security vulnerabilities in CGI code
** Reviewed by: Nathan & Noriko (Thanks!)
** Fix Description: Mostly cleaned up usage of sprintf.
** Platforms tested: Fedora Core 5
** Flag Day: no
** Doc impact: no
** QA impact: should be covered by regular nightly and manual testing
** New Tests integrated into TET: none
**
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.2  2001/11/16 00:18:58  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.1  1999/03/04 23:27:09  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.23.2.4  1999/02/04 23:57:10  cxue
** resource changes
**
** Revision 1.1.2.23.2.3  1999/02/04 02:24:48  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.23.2.2  1998/10/28 17:55:52  cxue
** support userdefined default setting
**
** Revision 1.1.2.23.2.1  1998/10/15 22:22:07  cxue
** resource framework for custom ui
**
** Revision 1.1.2.23  1998/08/26 00:29:08  pvo
** 316114 - ctrl-b crashes setup
**
** Revision 1.1.2.22  1998/08/19 17:49:54  cxue
** Change to support Config only directory server and user only directory server
**
** Revision 1.1.2.21  1998/07/18 01:26:54  pvo
** Add free(localLdapURL)
**
** Revision 1.1.2.20  1998/07/02 03:30:34  pvo
** Some minor name changes
**
** Revision 1.1.2.19  1998/05/13 04:39:32  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.18  1998/05/01 23:24:43  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.17  1998/04/01 02:42:32  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.16  1998/03/28 03:29:03  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.15  1998/03/13 21:24:49  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.14  1998/03/13 05:00:38  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.13  1998/03/09 22:09:39  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.12  2000/03/03 21:40:00  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.11  1998/02/26 04:02:00  pvo
** Fixed break in download.cc due to the call to set_ldap_option() in libinstall.
** Also try to detect input error when asking for installDN and try to
** create installDN if possible in case the user has the privilege to create
** it.
**
** Revision 1.1.2.10  1998/02/24 00:23:08  pvo
** Support for topology changes.
**
** Revision 1.1.2.9  1998/02/04 00:01:55  pvo
** 104824 - DEC uninstall goes out of control
**
** Revision 1.1.2.8  1998/01/29 21:52:24  pvo
** Bug 105041 - add -h (help) screen for setup
** Bug 104968 - change default LDAP suffix to domain.com rather than example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.7  1997/12/18 02:10:23  pvo
** Fixed problem on HPUX with tmpnam.
**
** Revision 1.1.2.6  1997/12/16 23:39:33  pvo
** Changed cn=NetscapeRoot to o=NetscapeRoot.
**
** Revision 1.1.2.5  1997/12/15 23:50:28  pvo
** Fixed FTP path.
**
** Revision 1.1.2.4  1997/12/12 21:54:23  pvo
** Various bug fixes.
**
** Revision 1.1.2.3  1997/12/03 19:02:35  pvo
** Display error if cn=NetscapeRoot is not setup correctly.
**
** Revision 1.1.2.2  1997/12/02 04:28:29  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.1  1997/11/26 02:03:03  pvo
** Restructure files and add Download support.
**
**
***********************************************************************/

extern "C" {
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>

}

#include "ldapu.h"
#include "utf8.h"
#include "ux-util.h"
#include "ux-setup.h"
#include "nsutils.h"
#include "dialog.h"

const char *SITE_INFO_FILE = "suitespot.sif";
const char *PLATFORM_INFO_FILE = "suitespot.oif";
const char *DOWNLOAD_INFO_FILE = "suitespot.dif";

/* Temporary for now */
const char *NETSCAPE_URL = "ftp://lupine/pub/download";

const char *SYS_VERSION = "SysVersion";
const char *DOWNLOAD_ROOT = "DownloadRoot";
   const char *DEFAULT_DNLD_ROOT = DEFAULT_SOURCE;

const char *DOWNLOAD_OPTION = "DownloadOption";
   const char *DOWNLOAD_ONLY= "1";
   const char *DOWNLOAD_AND_INSTALL = "2";

const char *DOWNLOAD_SOURCE = "DownloadSource";
   const char *FROM_NETSCAPE = "1";
   const char *FROM_INTRANET = "2";

extern const char *SOURCE_URL = "SourceURL";
extern const char *NETSCAPE_SOURCE_URL = "NetscapeURL";
extern const char *INTRANET_SOURCE_URL = "IntranetURL";

extern const char *PLATFORM = "Platform";

SetUp::SetUp()
  :_installInfo(0), _dialogManager(0)
{
   char buf[SML_BUF];

   _workDir = InstUtil::getCurrentDir();
   if (InstUtil::dirWritable(_workDir))
   {
      _tmpDir = _workDir;
   }
   else
   {
      char *td = tmpnam(buf);

      _tmpDir = InstUtil::dirName(td);

   }
   _newComponents = new Components;
   _oldComponents = new Components;

   initUserDefault();
}

Product *
SetUp::oldSuiteSpot()
{
   return &_oldSuiteSpot;
}

Product *
SetUp::newSuiteSpot()
{
   return &_newSuiteSpot;
}

int
SetUp::getInstalledProducts(const NSString & serverRoot)
{
   NSString sourceDir;

   if (InstUtil::dirExists(serverRoot) == False)
   {
      return -1;
   }

   if (_serverRoot == (char *) NULL || _serverRoot != serverRoot)
   {
      if (_serverRoot != (char *) NULL)
      {
         if (_oldComponents)
         {
            delete _oldComponents;
            _oldComponents = new Components;
         }
         _oldSuiteSpot.init();
      }

      _serverRoot = serverRoot;

      _existingInfo.clear();

      sourceDir = serverRoot + "/" + SETUP_DIR;

      if (_oldSuiteSpot.retrieveSuiteSpot(sourceDir, SUITESPOT_INF) == 0 && _oldSuiteSpot.numComponents())
      {
         _oldSuiteSpot.getAllComponents(*_oldComponents); 
      }
      else
      {
         return -1;
      }

      //  We are sure it's re-installation, let's collect some
      //  information from server root

      char *ldapURL = NULL, *ldapUser = NULL, *adminDomain = NULL;
      _existingInfo.set(SETUP_GLOBAL_INF_SVR_ROOT, _serverRoot);

      setupGetDefaultLdapInfo(_serverRoot, &ldapURL, &ldapUser, &adminDomain );

      if (ldapURL)
         _existingInfo.set(SETUP_GLOBAL_INF_LDAP_URL, ldapURL);
      if (ldapUser)
         _existingInfo.set(SETUP_GLOBAL_INF_LDAP_USER, ldapUser);
      if (adminDomain)
         _existingInfo.set(SETUP_GLOBAL_INF_ADMIN_DOMAIN, adminDomain);
   }

   return 0;
}

int
SetUp::existingLdapCheck()
{
   char *ldapURL = NULL, *ldapUser = NULL, *installDN = NULL;
   LDAPURLDesc *ludpp = NULL;
   int rc = -1;

   setupGetDefaultLdapInfo (_serverRoot, &ldapURL, &ldapUser, &installDN);

   if (ldapURL && ldap_url_parse(ldapURL, &ludpp) == 0)
   {
      _installInfo->set(CONFIG_LDAP_URL, ldapURL);
      _installInfo->set(DS_ADMIN_DOMAIN, installDN);

      ldap_free_urldesc(ludpp);

      if (ldapUser)
      {
         _installInfo->set(MC_ADMIN_ID, ldapUser);
      }

      // Now check whether this LDAP URL is valid or not
      // rc = verifyLdapUrl(ldapURL, NULL, NULL);
      rc = 0;
   }


   if (ldapURL)
      free(ldapURL);
   if (ldapUser)
      free(ldapUser);

   return rc;
}

NSString
SetUp::verifyLdapServer (const char *ldapURL)
{
   int err;
   char  errMsg[SML_BUF];
   char *localLdapURL = UTF8ToLocal(ldapURL);

   err = verifyLdapUrl(ldapURL, _bindDN, _bindPwd);

   switch (err)
   {
      case OKAY:
        errMsg[0] = 0;
        break;
      case INVALID_URL:
        snprintf(errMsg, sizeof(errMsg), "The URL \"%s\" is not of valid format.\n", localLdapURL);
        break;
      case CONN_FAILED:
        snprintf(errMsg, sizeof(errMsg),
"Cannot connect to URL \"%s\".\n"
"       The server may have been down. Please fix the problem\n"
"       before proceeding with installation.\n",
                localLdapURL);
        break;
      case INVALID_DN:
        snprintf(errMsg, sizeof(errMsg),
"setup cannot verify the base suffix as specified in\n"
"            \"%s\".\n"
"       Please check the base suffix and re-enter the URL.\n",
                        localLdapURL);
        break;
      default:
        // Assume it's up and running
        break;
   }
   free(localLdapURL);
   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

NSString
SetUp::verifyLdapUser(const char *ldapURL, const char *ldapUser, const char *ldapPwd)
{
   LdapError err;
   char errMsg[SML_BUF];
   char *localLdapURL = UTF8ToLocal(ldapURL);

   errMsg[0] = 0;

   Ldap ldap(err, ldapURL, ldapUser, ldapPwd, _bindDN, _bindPwd);

   if (err)
   {
       switch(err)
       {
          case INVALID_INPUT:
             snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYLDAPUSER1));
             break;
          case INVALID_URL:
             snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYLDAPUSER2), localLdapURL);
             break;
          case INVALID_AUTH:
             snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYLDAPUSER3));
             break;
          case CONN_FAILED:
             snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYLDAPUSER4));
             break;
          case INVALID_USER:
             snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYLDAPUSER5));
             break;
          default:
             snprintf(errMsg, sizeof(errMsg), getResource(RES_MESSAGE_VERIFYLDAPUSER6));
             break;
       }
   }

   free (localLdapURL);

   errMsg[sizeof(errMsg)-1] = 0;
   return errMsg;
}

void 
SetUp::initUserDefault()
{
   if (getenv("HOME") != NULL)
   {
      char filename[200];

      snprintf(filename, sizeof(filename), "%s/%s", getenv("HOME"), ".nssetup-default");
      filename[sizeof(filename)-1] = 0;
      _userDefault.setFormat(1);
      _userDefault.read(filename);
   }
}

void 
SetUp::terminate(int errcode)
{
   if (_infoFile != (char *) NULL)
      unlink(_infoFile.data());

   DialogManager::disableWinMode();

   exit(errcode);
}
