/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
#include "nsdefs.h"
#include "product.h"
#include "uxres.h"
#include "setupdefs.h"

extern "C" {
#if defined(AIX)
#include <strings.h>
#endif
#include <string.h>
}

// NVPair customResource;
NVPair globalResource;

void initResource(NVPair *nv)
{
   const char *s;
   int i;

   if (globalResource.isEmpty() == True)
   {
      for (i = 0; string_table[i].id; i++)
         globalResource.set(string_table[i].id, string_table[i].str);
   }   
   
   if (nv == NULL)
      return;

   for (i = 0; i < nv->numName(); i ++)
      setResource(nv->name(i), nv->get(nv->name(i))); 
}

void setResource(const char *id, const char *text)
{
   if (text && text[0])
      globalResource.set(id, text);
}

char * getResourceString(NVPair & nv, const char *label)
{
   int i, j;
   int len;
   char *new_value;
   const char *value;

   value = nv.get(label);

   if (value == NULL)
      return NULL;

   len = strlen(value);
   new_value = (char *)malloc(len + 1);

   for (i = 0, j = 0; i < len; i++, j++)
   {
      // Surrounding quotes should be dropped
      if ( value[i] == '"' )
      {
         // Decrement j so we don't get an extra space
         j--;
         continue;
      }

      // Take care of newlines and escaped quotes
      if ( value[i] == '\\' )
      {
         if ( value[i+1] == 'n' )
         {
            new_value[j] = '\n';
            i++;
         }
         else if ( value[i+1] == '"' )
         {
            new_value[j] = value[i+1];
            i++;
         }
      }
      else
      {
         new_value[j] = value[i];
      }
   }

   new_value[j] = '\0';

   return new_value;
}

NSString getResource(const char *id)
{
   int i;
   char *str = NULL;
   char *text;
   NSString rtvalue;

   if (globalResource.isEmpty() == True)
      initResource(NULL);

   str = getResourceString(globalResource, id);

   if (str == NULL)
      return NULL;

   rtvalue = str;
   return rtvalue;
}

NSString getResource(const char *id, const char *arg)
{
   int i;
   char *str = NULL;
   char *text;
   NSString rtvalue;

   if (globalResource.isEmpty() == True)
      initResource(NULL);

   str = getResourceString(globalResource, id);

   if (str == NULL)
      return NULL;

   rtvalue = str;

   if (NULL != arg)
   {
      if (text = getResourceString(globalResource, arg))
      {
         char *str_rtvalue = (char *)malloc(strlen(str) + strlen(text) + 1);
         if (NULL == str_rtvalue)
             return NULL;
         sprintf(str_rtvalue, str, text);
         rtvalue = str_rtvalue;
         free(str);
         free(text);
      }
   }

   return rtvalue;
}
