#
# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation version
# 2.1 of the License.
#  
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#  
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# END COPYRIGHT BLOCK
#
# This is the default target for all Makefiles which include this file.
# Those Makefiles should define this target with the appropriate dependencies
# and build rules.
#
all:

include $(BUILD_ROOT)/nsdefs.mk
include $(BUILD_ROOT)/component_versions.mk

NSOS_TEST1       := $(shell uname -m)

ifeq (86, $(findstring 86, $(NSOS_TEST1)))
  ifeq (x86, $(findstring x86, $(NSOS_TEST1)))
    CPU_ARCH := $(NSOS_TEST1)
  else
    CPU_ARCH := x86
  endif
else 
  CPU_ARCH := $(NSOS_TEST1)
endif

NSCONFIG         = $(NSPLATFORM_DIR)

ifeq ($(USE_PTHREADS),1)
  PTHREAD_TAG   = _PTH
endif

ifeq ($(NSOS_ARCH),Linux)
  ifeq ($(USE_LIBC),1)
    LIBC_VERSION  = _libc
  else
    LIBC_VERSION  = _glibc
  endif
  PTHREAD_TAG   = _PTH
  NSCONFIG        = $(NSOS_ARCH)$(NSOS_RELEASE)_$(CPU_ARCH)$(LIBC_VERSION)$(PTHREAD_TAG)
  NSCONFIG_NOTAG  = $(NSCONFIG)
  NSOBJDIR_NAME   = $(NSCONFIG)$(NSOBJDIR_TAG).OBJ
else
  ifeq ($(NSOS_ARCH),RHEL)
    PTHREAD_TAG   = 
    NSCONFIG        = $(NSOS_ARCH)$(NSOS_RELEASE)_$(CPU_ARCH)_$(GCC_VERSION)
    NSCONFIG_NOTAG  = $(NSCONFIG)
    NSOBJDIR_NAME     = $(NSCONFIG)$(PTHREAD_TAG)$(NSOBJDIR_TAG).OBJ
  else
    ifeq ($(NSOS_TEST1),i86pc)
      NSCONFIG         = $(NSOS_ARCH)$(NSOS_RELEASE)_$(NSOS_TEST1)
      NSCONFIG_NOTAG   = $(NSOS_ARCH)$(NSOS_RELEASE_NOTAG)_$(NSOS_TEST1)
    endif
    ifeq ($(NSOS_ARCH),HP-UX)
      ifeq ($(NSOS_TEST1), ia64)
        NSCONFIG         = $(NSOS_ARCH)$(NSOS_RELEASE)_$(NSOS_TEST1)
        NSCONFIG_NOTAG   = $(NSOS_ARCH)$(NSOS_RELEASE_NOTAG)_$(NSOS_TEST1)
      else
        NSCONFIG         = $(NSOS_ARCH)$(NSOS_RELEASE)
        NSCONFIG_NOTAG   = $(NSOS_ARCH)$(NSOS_RELEASE_NOTAG)
      endif
    else
        NSCONFIG         = $(NSOS_ARCH)$(NSOS_RELEASE)
        NSCONFIG_NOTAG   = $(NSOS_ARCH)$(NSOS_RELEASE_NOTAG)
    endif
    ifeq ($(NSOS_ARCH),WINNT)
      ifeq ($(DEBUG), fulld)
       NSOBJDIR_NAME     = $(NSCONFIG)$(NSOBJDIR_TAG).OBJD
      endif
    endif
    NSOBJDIR_NAME     = $(NSCONFIG)$(NS64TAG)$(PTHREAD_TAG)$(NSOBJDIR_TAG).OBJ
  endif
endif

MAKE=gmake $(BUILDOPT) USE_PTHREADS=$(USE_PTHREADS) SECURITY=$(SECURITY) MOZILLA_SOURCE_ROOT_EXT=$(MOZILLA_SOURCE_ROOT_EXT) 

export NSPR20=1

NSDEFS=SERVER_BUILD=1 XCFLAGS=$(MCL_CFLAGS) USE_PTHREADS=$(USE_PTHREADS) \
       NS_PRODUCT=$(NS_PRODUCT) VERSION=$(SERVER_VERSION) \
       NS_USE_NATIVE=$(NS_USE_NATIVE) NSPR_BASENAME=$(NSPR_BASENAME)

NSPR_DEFINES = -DSERVER_BUILD

# ------------------------ Product-specific flags ------------------------


ifeq ($(PRODUCT), "Fedora Servers Common Installer")
MCC_SERVER=
endif

ifeq ($(PRODUCT), "Fedora Administration Server")
ifeq ($(SECURITY), export)
MCC_SERVER=-DMCC_HTTPD -DMCC_ADMSERV -DNS_EXPORT -DNET_SSL -DCLIENT_AUTH $(NSPR_DEFINES)
endif
ifeq ($(SECURITY), domestic)
MCC_SERVER=-DMCC_HTTPD -DMCC_ADMSERV -DNS_DOMESTIC -DNET_SSL -DCLIENT_AUTH $(NSPR_DEFINES)
endif
ifeq ($(SECURITY), none)
MCC_SERVER=-DMCC_HTTPD -DMCC_ADMSERV -DNS_UNSECURE $(NSPR_DEFINES)
endif
endif

ifeq ($(BUILD_MODULE), SETUP_ADMIN)
MCC_SERVER=-DMCC_ADMSERV
NS_PRODUCT = MCC_ADMSERV
endif

ifeq ($(BUILD_MODULE), UNINST_SHELL)
MCC_SERVER=-DMCC_ADMSERV
endif

ifeq ($(BUILD_MODULE), UNINST_ADMIN)
MCC_SERVER=-DMCC_HTTPD -DMCC_ADMSERV
endif


MCC_SERVER += -DSPAPI20 -DBUILD_NUM=$(GET_BUILD_NUM)

# ----------- Default Flags, may be overridden below ------------

CPPCMD=$(CC) -E
HTTPDSO_NAME=ns-httpd
MATHLIB=-lm
LIB_SUFFIX=a
AR=ar rcv $@
DLL_PRESUF=
DLL_SUFFIX=so
LDAP_DLL_PRESUF=
LDAP_DLL_SUFFIX=$(DLL_SUFFIX)
LINK_DLL=ld $(DLL_LDFLAGS) -o $@
LINK_PLUGIN=$(LINK_DLL)

# ---------------------- OS-specific compile flags -----------------------


# Used by peer snmp defines below
DEVROOT = $(MCOM_ROOT)/components/peer

#-------------------
ifeq ($(ARCH), AIX)
#-------------------
DLL_PRESUF=
DLL_SUFFIX=so
#LDAP_DLL_PRESUF=_shr
LDAP_DLL_PRESUF=
LDAP_DLL_SUFFIX=so
ifeq ($(NSOS_RELEASE), 4.3)
AIX4_3=-D AIX4_3
endif
ifeq ($(USE_64), 1)
OBJECT_MODE=64 
export OBJECT_MODE 
endif
CC=xlC_r -qarch=com -DAIX -DAIXV3 -DAIXV4 -DSYSV $(AIX4_3)
CXX=$(CC)
CCC=$(CC)
CPPCMD=/usr/ccs/lib/cpp -P
ARCH_DEBUG=-g
ARCH_OPT=-O
RANLIB=ranlib
NONSHARED_FLAG=-bnso -bI:/lib/syscalls.exp
EXPORT_FILE=$*.exp
DEF_LIBPATH=/usr/lib/threads:/usr/lpp/xlC/lib:/usr/lib:/lib
MKSHLIB_FLAGS=-p 0 -blibpath:$(DEF_LIBPATH)
DLL_LDFLAGS=-bM:SRE -bnoentry -blibpath:$(DEF_LIBPATH)
# AIX plugins can have unresolved references
LINK_PLUGIN+=-berok
LINK_DLL=$(ABS_ROOT)/build/aixmkshlib -o $@ $(MKSHLIB_FLAGS)
MKSHLIB=$(LINK_DLL)
EXTRA_LIBS=-ldl

VERITY_ARCH=_rs6k41
NSAPI_CAPABLE=true
USE_PTHREADS=0
DL_LIB =-lsvld
NSPR_DEFINES += -DNSPR20
RWTOOLS_VERSION = latest
HTTPDSO_NAME=libnshttpd
PEER_ARCH=aix

else
#-------------------
ifeq ($(ARCH), HPUX)
#-------------------
HPUX_ARCH = $(shell uname -m)
ifeq ($(HPUX_ARCH), ia64)
    DLL_SUFFIX=so
else
    DLL_SUFFIX=sl
endif
LDAP_DLL_SUFFIX=$(DLL_SUFFIX)

ifeq ($(NSOS_RELEASE), B.11.00)
	MODERNHP=1
endif
ifeq ($(NSOS_RELEASE), B.11.11)
	MODERNHP=1
endif
ifeq ($(NSOS_RELEASE), B.11.23)
	MODERNHP=1
endif

ifdef MODERNHP
CC=cc -Ae +ESlit
CXX=aCC
# ARCH_CFLAGS=-AA # if set on PA_RISC, build fails by <iostream> not found
ifeq ($(NSOS_RELEASE), B.11.23)
  ARCH_CFLAGS=-AA # the default on ipf, but also use it elsewhere
  ifeq ($(USE_64), 1)
    BIT_SWITCH=+DD64
  else
    BIT_SWITCH=+DD32
  endif
else # 11.11 or earlier
  ARCH_CFLAGS=
  ifeq ($(USE_64), 1)
    BIT_SWITCH=+DA2.0W +DS2.0
  else
    BIT_SWITCH=+DAportable +DS1.1
  endif
endif # B.11.23
ARCH_CFLAGS+=$(BIT_SWITCH) -D_HPUX_SOURCE -D_POSIX_C_SOURCE=199506L -D__STDC_EXT__ +z -DHPUX11 
EXTRA_LIBS= -ldld -lm -lpthread
else
CC=cc
ARCH_CFLAGS=-D_HPUX_SOURCE -Aa +DA1.0 +z
EXTRA_LIBS= -ldld -lm -ldce
endif

ifeq ($(USE_64), 1)
  ARCH_CFLAGS+=-DHPUX_64
endif

CCC=$(CXX)

ARCH_DEBUG=-g
ARCH_OPT=-O

RANLIB=true
NONSHARED_FLAG=-Wl,-a,archive

#
# Flags passed to CC to pass onto the linker; separate it from EXTRA_LIBS
LD_CFLAGS=-Wl,-E $(BIT_SWITCH)
DLL_CFLAGS=+z
DLL_LDFLAGS=-b $(BIT_SWITCH)
MODULE_CFLAGS += $(BIT_SWITCH)
NSAPI_CAPABLE=true
VERITY_ARCH=_hp800
USE_PTHREADS=0
NSPR_DEFINES += -DNSPR20 -D_PR_NTHREAD -D_PR_USECPU -D_REENTRANT
LINK_DLL=$(CCC) $(DLL_LDFLAGS) -o $@
HTTPDSO_NAME=libnshttpd
PEER_ARCH=hp
RWTOOLS_VERSION = latest

else
#-------------------
ifeq ($(ARCH), Linux)
#-------------------
NS_USE_GCC	= 1
CC=gcc
CCC=g++
ARCH_DEBUG=-g
ARCH_OPT=-O2
ARCH_CFLAGS=-fPIC -pipe -DLINUX -Dlinux -DBSD -D_POSIX_SOURCE -D_XOPEN_SOURCE -D_BSD_SOURCE -DHAVE_STRERROR -DNO_DBM
USE_PTHREADS=1
NSPR_DEFINES += -DNSPR20 -Di386
RANLIB=ranlib
BUILD_IIOPLIB=no
ifeq ($(USE_PTHREADS), 1)
  NSPR_DEFINES         += -D_REENTRANT -D_PR_NEED_FAKE_POLL
endif
LINK_DLL=$(CC) -shared -Wl,-soname -Wl,$(@:$(OBJDIR)/%.so=%.so)
EXTRA_LIBS=-lpthread -lc -lresolv -ldl
PEER_ARCH=linux

else
#-------------------
ifeq ($(ARCH), OSF1)
#-------------------
CC=cc
CCC=cxx
CXX=cxx
ARCH_DEBUG=-g
ARCH_OPT=-O2
ifeq ($(NSOS_OSF1REV),878)
  ARCH_CFLAGS=-DOSF1V4D -DIS_64 -ieee_with_inexact -pthread
else
  ARCH_CFLAGS=-DIS_64 -ieee_with_inexact -pthread
endif
# ranlib no longer needed on OSF1 V4.0
RANLIB=true
DLL_LDFLAGS=-shared -all -expect_unresolved "*"
NONSHARED_FLAG=-non_shared
NSAPI_CAPABLE=true
VERITY_ARCH=_aosf40
EXTRA_LIBS=
USE_PTHREADS=0
NSPR_DEFINES += -DNSPR20
RWTOOLS_VERSION = latest
PEER_ARCH=osf
PEERDIR=$(DEVROOT)/osf/dev
ADM_RELDATE=19970605

export NO_DB2=1
export NO_ODBC=1

else
#----------------------
ifeq ($(ARCH), SOLARIS)
#----------------------

#
# 7/12/96 Adrian - Switch to use SparcWorks for 3.0 Development
#		   Therefore no need to separate from catalog server
#
OSVERSION := $(shell uname -r | sed "y/./0/")
NS_USE_NATIVE=1
CC=cc
CXX=CC
CCC=$(CXX)
ARCH_OPT=-xO2
ifeq ($(USE_64), 1)
BIT_SWITCH=-xarch=v9
CXX+= $(BIT_SWITCH)
endif
ARCH_CFLAGS=$(BIT_SWITCH) -KPIC -DSVR4 -D__svr4 -D__svr4__ -D_SVID_GETTOD -DOSVERSION=$(OSVERSION)
ARCH_DEBUG=-g
RANLIB=true

EXTRA_LIBS = -lthread -lposix4 -lsocket -lnsl -ldl -lresolv -lgen

ifeq ($(USE_FORTE6), 1)
EXTRA_LIBS := $(EXTRA_LIBS) -lCrun -lCstd
endif

DLL_LDFLAGS=-G
NONSHARED_FLAG=-static
NSAPI_CAPABLE=true
NLIST=-lelf
VERITY_ARCH=_solaris
NSPR_DEFINES += -DNSPR20 -D_PR_NTHREAD -D_REENTRANT
PEER_ARCH=solaris

BUILD_NMC=true

else
#-------------------------
ifeq ($(ARCH), SOLARISx86)
#-------------------------
OSVERSION := $(shell uname -r | sed "y/./0/")
NS_USE_NATIVE=1
# Add SOLARIS define as well as SOLARISx86 which happens automagically
CC=cc -DSOLARIS
CXX=CC -DSOLARIS
CCC=$(CXX)
ARCH_OPT=-xO2
ARCH_CFLAGS=-DSVR4 -D__svr4 -D__svr4__ -D_SVID_GETTOD -DOSVERSION=$(OSVERSION)
ARCH_DEBUG=-g
RANLIB=true

EXTRA_LIBS = -lsocket -lthread -lposix4 -lnsl -ldl -lresolv -lm

DLL_LDFLAGS=-G
LDAP_VERSION    = oem
RWTOOLS_VERSION = latest
ADM_RELDATE     = oem
AUTOCATALOG_VERSION = oem
NONSHARED_FLAG=-static
NSAPI_CAPABLE=true
NLIST=-lelf
NO_MSOFT_OBJ=1
BUILD_WEBPUB=no
BUILD_IIOPLIB=no
NSPR_DEFINES += -DNSPR20 -D_PR_NTHREAD -D_REENTRANT
export NO_INFORMIX=1
export NO_ORACLE=1
export NO_SYBASE=1
export NO_ODBC=1
export NO_DB2=1
PEER_ARCH=solarisx86

else
#-------------------
ifeq ($(ARCH), WINNT)
#-------------------
LIB_SUFFIX=lib
DLL_SUFFIX=dll
LDAP_DLL_SUFFIX=$(DLL_SUFFIX)
PROCESSOR := $(shell uname -p)
PEER_ARCH=nti31
ifeq ($(PROCESSOR), I386)
CPU_ARCH = x386
VERITY_ARCH=_nti31
ARCH_OPT=-DNDEBUG -O2
ifeq ($(DEBUG), fulld)
CC=cl -nologo -MDd -W3 -GT -GX -D_X86_ -Dx386 -DWIN32 -D_WINDOWS -D_RWTOOLSDLL
CCP=cl -nologo -MDd -W3 -GT -GX -D_X86_ -Dx386 -DWIN32 -D_WINDOWS -D_MBCS -D_AFXDLL -D_RWTOOLSDLL
else
CC=cl -nologo -MD -W3 -GT -GX -D_X86_ -Dx386 -DWIN32 -D_WINDOWS -D_RWTOOLSDLL
CCP=cl -nologo -MD -W3 -GT -GX -D_X86_ -Dx386 -DWIN32 -D_WINDOWS -D_MBCS -D_AFXDLL -D_RWTOOLSDLL
endif
BUILD_NMC=true

else
ifeq ($(PROCESSOR), MIPS)
CPU_ARCH = MIPS
VERITY_ARCH=verity_is_undefined
ARCH_OPT=-DNDEBUG -O2
CC=cl -nologo -MD -W3 -GT -GX -D_MIPS_ -DWIN32 -D_WINDOWS -D_RWTOOLSDLL
CCP=cl -nologo -MD -W3 -GT -GX -D_X86_ -Dx386 -DWIN32 -D_WINDOWS -D_MBCS -D_AFXDLL -D_RWTOOLSDLL
else
ifeq ($(PROCESSOR), ALPHA)
CPU_ARCH = ALPHA
VERITY_ARCH=_ant35
ARCH_OPT=-DNDEBUG -Od
CC=cl -nologo -MD -W3 -GT -GX -D_ALPHA=1 -DWIN32 -D_WINDOWS -D_RWTOOLSDLL
CCP=cl -nologo -MD -W3 -GT -GX -D_ALPHA_ -DWIN32 -D_WINDOWS -D_MBCS -D_AFXDLL -D_RWTOOLSDLL
BUILD_IIOPLIB=no
export NO_SYBASE=1
export NO_ODBC=1
export NO_DB2=1
else
CPU_ARCH = processor_is_undefined
endif
endif
endif

RC=rc $(MCC_SERVER)
MC=mc
ARCH_DEBUG=-D_DEBUG -Od	-Z7 -Dmiodrag
ARCH_CFLAGS=
ARCH_LINK_DEBUG=-DEBUG
ARCH_LINK_OPT=
RANLIB=echo

EXTRA_LIBS=wsock32.lib kernel32.lib user32.lib gdi32.lib winspool.lib \
           comdlg32.lib advapi32.lib shell32.lib ole32.lib oleaut32.lib \
           rpcrt4.lib uuid.lib odbc32.lib odbccp32.lib winmm.lib

DLL_LDFLAGS=
NONSHARED_FLAG=
NSAPI_CAPABLE=true
NO_MSOFT_OBJ=1
NSPR_DEFINES += -DNSPR20 -D_PR_NTHREAD -D_PR_USECPU

else

CC=UNKNOWN_SYSTEM_TYPE

endif
endif
endif
endif
endif
endif
endif

# Peer SNMP defines
ifeq ($(BUILD_SNMP), yes)
MCC_SERVER += -DPEER_SNMP
SNMP=libsnmp
SNMP_INCLUDE=$(DEVROOT)/include
PEERLIB=$(DEVROOT)/$(PEER_ARCH)/dev
SNMPNOLIB=snmp
ifeq ($(ARCH), WINNT)
PEERLIBOBJ=$(PEERLIB)/mgmt.lib
else
PEERLIBOBJ=$(PEERLIB)/mgmt.o
endif
else
SNMP=
SNMP_INCLUDE=.
PEERLIB=
PEERLIBOBJ=
SNMPNOLIB=
endif

ifeq ($(NSOS_ARCH),WINNT)
  ifneq ($(PROCESSOR_ARCHITECTURE),x86)
    NSOBJDIR_NAME     = $(NSCONFIG)$(PROCESSOR_ARCHITECTURE)$(NSOBJDIR_TAG).OBJ
    NSOBJDIR_NAME_32     = $(NSCONFIG)$(PROCESSOR_ARCHITECTURE)$(NSOBJDIR_TAG).OBJ
  else
    NSOBJDIR_NAME_32     = $(subst $(NS64TAG),,$(NSOBJDIR_NAME))
  endif
  NSOBJDIR_NAME1    = $(NSOBJDIR_NAME)
else
  NSOBJDIR_NAME_32  = $(subst $(NS64TAG),,$(NSOBJDIR_NAME))
  NSOBJDIR_NAME1    = $(NSOBJDIR_NAME)
endif
# platforms without full rtl debugging versions of libraries i.e. not NT
NSCP_DIST                         := $(ABS_ROOT_PARENT)/dist
NSCP_DISTDIR          := $(NSCP_DIST)/$(NSOBJDIR_NAME)
FULL_RTL_OBJDIR           := $(NSOBJDIR_NAME)
NSCP_DISTDIR_FULL_RTL := $(NSCP_DISTDIR)

ifeq ($(ARCH), WINNT)
	ifeq ($(DEBUG), fulld)
		NSCP_DISTDIR_FULL_RTL = $(BUILD_ROOT)/../dist/$(FULL_RTL_OBJDIR)
	endif
endif

$(NSCP_DISTDIR_FULL_RTL): 
	echo depending upon $(NSCP_DISTDIR_FULL_RTL) 
	mkdir -p $@

include $(ABS_ROOT)/components.mk

# ------------------------ The actual build rules ------------------------


ifeq ($(DEBUG), full)
MCC_DEBUG=-DMCC_DEBUG $(ARCH_DEBUG)
NSOBJDIR_TAG=_DBG
ML_DEBUG=$(ARCH_LINK_DEBUG)

else
ifeq ($(DEBUG), fulld)
MCC_DEBUG=-DMCC_DEBUG $(ARCH_DEBUG)
NSOBJDIR_TAG=_DBG
ML_DEBUG=$(ARCH_LINK_DEBUG)

else
ifeq ($(DEBUG), purify)

MCC_DEBUG=$(ARCH_DEBUG)
NSOBJDIR_TAG=_DBG
ML_DEBUG=$(ARCH_LINK_DEBUG)
PUREOPTS=-follow-child-processes=true
PURIFY=purify $(PUREOPTS)

else
MCC_DEBUG=$(ARCH_OPT)
NSOBJDIR_TAG=_OPT
ML_DEBUG=$(ARCH_LINK_OPT)
BUILDOPT = BUILD_OPT=1
endif
endif
endif

ifeq ($(PRODUCT), "Fedora Administration Server")
ADM_VERSION =
else
ifndef ADM_RELDATE
ADM_RELDATE = 19970811
endif
ifeq ($(SECURITY), domestic)
ADM_VERSION = $(ADM_RELDATE)D
else
ADM_VERSION = $(ADM_RELDATE)E
endif
endif

RM=rm
DEPEND=makedepend
STRIP=strip

# AS INCLUDE DEFDINED HERE
MCC_INCLUDE=-I$(ABS_ROOT)/include \
            -I$(NSCP_DISTDIR)/include \
            -I$(LDAP_INCLUDE)

ifeq ($(ARCH), WINNT)
XP_FLAG=-DXP_WIN32  -DXP_WIN -D_WINDOWS -DXP_PC -DXP_WINNT
else
XP_FLAG=-DXP_UNIX
endif
# CFLAGS_NO_ARCH is needed temporarily by NS_MAIL
CFLAGS_NO_ARCH=$(NODE_FLAG) $(BOMB) $(MODULE_CFLAGS) $(XP_FLAG) \
       -D$(ARCH) $(MCC_DEBUG) $(MCC_SERVER)
CFLAGS=$(ARCH_CFLAGS) $(CFLAGS_NO_ARCH)
ifeq ($(BUILD_DLL), yes)
CFLAGS += -DBUILD_DLL
endif
LFLAGS=$(ML_DEBUG)

ifeq ($(BSCINFO), yes)
CBSCFLAGS=-FR$(OBJDEST)/$*.sbr
endif

include $(ABS_ROOT)/nscore.mk

# These are the defaults, individual Makefiles can override them as necessary
FVERSION=$(MAJOR_VERSION).$(MINOR_VERSION)
FVERSION_FLAGS=-v$(FVERSION)

ifeq ($(ARCH), WINNT)
$(OBJDEST)/%.res: %.rc 
	$(RC) -Fo$@ $*.rc
endif

ifndef NOSTDCOMPILE
$(OBJDEST)/%.o: %.cpp
ifeq ($(ARCH), WINNT)
	$(CCP) -c $(CFLAGS) $(MCC_INCLUDE) $< -Fo$(OBJDEST)/$*.o
else
	$(CCC) -c $(CFLAGS) $(MCC_INCLUDE) $< -o $(OBJDEST)/$*.o
endif
endif

ifndef NOSTDCOMPILE
$(OBJDEST)/%.o: %.c
ifeq ($(ARCH), WINNT)
	$(CC) -c $(CFLAGS) $(MCC_INCLUDE) $< -Fo$(OBJDEST)/$*.o $(CBSCFLAGS)
else
	$(CC) -c $(CFLAGS) $(MCC_INCLUDE) $< -o $(OBJDEST)/$*.o
endif
endif

ifeq ($(ARCH), WINNT)
AR=lib /nologo /out:"$@"
LINK_SUBSYSTEM  = windows
LINK_DLL	= link /nologo /MAP /DLL /PDB:NONE $(ML_DEBUG) /SUBSYSTEM:WINDOWS $(LLFLAGS) $(DLL_LDFLAGS) /out:"$@"
LINK_EXE	= link -OUT:"$@" /MAP  $(ARCH_LINK_DEBUG) $(LCFLAGS) /NOLOGO /PDB:NONE /INCREMENTAL:NO \
                  /SUBSYSTEM:$(LINK_SUBSYSTEM) $(OBJS) $(DEPLIBS) $(EXTRA_LIBS)
BSCMAKE = bscmake.exe /nologo /o $@
endif # WINNT

ifndef NOSTDCLEAN
clean:
	$(RM) -f .depends.$(NSOBJDIR_NAME)  $(LIBS) $(OBJDEST)/*.o *_pure_* $(BINS) $(PUREFILES)
endif

ifndef NOSTDDEPEND
ifeq ($(ARCH), WINNT)
INCLUDE_DEPENDS = $(NULL)
depend:
	echo making depends
else
INCLUDE_DEPENDS = .depends.$(NSOBJDIR_NAME)
.depends.$(NSOBJDIR_NAME):
	touch .depends.$(NSOBJDIR_NAME)

depend:
	$(DEPEND) -f.depends.$(NSOBJDIR_NAME) -- $(MCC_INCLUDE) $(CFLAGS) *.c *.cpp
endif
endif

ifndef NOSTDSTRIP
strip:
	$(STRIP) $(BINS)
endif


